 /*********************************************************************/
 /*                                                                   */
 /* msi.cpy - - Interface for external access to Installer Service    */
 /*                                                                   */
 /* Version 1.0                                                       */
 /*                                                                   */
 /* NOTES:  All buffers sizes are TCHAR count, null included only on  */
 /*         input                                                     */
 /*         Return argument pointers may be null if not interested    */
 /*         in value                                                  */
 /*                                                                   */
 /*                                                                   */
 /* Copyright (c) 1999, Microsoft Corp.      All rights reserved.     */
 /*                                                                   */
 /*********************************************************************/

 %dcl #MSI_H_ char ext;
 %if #MSI_H_ = '' %then %do;
 %#MSI_H_ = 'Y';

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include windef;
 %end;

 %dcl UNICODE char ext;

 /* ----------------------------------------------------------------- */
 /* Installer generic handle definitions                              */
 /* ----------------------------------------------------------------- */

 define alias MSIHANDLE fixed bin(32) unsigned;
    /* abstract generic handle, 0 == no handle */


 /* Close a open handle of any type                                   */
 /* All handles obtained from API calls must be closed when no longer */
 /* needed. Normally succeeds, returning TRUE.                        */

 dcl MsiCloseHandle
     entry (type MSIHANDLE )
     returns (type UINT byvalue optional)
     WINAPI external('MsiCloseHandle');

 /* Close all handles open in the process, a diagnostic call          */
 /* This should NOT be used as a cleanup mechanism -- use PMSIHANDLE  */
 /* class (PL/I Note; for C++, not yet translated for PL/I!!!         */
 /* Can be called at termination to assure that all handles have been */
 /* closed. Returns 0 if all handles have been close, else number of  */
 /* open handles                                                      */

 dcl MsiCloseAllHandles
     entry ()
     returns (type UINT byvalue optional)
     WINAPI external('MsiCloseAllHandles');


 /* Install message type for callback is a combination                */
 /* of the following:                                                 */
 /*  A message box style:      MB_*, where MB_OK is the default       */
 /*  A message box icon type:  MB_ICON*, where no icon is the default */
 /*  A default button:         MB_DEFBUTTON?, where MB_DEFBUTTON1 is  */
 /*                            the default                            */
 /*  One of the following install message types, no default           */

 /* PL/I Note:Original C is enum, makes no sense in PL/I */
 define alias INSTALLMESSAGE         type DWORD;
 dcl ( INSTALLMESSAGE_FATALEXIT      value('00000000'xu),
          /* premature termination, possibly fatal OOM */
       INSTALLMESSAGE_ERROR          value('01000000'xu),
          /* formatted error message */
       INSTALLMESSAGE_WARNING        value('02000000'xu),
          /* formatted warning message */
       INSTALLMESSAGE_USER           value('03000000'xu),
          /* user request message */
       INSTALLMESSAGE_INFO           value('04000000'xu),
          /* informative message for log */
       INSTALLMESSAGE_FILESINUSE     value('05000000'xu),
          /* list of files in use that need to be replaced */
       INSTALLMESSAGE_RESOLVESOURCE  value('06000000'xu),
          /* request to determine a valid source location */
       INSTALLMESSAGE_OUTOFDISKSPACE value('07000000'xu),
          /* insufficient disk space message */
       INSTALLMESSAGE_ACTIONSTART    value('08000000'xu),
          /* start of action: action name & description */
       INSTALLMESSAGE_ACTIONDATA     value('09000000'xu),
          /* formatted data associated with individual action item */
       INSTALLMESSAGE_PROGRESS       value('0A000000'xu),
          /* progress gauge info: units so far, total */
       INSTALLMESSAGE_COMMONDATA     value('0B000000'xu),
          /* product info for dialog: language Id, dialog caption */
       INSTALLMESSAGE_INITIALIZE     value('0C000000'xu),
          /* sent prior to UI initialization, no string data */
       INSTALLMESSAGE_TERMINATE      value('0D000000'xu),
          /* sent after UI termination, no string data */
       INSTALLMESSAGE_SHOWDIALOG     value('0E000000'xu)
          /* sent prior to display or authored dialog or wizard */
     ) type INSTALLMESSAGE;

 define alias INSTALLUI_HANDLERA
              entry (type LPVOID,
                     type UINT,
                     type LPCSTR)
              returns (fixed bin(31) byvalue optional)
              limited WINAPI;

 define alias INSTALLUI_HANDLERW
              entry (type LPVOID,
                     type UINT,
                     type LPCWSTR)
              returns (fixed bin(31) byvalue optional)
              limited WINAPI;

 %if UNICODE ^= '' %then %do;
 define alias INSTALLUI_HANDLER type INSTALLUI_HANDLERW;
 %end; %else %do;
 define alias INSTALLUI_HANDLER type INSTALLUI_HANDLERA;
 %end; /* !UNICODE */

 /* PL/I Note:Original C is enum, makes no sense in PL/I */
 define alias INSTALLUILEVEL fixed bin(31);
 dcl ( INSTALLUILEVEL_NOCHANGE     value(0),
          /* UI level is unchanged */
       INSTALLUILEVEL_DEFAULT      value(1),
          /* default UI is used */
       INSTALLUILEVEL_NONE         value(2),
          /* completely silent installation */
       INSTALLUILEVEL_BASIC        value(3),
          /* simple progress and error handling */
       INSTALLUILEVEL_REDUCED      value(4),
          /* authored UI, wizard dialogs suppressed */
       INSTALLUILEVEL_FULL         value(5),
          /* authored UI with wizards, progress, errors */
       INSTALLUILEVEL_ENDDIALOG    value('80'xn),
          /* display success/failure dialog at end of install */
       INSTALLUILEVEL_PROGRESSONLY value('40'xn)
          /* display only progress dialog */
    ) type INSTALLUILEVEL;

 define ordinal INSTALLSTATE
   ( INSTALLSTATE_NOTUSED      value(-7),
        /* component disabled */
     INSTALLSTATE_BADCONFIG    value(-6),
        /* configuration data corrupt */
     INSTALLSTATE_INCOMPLETE   value(-5),
        /* installation suspended or in progress */
     INSTALLSTATE_SOURCEABSENT value(-4),
        /* run from source, source is unavailable */
     INSTALLSTATE_MOREDATA     value(-3),
        /* return buffer overflow */
     INSTALLSTATE_INVALIDARG   value(-2),
        /* invalid function argument */
     INSTALLSTATE_UNKNOWN      value(-1),
        /* unrecognized product or feature */
     INSTALLSTATE_BROKEN       value(0),
        /* broken */
     INSTALLSTATE_ADVERTISED   value(1),
        /* advertised feature */
     INSTALLSTATE_ABSENT       value(2),
        /* uninstalled (or action state absent but clients remain) */
     INSTALLSTATE_LOCAL        value(3),
        /* installed on local drive */
     INSTALLSTATE_SOURCE       value(4),
        /* run from source, CD or net */
     INSTALLSTATE_DEFAULT      value(5)
        /* use default, local or source */
   ) prec(31);

 dcl INSTALLSTATE_REMOVED type INSTALLSTATE
                               value(INSTALLSTATE_ADVERTISED);
        /* component being removed (action state, not settable) */

 define ordinal USERINFOSTATE
   ( USERINFOSTATE_MOREDATA   value(-3),/* return buffer overflow */
     USERINFOSTATE_INVALIDARG value(-2),/* invalid function argument */
     USERINFOSTATE_UNKNOWN    value(-1),/* unrecognized product */
     USERINFOSTATE_ABSENT     value( 0),
        /* user info and PID not initialized */
     USERINFOSTATE_PRESENT    value( 1)
        /* user info and PID initialized */
   ) prec(31);

 define ordinal INSTALLLEVEL
   ( INSTALLLEVEL_DEFAULT value(0),/* install authored default */
     INSTALLLEVEL_MINIMUM value(1),/* install only required features */
     INSTALLLEVEL_MAXIMUM value('FFFF'xn)/* install all features */
   ) prec(31); /* intermediate levels dependent on authoring */


 /* PL/I Note:Original C is enum, makes no sense in PL/I */
 define alias REINSTALLMODE fixed bin(32) unsigned;
 dcl ( REINSTALLMODE_REPAIR           value('00000001'xu),
          /* Reserved bit - currently ignored */
       REINSTALLMODE_FILEMISSING      value('00000002'xu),
          /* Reinstall only if file is missing */
       REINSTALLMODE_FILEOLDERVERSION value('00000004'xu),
          /* Reinstall if file is missing, or older version */
       REINSTALLMODE_FILEEQUALVERSION value('00000008'xu),
          /* Reinstall if file is missing, or equal or older version */
       REINSTALLMODE_FILEEXACT        value('00000010'xu),
          /* Reinstall if file is missing, or not exact version */
       REINSTALLMODE_FILEVERIFY       value('00000020'xu),
          /* checksum executables, reinstall if missing or corrupt */
       REINSTALLMODE_FILEREPLACE      value('00000040'xu),
          /* Reinstall all files, regardless of version */
       REINSTALLMODE_MACHINEDATA      value('00000080'xu),
          /* insure required machine reg entries */
       REINSTALLMODE_USERDATA         value('00000100'xu),
          /* insure required user reg entries */
       REINSTALLMODE_SHORTCUT         value('00000200'xu),
          /* validate shortcuts items */
       REINSTALLMODE_PACKAGE          value('00000400'xu)
          /* use re-cache source install package */
     ) type REINSTALLMODE;

 /* PL/I Note:Original C is enum, makes no sense in PL/I */
 define alias INSTALLLOGMODE type DWORD;
 dcl ( INSTALLLOGMODE_FATALEXIT
          value(isll (1, isrl (INSTALLMESSAGE_FATALEXIT, 24))),
       INSTALLLOGMODE_ERROR
          value(isll (1, isrl (INSTALLMESSAGE_ERROR, 24))),
       INSTALLLOGMODE_WARNING
          value(isll (1, isrl (INSTALLMESSAGE_WARNING, 24))),
       INSTALLLOGMODE_USER
          value(isll (1, isrl (INSTALLMESSAGE_USER, 24))),
       INSTALLLOGMODE_INFO
          value(isll (1, isrl (INSTALLMESSAGE_INFO, 24))),
       INSTALLLOGMODE_RESOLVESOURCE
          value(isll (1, isrl (INSTALLMESSAGE_RESOLVESOURCE, 24))),
       INSTALLLOGMODE_OUTOFDISKSPACE
          value(isll (1, isrl (INSTALLMESSAGE_OUTOFDISKSPACE, 24))),
       INSTALLLOGMODE_ACTIONSTART
          value(isll (1, isrl (INSTALLMESSAGE_ACTIONSTART, 24))),
       INSTALLLOGMODE_ACTIONDATA
          value(isll (1, isrl (INSTALLMESSAGE_ACTIONDATA, 24))),
       INSTALLLOGMODE_COMMONDATA
          value(isll (1, isrl (INSTALLMESSAGE_COMMONDATA, 24))),
       INSTALLLOGMODE_PROPERTYDUMP           /* log only */
          value(isll (1, isrl (INSTALLMESSAGE_PROGRESS, 24))),
       INSTALLLOGMODE_VERBOSE                /* log only */
          value(isll (1, isrl (INSTALLMESSAGE_INITIALIZE, 24))),
       INSTALLLOGMODE_PROGRESS               /* external handler only */
          value(isll (1, isrl (INSTALLMESSAGE_PROGRESS, 24))),
       INSTALLLOGMODE_INITIALIZE             /* external handler only */
          value(isll (1, isrl (INSTALLMESSAGE_INITIALIZE, 24))),
       INSTALLLOGMODE_TERMINATE              /* external handler only */
          value(isll (1, isrl (INSTALLMESSAGE_TERMINATE, 24))),
       INSTALLLOGMODE_SHOWDIALOG             /* external handler only */
          value(isll (1, isrl (INSTALLMESSAGE_SHOWDIALOG, 24)))
     ) type INSTALLLOGMODE;

 define alias INSTALLLOGATTRIBUTES fixed bin(32) unsigned;
 dcl ( INSTALLLOGATTRIBUTES_APPEND        value(raise2 (1, 0)),
       INSTALLLOGATTRIBUTES_FLUSHEACHLINE value(raise2 (1, 1))
     ) type INSTALLLOGATTRIBUTES;

 define alias INSTALLFEATUREATTRIBUTE type DWORD;
 dcl ( INSTALLFEATUREATTRIBUTE_FAVORLOCAL
          value(raise2 (1, 0)),
       INSTALLFEATUREATTRIBUTE_FAVORSOURCE
          value(raise2 (1, 1)),
       INSTALLFEATUREATTRIBUTE_FOLLOWPARENT
          value(raise2 (1, 2)),
       INSTALLFEATUREATTRIBUTE_FAVORADVERTISE
          value(raise2 (1, 3)),
       INSTALLFEATUREATTRIBUTE_DISALLOWADVERTISE
          value(raise2 (1, 4)),
       INSTALLFEATUREATTRIBUTE_NOUNSUPPORTEDADVERTISE
          value(raise2 (1,5))
     ) type INSTALLFEATUREATTRIBUTE;

 define ordinal INSTALLMODE
   ( INSTALLMODE_NOSOURCERESOLUTION value(-3),
          /* skip source resolution */
     INSTALLMODE_NODETECTION        value(-2),
          /* skip detection */
     INSTALLMODE_EXISTING           value(-1),
          /* provide, if available */
     INSTALLMODE_DEFAULT            value( 0)
          /* install, if absent */
   ) prec(31);


 dcl MAX_FEATURE_CHARS fixed bin(31) value(38);
 /* maximum chars in feature name (same as string GUID) */


 /* Product info attributes: advertised information */

 dcl INSTALLPROPERTY_TRANSFORMS       TCHAR value('Transforms');
 dcl INSTALLPROPERTY_LANGUAGE         TCHAR value('Language');
 dcl INSTALLPROPERTY_PRODUCTNAME      TCHAR value('ProductName');
 dcl INSTALLPROPERTY_ASSIGNMENTTYPE   TCHAR value('AssignmentType');
 dcl INSTALLPROPERTY_ADVTFLAGS        TCHAR value('AdvertiseFlags');
 dcl INSTALLPROPERTY_VOLUMEPREFERENCE TCHAR value('VolumePref');
 dcl INSTALLPROPERTY_PACKAGECODE      TCHAR value('PackageCode');
 dcl INSTALLPROPERTY_UPGRADECODE      TCHAR value('UpgradeCode');
 dcl INSTALLPROPERTY_VERSION          TCHAR value('Version');

 /* Product info attributes: installed information */

 dcl INSTALLPROPERTY_INSTALLEDPRODUCTNAME TCHAR
        value('InstalledProductName');
 dcl INSTALLPROPERTY_VERSIONSTRING    TCHAR value('VersionString');
 dcl INSTALLPROPERTY_HELPLINK         TCHAR value('HelpLink');
 dcl INSTALLPROPERTY_HELPTELEPHONE    TCHAR value('HelpTelephone');
 dcl INSTALLPROPERTY_INSTALLLOCATION  TCHAR value('InstallLocation');
 dcl INSTALLPROPERTY_INSTALLSOURCE    TCHAR value('InstallSource');
 dcl INSTALLPROPERTY_INSTALLDATE      TCHAR value('InstallDate');
 dcl INSTALLPROPERTY_PUBLISHER        TCHAR value('Publisher');
 dcl INSTALLPROPERTY_LOCALPACKAGE     TCHAR value('LocalPackage');
 dcl INSTALLPROPERTY_URLINFOABOUT     TCHAR value('URLInfoAbout');
 dcl INSTALLPROPERTY_URLUPDATEINFO    TCHAR value('URLUpdateInfo');
 dcl INSTALLPROPERTY_VERSIONMINOR     TCHAR value('VersionMinor');
 dcl INSTALLPROPERTY_VERSIONMAJOR     TCHAR value('VersionMajor');


 define ordinal INSTALLTYPE
   ( INSTALLTYPE_DEFAULT       value(0),
        /* set to indicate default behavior */
     INSTALLTYPE_NETWORK_IMAGE value(1)
        /* set to indicate network install */
   ) prec(31);


 /* ----------------------------------------------------------------- */
 /* Functions to set the UI handling and logging. The UI will be used */
 /* for error, progress, and log messages for all subsequent calls to */
 /* Installer Service API functions that require UI.                  */
 /* ----------------------------------------------------------------- */

 /* Enable internal UI */

 dcl MsiSetInternalUI
     entry (type INSTALLUILEVEL,       /* UI level */
            type HWND byaddr optional) /* handle of owner window */
     returns (type INSTALLUILEVEL byvalue optional)
     WINAPI external('MsiSetInternalUI');

 /* Enable external UI handling, returns any previous handler or NULL */
 /* if none. Messages are designated with a combination of bits from  */
 /* INSTALLLOGMODE enum.                                              */

 dcl MsiSetExternalUIA
     entry (type INSTALLUI_HANDLERA,/* for progress and error handling*/
            type DWORD,   /* bit flags designating messages to handle */
            type LPVOID)  /* application context */
     returns (type INSTALLUI_HANDLERA byvalue optional)
     WINAPI external('MsiSetExternalUIA');
 dcl MsiSetExternalUIW
     entry (type INSTALLUI_HANDLERW,/* for progress and error handling*/
            type DWORD,   /* bit flags designating messages to handle */
            type LPVOID)  /* application context */
     returns (type INSTALLUI_HANDLERW byvalue optional)
     WINAPI external('MsiSetExternalUIW');

 %dcl MsiSetExternalUI char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSetExternalUI='MsiSetExternalUIW';
 %end; %else %do;
   %MsiSetExternalUI='MsiSetExternalUIA';
 %end; /* !UNICODE */


 /* Enable logging to a file for all install sessions for the client  */
 /* process, with control over which log messages are passed to the   */
 /* specified log file. Messages are designated with a combination of */
 /* bits from INSTALLLOGMODE enum.                                    */

 dcl MsiEnableLogA
     entry (type DWORD, /* bit flags designating operations to report */
            char(*) varz byaddr nonasgn optional,
                        /* log file, or NULL to disable logging */
            type DWORD) /* INSTALLLOGATTRIBUTES flags */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnableLogA');
 dcl MsiEnableLogW
     entry (type DWORD, /* bit flags designating operations to report */
            WCHAR(*) varz byaddr nonasgn optional,
                        /* log file, or NULL to disable logging */
            type DWORD) /* INSTALLLOGATTRIBUTES flags */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnableLogW');

 %dcl MsiEnableLog char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnableLog='MsiEnableLogW';
 %end; %else %do;
   %MsiEnableLog='MsiEnableLogA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Functions to query and configure a product as a whole.            */
 /* A component descriptor string may be used instead of the product  */
 /* code.                                                             */
 /* ----------------------------------------------------------------- */

 /* Return the installed state for a product */

 dcl MsiQueryProductStateA
     entry (char(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiQueryProductStateA');
 dcl MsiQueryProductStateW
     entry (WCHAR(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiQueryProductStateW');

 %dcl MsiQueryProductState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiQueryProductState='MsiQueryProductStateW';
 %end; %else %do;
   %MsiQueryProductState='MsiQueryProductStateA';
 %end; /* !UNICODE */

 /* Return product info */

 dcl MsiGetProductInfoA
     entry (char(*) varz byaddr nonasgn,
                          /* product code, string GUID, or descriptor */
            char(*) varz byaddr nonasgn,
                          /* attribute name, case-sensitive */
            char(*) varz byaddr optional,
                          /* returned value, NULL if not desired */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductInfoA');
 dcl MsiGetProductInfoW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code, string GUID, or descriptor */
            WCHAR(*) varz byaddr nonasgn,
                          /* attribute name, case-sensitive */
            WCHAR(*) varz byaddr optional,
                          /* returned value, NULL if not desired */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductInfoW');

 %dcl MsiGetProductInfo char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetProductInfo='MsiGetProductInfoW';
 %end; %else %do;
   %MsiGetProductInfo='MsiGetProductInfoA';
 %end; /* !UNICODE */

 /* Install a new product. */
 /* Either may be NULL, but the DATABASE property must be specified */

 dcl MsiInstallProductA
     entry (char(*) varz byaddr nonasgn optional,
                          /* location of package to install */
            char(*) varz byaddr nonasgn optional)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallProductA');
 dcl MsiInstallProductW
     entry (WCHAR(*) varz byaddr nonasgn optional,
                          /* location of package to install */
            WCHAR(*) varz byaddr nonasgn optional)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallProductW');

 %dcl MsiInstallProduct char ext;
 %if UNICODE ^= '' %then %do;
   %MsiInstallProduct='MsiInstallProductW';
 %end; %else %do;
   %MsiInstallProduct='MsiInstallProductA';
 %end; /* !UNICODE */

 /* Install/uninstall an advertised or installed product      */
 /* No action if installed and INSTALLSTATE_DEFAULT specified */

 dcl MsiConfigureProductA
     entry (char(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
            fixed bin(31),/* how much of the product to install */
            type INSTALLSTATE)
                          /* local/source/default/absent/lock/uncache */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureProductA');
 dcl MsiConfigureProductW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
            fixed bin(31),/* how much of the product to install */
            type INSTALLSTATE)
                          /* local/source/default/absent/lock/uncache */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureProductW');

 %dcl MsiConfigureProduct char ext;
 %if UNICODE ^= '' %then %do;
   %MsiConfigureProduct='MsiConfigureProductW';
 %end; %else %do;
   %MsiConfigureProduct='MsiConfigureProductA';
 %end; /* !UNICODE */

 /* Install/uninstall an advertised or installed product      */
 /* No action if installed and INSTALLSTATE_DEFAULT specified */

 dcl MsiConfigureProductExA
     entry (char(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
            fixed bin(31),/* how much of the product to install */
            type INSTALLSTATE,
                          /* local/source/default/absent/lock/uncache */
            char(*) varz byaddr nonasgn optional)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureProductExA');
 dcl MsiConfigureProductExW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
            fixed bin(31),
                          /* how much of the product to install */
            type INSTALLSTATE,
                          /* local/source/default/absent/lock/uncache */
            WCHAR(*) varz byaddr nonasgn optional)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureProductExW');

 %dcl MsiConfigureProductEx char ext;
 %if UNICODE ^= '' %then %do;
   %MsiConfigureProductEx='MsiConfigureProductExW';
 %end; %else %do;
   %MsiConfigureProductEx='MsiConfigureProductExA';
 %end; /* !UNICODE */

 /* Reinstall product, used to validate or correct problems */

 dcl MsiReinstallProductA
     entry (char(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
           type DWORD)    /* one or more REINSTALLMODE modes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiReinstallProductA');
 dcl MsiReinstallProductW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code OR descriptor */
            type DWORD)   /* one or more REINSTALLMODE modes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiReinstallProductW');

 %dcl MsiReinstallProduct char ext;
 %if UNICODE ^= '' %then %do;
   %MsiReinstallProduct='MsiReinstallProductW';
 %end; %else %do;
   %MsiReinstallProduct='MsiReinstallProductA';
 %end; /* !UNICODE */


 /* Return the product code for a registered component, */
 /* called once by apps                                 */

 dcl MsiGetProductCodeA
     entry (char(*) varz byaddr nonasgn,
                     /* component Id registered for this product */
            char(*) varz byaddr)
                     /* returned string GUID, sized for 39 characters */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductCodeA');
 dcl MsiGetProductCodeW
     entry (WCHAR(*) varz byaddr nonasgn,
                     /* component Id registered for this product */
            WCHAR(*) varz byaddr)
                     /* returned string GUID, sized for 39 characters */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductCodeW');

 %dcl MsiGetProductCode char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetProductCode='MsiGetProductCodeW';
 %end; %else %do;
   %MsiGetProductCode='MsiGetProductCodeA';
 %end; /* !UNICODE */

 /* Return the registered user information for an installed product */

 dcl MsiGetUserInfoA
     entry (char (*) varz byaddr nonasgn, /* product code, string GUID*/
            char (*) varz byaddr,     /* return user name             */
            type DWORD byaddr,        /* in/out buffer character count*/
            char (*) varz byaddr,     /* return company name          */
            type DWORD byaddr,        /* in/out buffer character count*/
            char (*) varz byaddr,     /* return product serial number */
            type DWORD byaddr)        /* in/out buffer character count*/
     returns (type USERINFOSTATE byvalue optional)
     WINAPI external('MsiGetUserInfoA');
 dcl MsiGetUserInfoW
     entry (WCHAR (*) varz byaddr nonasgn,/* product code, string GUID*/
            WCHAR (*) varz byaddr,    /* return user name             */
            type DWORD byaddr,        /* in/out buffer character count*/
            WCHAR (*) varz byaddr,    /* return company name          */
            type DWORD byaddr,        /* in/out buffer character count*/
            WCHAR (*) varz byaddr,    /* return product serial number */
            type DWORD byaddr)        /* in/out buffer character count*/
     returns (type USERINFOSTATE byvalue optional)
     WINAPI external('MsiGetUserInfoW');

 %dcl MsiGetUserInfo char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetUserInfo='MsiGetUserInfoW';
 %end; %else %do;
   %MsiGetUserInfo='MsiGetUserInfoA';
 %end; /* !UNICODE */

 /* Obtain and store user info and PID from installation wizard */
 /* (first run)                                                 */

 dcl MsiCollectUserInfoA
     entry (char(*) varz byaddr nonasgn) /* product code, string GUID */
     returns (type UINT byvalue optional)
     WINAPI external('MsiCollectUserInfoA');
 dcl MsiCollectUserInfoW
     entry (WCHAR(*) varz byaddr nonasgn)/* product code, string GUID */
     returns (type UINT byvalue optional)
     WINAPI external('MsiCollectUserInfoW');

 %dcl MsiCollectUserInfo char ext;
 %if UNICODE ^= '' %then %do;
   %MsiCollectUserInfo='MsiCollectUserInfoW';
 %end; %else %do;
   %MsiCollectUserInfo='MsiCollectUserInfoA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Functions to patch existing products                              */
 /* ----------------------------------------------------------------- */

 /* Patch all possible installed products. */

 dcl MsiApplyPatchA
     entry (char(*) varz byaddr nonasgn, /* location of patch package */
            char(*) varz byaddr nonasgn optional,
               /* location of package for install to patch <optional> */
            type INSTALLTYPE,            /* type of install to patch */
            char(*) varz byaddr nonasgn)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiApplyPatchA');
 dcl MsiApplyPatchW
     entry (WCHAR(*) varz byaddr nonasgn,/* location of patch package */
            WCHAR(*) varz byaddr nonasgn optional,
               /* location of package for install to patch <optional> */
            type INSTALLTYPE,             /* type of install to patch */
            Wchar(*) varz byaddr nonasgn)
                          /* command line <property settings> */
     returns (type UINT byvalue optional)
     WINAPI external('MsiApplyPatchW');

 %dcl MsiApplyPatch char ext;
 %if UNICODE ^= '' %then %do;
   %MsiApplyPatch='MsiApplyPatchW';
 %end; %else %do;
   %MsiApplyPatch='MsiApplyPatchA';
 %end; /* !UNICODE */


 /* ----------------------------------------------------------------- */
 /* Functions to query and configure a feature within a product.      */
 /* Separate wrapper functions are provided that accept a descriptor  */
 /* string.                                                           */
 /* ----------------------------------------------------------------- */

 /* Return the installed state for a product feature */

 dcl MsiQueryFeatureStateA
     entry (char(*) varz byaddr nonasgn,
            char(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiQueryFeatureStateA');
 dcl MsiQueryFeatureStateW
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiQueryFeatureStateW');

 %dcl MsiQueryFeatureState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiQueryFeatureState='MsiQueryFeatureStateW';
 %end; %else %do;
   %MsiQueryFeatureState='MsiQueryFeatureStateA';
 %end; /* !UNICODE */

 /* Indicate intent to use a product feature, increments usage count */
 /* Prompts for CD if not loaded, does not install feature           */

 dcl MsiUseFeatureA
     entry (char(*) varz byaddr nonasgn,
            char(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiUseFeatureA');
 dcl MsiUseFeatureW
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn)
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiUseFeatureW');

 %dcl MsiUseFeature char ext;
 %if UNICODE ^= '' %then %do;
   %MsiUseFeature='MsiUseFeatureW';
 %end; %else %do;
   %MsiUseFeature='MsiUseFeatureA';
 %end; /* !UNICODE */

 /* Indicate intent to use a product feature, increments usage count */
 /* Prompts for CD if not loaded, does not install feature. Allows   */
 /* for bypassing component detection where performance is critical  */

 dcl MsiUseFeatureExA
     entry (char(*) varz byaddr nonasgn, /* product code */
            char(*) varz byaddr nonasgn, /* feature ID */
            type DWORD,            /* INSTALLMODE_NODETECTION, else 0 */
            type DWORD)                  /* reserved, must be 0 */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiUseFeatureExA');
 dcl MsiUseFeatureExW
     entry (WCHAR(*) varz byaddr nonasgn, /* product code */
            WCHAR(*) varz byaddr nonasgn, /* feature ID */
            type DWORD,            /* INSTALLMODE_NODETECTION, else 0 */
            type DWORD)                   /* reserved, must be 0 */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiUseFeatureExW');

 %dcl MsiUseFeatureEx char ext;
 %if UNICODE ^= '' %then %do;
   %MsiUseFeatureEx='MsiUseFeatureExW';
 %end; %else %do;
   %MsiUseFeatureEx='MsiUseFeatureExA';
 %end; /* !UNICODE */
 /* Return the usage metrics for a product feature */

 dcl MsiGetFeatureUsageA
     entry (char(*) varz byaddr nonasgn, /* product code */
            char(*) varz byaddr nonasgn, /* feature ID */
            type DWORD byaddr,           /* returned use count */
            type WORD byaddr) /* last date used (DOS date format) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureUsageA');
 dcl MsiGetFeatureUsageW
     entry (WCHAR(*) varz byaddr nonasgn, /* product code */
            WCHAR(*) varz byaddr nonasgn, /* feature ID */
            type DWORD byaddr,            /* returned use count */
            type WORD byaddr)  /* last date used (DOS date format) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureUsageW');

 %dcl MsiGetFeatureUsage char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFeatureUsage='MsiGetFeatureUsageW';
 %end; %else %do;
   %MsiGetFeatureUsage='MsiGetFeatureUsageA';
 %end; /* !UNICODE */

 /* Force the installed state for a product feature */

 dcl MsiConfigureFeatureA
     entry (char(*) varz byaddr nonasgn,
            char(*) varz byaddr nonasgn,
            type INSTALLSTATE)
                          /* local/source/default/absent/lock/uncache */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureFeatureA');
 dcl MsiConfigureFeatureW
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            type INSTALLSTATE)
                          /* local/source/default/absent/lock/uncache */
     returns (type UINT byvalue optional)
     WINAPI external('MsiConfigureFeatureW');

 %dcl MsiConfigureFeature char ext;
 %if UNICODE ^= '' %then %do;
   %MsiConfigureFeature='MsiConfigureFeatureW';
 %end; %else %do;
   %MsiConfigureFeature='MsiConfigureFeatureA';
 %end; /* !UNICODE */

 /* Reinstall feature, used to validate or correct problems */

 dcl MsiReinstallFeatureA
     entry (char(*) varz byaddr nonasgn, /* product code */
            char(*) varz byaddr nonasgn optional,
                          /* feature ID, NULL for entire product */
            type DWORD)   /* one or more REINSTALLMODE modes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiReinstallFeatureA');
 dcl MsiReinstallFeatureW
     entry (WCHAR(*) varz byaddr nonasgn, /* product code */
            WCHAR(*) varz byaddr nonasgn optional,
                          /* feature ID, NULL for entire product */
            type DWORD)   /* one or more REINSTALLMODE modes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiReinstallFeatureW');

 %dcl MsiReinstallFeature char ext;
 %if UNICODE ^= '' %then %do;
   %MsiReinstallFeature='MsiReinstallFeatureW';
 %end; %else %do;
   %MsiReinstallFeature='MsiReinstallFeatureA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Functions to return a path to a particular component.             */
 /* The state of the feature being used should have been checked      */
 /* previously.                                                       */
 /* ----------------------------------------------------------------- */

 /* Return full component path, performing any necessary installation */
 /* calls MsiQueryFeatureState to detect that all components are      */
 /* installed then calls MsiConfigureFeature if any of its components */
 /* are uninstalled then calls MsiLocateComponent to obtain the path  */
 /* of its key file                                                   */

 dcl MsiProvideComponentA
     entry (char(*) varz byaddr nonasgn,
                          /* product code in case install required    */
            char(*) varz byaddr nonasgn,
                          /* feature ID in case install required      */
            char(*) varz byaddr nonasgn,     /* component ID          */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            WCHAR(*) varz byaddr,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideComponentA');
 dcl MsiProvideComponentW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code in case install required    */
            WCHAR(*) varz byaddr nonasgn,
                          /* feature ID in case install required      */
            WCHAR(*) varz byaddr nonasgn,     /* component ID         */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            WCHAR(*) varz byaddr optional,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideComponentW');

 %dcl MsiProvideComponent char ext;
 %if UNICODE ^= '' %then %do;
   %MsiProvideComponent='MsiProvideComponentW';
 %end; %else %do;
   %MsiProvideComponent='MsiProvideComponentA';
 %end; /* !UNICODE */

 /* For an advertised component that registers descriptor strings,    */
 /* return full component path, performing any necessary installation.*/
 /* Calls MsiProvideComponent to install and return the path.         */

 dcl MsiProvideQualifiedComponentA
     entry (char(*) varz byaddr nonasgn,  /* component category ID    */
            char(*) varz byaddr nonasgn,
                          /* specifies which component to access      */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            char(*) varz byaddr optional,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideQualifiedComponentA');
 dcl MsiProvideQualifiedComponentW
     entry (WCHAR(*) varz byaddr nonasgn, /* component category ID    */
            WCHAR(*) varz byaddr nonasgn,
                          /* specifies which component to access      */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            WCHAR(*) varz byaddr optional,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideQualifiedComponentW');

 %dcl MsiProvideQualifiedComponent char ext;
 %if UNICODE ^= '' %then %do;
   %MsiProvideQualifiedComponent='MsiProvideQualifiedComponentW';
 %end; %else %do;
   %MsiProvideQualifiedComponent='MsiProvideQualifiedComponentA';
 %end; /* !UNICODE */

 /* For an advertised component that registers descriptor strings,    */
 /* return full component path, performing any necessary installation.*/
 /* If the szProduct is NULL the works same as                        */
 /* MsiProvideQualifiedComponent, else will look for the descriptor   */
 /* advertised by the particular product ONLY.                        */
 /* Calls MsiProvideComponent to install and return the path.         */

 dcl MsiProvideQualifiedComponentExA
     entry (char(*) varz byaddr nonasgn,  /* component category ID    */
            char(*) varz byaddr nonasgn,
                          /* specifies which component to access      */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            char(*) varz byaddr nonasgn optional, /* the product code */
            type DWORD,   /* not used, must be zero */
            type DWORD,   /* not used, must be zero */
            char(*) varz byaddr optional,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideQualifiedComponentExA');
 dcl MsiProvideQualifiedComponentExW
     entry (WCHAR(*) varz byaddr nonasgn, /* component category ID    */
            WCHAR(*) varz byaddr nonasgn,
                          /* specifies which component to access      */
            type DWORD,   /* either of type INSTALLMODE or            */
                          /* a combination of the REINSTALLMODE flags */
            WCHAR(*) varz byaddr nonasgn optional,/* the product code */
            type DWORD,   /* not used, must be zero */
            type DWORD,   /* not used, must be zero */
            WCHAR(*) varz byaddr optional,
                          /* returned path, NULL if not desired       */
            type DWORD byaddr optional)
                          /* in/out buffer character count            */
     returns (type UINT byvalue optional)
     WINAPI external('MsiProvideQualifiedComponentExW');

 %dcl MsiProvideQualifiedComponentEx char ext;
 %if UNICODE ^= '' %then %do;
   %MsiProvideQualifiedComponentEx='MsiProvideQualifiedComponentExW';
 %end; %else %do;
   %MsiProvideQualifiedComponentEx='MsiProvideQualifiedComponentExA';
 %end; /* !UNICODE */

 /* Return full path to an installed component */

 dcl MsiGetComponentPathA
     entry (char(*) varz byaddr nonasgn,
                          /* product code for client product */
            char(*) varz byaddr nonasgn,
                          /* component Id, string GUID */
            char(*) varz byaddr optional, /* returned path */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiGetComponentPathA');
 dcl MsiGetComponentPathW
     entry (WCHAR(*) varz byaddr nonasgn,
                          /* product code for client product */
            WCHAR(*) varz byaddr nonasgn,
                          /* component Id, string GUID */
            WCHAR(*) varz byaddr optional, /* returned path */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiGetComponentPathW');

 %dcl MsiGetComponentPath char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetComponentPath='MsiGetComponentPathW';
 %end; %else %do;
   %MsiGetComponentPath='MsiGetComponentPathA';
 %end; /* !UNICODE */


 /* ----------------------------------------------------------------- */
 /* Functions to iterate registered products, features, and           */
 /* components. As with reg keys, they accept a 0-based index         */
 /* into the enumeration.                                             */
 /* ----------------------------------------------------------------- */

 /* Enumerate the registered products, either installed or advertised */

 dcl MsiEnumProductsA
     entry (type DWORD, /* 0-based index into registered products */
            char (*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumProductsA');
 dcl MsiEnumProductsW
     entry (type DWORD, /* 0-based index into registered products */
            WCHAR(*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumProductsW');

 %dcl MsiEnumProducts char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnumProducts='MsiEnumProductsW';
 %end; %else %do;
   %MsiEnumProducts='MsiEnumProductsA';
 %end; /* !UNICODE */

 /* Enumerate the advertised features for a given product. If parent  */
 /* is not required, supplying NULL will improve performance.         */

 dcl MsiEnumFeaturesA
     entry (char(*) varz byaddr nonasgn,
            type DWORD, /* 0-based index into published features */
            char(*) varz byaddr,
                   /* feature name buffer,   size=MAX_FEATURE_CHARS+1 */
            char(*) varz byaddr optional)
                   /* parent feature buffer, size=MAX_FEATURE_CHARS+1 */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumFeaturesA');
 dcl MsiEnumFeaturesW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD, /* 0-based index into published features */
            WCHAR(*) varz byaddr,
                   /* feature name buffer,   size=MA)_FEATURE_CHARS+1 */
            WCHAR(*) varz byaddr optional)
                   /* parent feature buffer, size=MA)_FEATURE_CHARS+1 */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumFeaturesW');

 %dcl MsiEnumFeatures char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnumFeatures='MsiEnumFeaturesW';
 %end; %else %do;
   %MsiEnumFeatures='MsiEnumFeaturesA';
 %end; /* !UNICODE */

 /* Enumerate the installed components for all products */

 dcl MsiEnumComponentsA
     entry (type DWORD, /* 0-based index into installed components */
            char(*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumComponentsA');
 dcl MsiEnumComponentsW
     entry (type DWORD, /* 0-based index into installed components */
            WCHAR(*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumComponentsW');

 %dcl MsiEnumComponents char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnumComponents='MsiEnumComponentsW';
 %end; %else %do;
   %MsiEnumComponents='MsiEnumComponentsA';
 %end; /* !UNICODE */

 /* Enumerate the client products for a component */

 dcl MsiEnumClientsA
     entry (char(*) varz byaddr nonasgn,
            type DWORD, /* 0-based index into client products */
            char(*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumClientsA');
 dcl MsiEnumClientsW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD, /* 0-based index into client products */
            WCHAR(*) varz byaddr)
                   /* buffer of char count: 39 (size of string GUID) */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumClientsW');

 %dcl MsiEnumClients char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnumClients='MsiEnumClientsW';
 %end; %else %do;
   %MsiEnumClients='MsiEnumClientsA';
 %end; /* !UNICODE */

 /* Enumerate the qualifiers for an advertised component. */

 dcl MsiEnumComponentQualifiersA
     entry (char(*) varz byaddr nonasgn,
                   /* generic component ID that is qualified */
            type DWORD, /* 0-based index into qualifiers */
            char(*) varz byaddr, /* qualifier buffer */
            type DWORD byaddr,
                   /* in/out qualifier buffer character count */
            char(*) varz byaddr optional, /* description buffer */
            type DWORD byaddr optional)
                   /* in/out description buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumComponentQualifiersA');
 dcl MsiEnumComponentQualifiersW
     entry (WCHAR(*) varz byaddr nonasgn,
                   /* generic component ID that is qualified */
            type DWORD, /* 0-based index into qualifiers */
            WCHAR(*) varz byaddr, /* qualifier buffer */
            type DWORD byaddr,
                   /* in/out qualifier buffer character count */
            WCHAR(*) varz byaddr optional, /* description buffer */
            type DWORD byaddr optional)
                   /* in/out description buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnumComponentQualifiersW');

 %dcl MsiEnumComponentQualifiers char ext;
 %dcl MsiEnumComponentQualifiers char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEnumComponentQualifiers='MsiEnumComponentQualifiersW';
 %end; %else %do;
   %MsiEnumComponentQualifiers='MsiEnumComponentQualifiersA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Functions to obtain product or package information.               */
 /* ----------------------------------------------------------------- */

 /* Open the installation for a product to obtain detailed information*/

 dcl MsiOpenProductA
     entry (char(*) varz byaddr nonasgn, /* product code OR descriptor*/
            type MSIHANDLE byaddr)
                           /* returned product handle, must be closed */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenProductA');
 dcl MsiOpenProductW
     entry (WCHAR(*) varz byaddr nonasgn,/* product code OR descriptor*/
            type MSIHANDLE byaddr)
                           /* returned product handle, must be closed */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenProductW');

 %dcl MsiOpenProduct char ext;
 %if UNICODE ^= '' %then %do;
   %MsiOpenProduct='MsiOpenProductW';
 %end; %else %do;
   %MsiOpenProduct='MsiOpenProductA';
 %end; /* !UNICODE */

 /* Open a product package in order to access product properties */

 dcl MsiOpenPackageA
     entry (char(*) varz byaddr nonasgn,
                   /* path to package, or database handle: #nnnn */
            type MSIHANDLE byaddr)
                   /* returned product handle, must be closed */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenPackageA');
 dcl MsiOpenPackageW
     entry (WCHAR(*) varz byaddr nonasgn,
                   /* path to package, or database handle: #nnnn */
            type MSIHANDLE byaddr)
                   /* returned product handle, must be closed */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenPackageW');

 %dcl MsiOpenPackage char ext;
 %if UNICODE ^= '' %then %do;
   %MsiOpenPackage='MsiOpenPackageW';
 %end; %else %do;
   %MsiOpenPackage='MsiOpenPackageA';
 %end; /* !UNICODE */

 /* Provide the value for an installation property. */

 dcl MsiGetProductPropertyA
     entry (type MSIHANDLE,
                   /* product handle obtained from MsiOpenProduct */
            char(*) varz byaddr nonasgn,
                   /* property name, case-sensitive */
            char(*) varz byaddr optional,
                   /* returned value, NULL if not desired */
            type DWORD byaddr optional)
                   /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductPropertyA');
 dcl MsiGetProductPropertyW
     entry (type MSIHANDLE,
                   /* product handle obtained from MsiOpenProduct */
            WCHAR(*) varz byaddr nonasgn,
                   /* property name, case-sensitive */
            WCHAR(*) varz byaddr optional,
                   /* returned value, NULL if not desired */
            type DWORD byaddr optional)
                   /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetProductPropertyW');

 %dcl MsiGetProductProperty char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetProductProperty='MsiGetProductPropertyW';
 %end; %else %do;
   %MsiGetProductProperty='MsiGetProductPropertyA';
 %end; /* !UNICODE */


 /* Determine whether a file is a package       */
 /* Returns ERROR_SUCCESS if file is a package. */

 dcl MsiVerifyPackageA
     entry (char(*) varz byaddr nonasgn)  /* location of package */
     returns (type UINT byvalue optional)
     WINAPI external('MsiVerifyPackageA');
 dcl MsiVerifyPackageW
     entry (WCHAR(*) varz byaddr nonasgn) /* location of package */
     returns (type UINT byvalue optional)
     WINAPI external('MsiVerifyPackageW');

 %dcl MsiVerifyPackage char ext;
 %if UNICODE ^= '' %then %do;
   %MsiVerifyPackage='MsiVerifyPackageW';
 %end; %else %do;
   %MsiVerifyPackage='MsiVerifyPackageA';
 %end; /* !UNICODE */


 /* Provide descriptive information for product feature: title and    */
 /* description.                                                      */
 /* Returns the install level for the feature, or -1 if feature is    */
 /* unknown.                                                          */
 /*   0 = feature is not available on this machine                    */
 /*   1 = highest priority, feature installed if parent is installed  */
 /*  >1 = decreasing priority, feature installation based on          */
 /*       InstallLevel property                                       */

 dcl MsiGetFeatureInfoA
     entry (type MSIHANDLE,
                      /* product handle obtained from MsiOpenProduct  */
            char(*) varz byaddr nonasgn,              /* feature name */
            type DWORD byaddr,    /* attribute flags for the feature, */
                                  /* using INSTALLFEATUREATTRIBUTE    */
            char(*) varz byaddr optional,
                      /* returned localized name, NULL if not desired */
            type DWORD byaddr optional,
                      /* in/out buffer character count                */
            char(*) varz byaddr optional,
                      /* returned description, NULL if not desired    */
            type DWORD byaddr optional)
                      /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureInfoA');
 dcl MsiGetFeatureInfoW
     entry (type MSIHANDLE,
                      /* product handle obtained from MsiOpenProduct  */
            WCHAR(*) varz byaddr nonasgn,             /* feature name */
            type DWORD byaddr,    /* attribute flags for the feature, */
                                  /* using INSTALLFEATUREATTRIBUTE    */
            WCHAR(*) varz byaddr optional,
                      /* returned localized name, NULL if not desired */
            type DWORD byaddr optional,
                      /* in/out buffer character count                */
            WCHAR(*) varz byaddr optional,
                      /* returned description, NULL if not desired    */
            type DWORD byaddr optional)
                      /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureInfoW');

 %dcl MsiGetFeatureInfo char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFeatureInfo='MsiGetFeatureInfoW';
 %end; %else %do;
   %MsiGetFeatureInfo='MsiGetFeatureInfoA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Functions to access or install missing components and files.      */
 /* These should be used as a last resort.                            */
 /* ----------------------------------------------------------------- */

 /* Install a component unexpectedly missing, provided only for error */
 /* recovery. This would typically occur due to failue to establish   */
 /* feature availability. The product feature having the smallest     */
 /* incremental cost is installed                                     */

 dcl MsiInstallMissingComponentA
     entry (char(*) varz byaddr nonasgn, /* product code */
            char(*) varz byaddr nonasgn, /* component Id, string GUID */
            type INSTALLSTATE) /* local/source/default, absent invalid*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallMissingComponentA');
 dcl MsiInstallMissingComponentW
     entry (WCHAR(*) varz byaddr nonasgn,/* product code */
            WCHAR(*) varz byaddr nonasgn,/* component Id, string GUID */
            type INSTALLSTATE) /* local/source/default, absent invalid*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallMissingComponentW');

 %dcl MsiInstallMissingComponent char ext;
 %if UNICODE ^= '' %then %do;
   %MsiInstallMissingComponent='MsiInstallMissingComponentW';
 %end; %else %do;
   %MsiInstallMissingComponent='MsiInstallMissingComponentA';
 %end; /* !UNICODE */

 /* Install a file unexpectedly missing, provided only for error      */
 /* recovery. This would typically occur due to failue to establish   */
 /* feature availability. The missing component is determined from    */
 /* the product's File table, then the product feature having the     */
 /* smallest incremental cost is installed                            */

 dcl MsiInstallMissingFileA
     entry (char(*) varz byaddr nonasgn,  /* product code */
            char(*) varz byaddr nonasgn)  /* file name, without path */
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallMissingFileA');
 dcl MsiInstallMissingFileW
     entry (WCHAR(*) varz byaddr nonasgn, /* product code */
            WCHAR(*) varz byaddr nonasgn) /* file name, without path */
     returns (type UINT byvalue optional)
     WINAPI external('MsiInstallMissingFileW');

 %dcl MsiInstallMissingFile char ext;
 %if UNICODE ^= '' %then %do;
   %MsiInstallMissingFile='MsiInstallMissingFileW';
 %end; %else %do;
   %MsiInstallMissingFile='MsiInstallMissingFileA';
 %end; /* !UNICODE */

 /* Return full path to an installed component without a product code */
 /* This function attempts to determine the product using             */
 /* MsiGetProductCode but is not guaranteed to find the correct       */
 /* product for the caller.                                           */
 /* MsiGetComponentPath should always be called when possible.        */

 dcl MsiLocateComponentA
     entry (char(*) varz byaddr nonasgn,  /* component Id, string GUID*/
            char(*) varz byaddr optional, /* returned path */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiLocateComponentA');
 dcl MsiLocateComponentW
     entry (WCHAR(*) varz byaddr nonasgn, /* component Id, string GUID*/
            WCHAR(*) varz byaddr optional,/* returned path */
            type DWORD byaddr optional)
                          /* in/out buffer character count */
     returns (type INSTALLSTATE byvalue optional)
     WINAPI external('MsiLocateComponentW');

 %dcl MsiLocateComponent char ext;
 %if UNICODE ^= '' %then  %do;
   %MsiLocateComponent='MsiLocateComponentW';
 %end; %else %do;
   %MsiLocateComponent='MsiLocateComponentA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Utility functions                                                 */
 /* ----------------------------------------------------------------- */

 /* Give the version string and language for a specified file */

 dcl MsiGetFileVersionA
     entry (char (*) varz byaddr nonasgn,
                          /* path to the file */
            char (*) varz byaddr optional,
                          /* returned version string */
            type DWORD byaddr optional,
                          /* in/out buffer byte count */
            char (*) varz byaddr optional,
                          /* returned language string */
            type DWORD byaddr optional)
                          /* in/out buffer byte count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFileVersionA');
 dcl MsiGetFileVersionW
     entry (WCHAR (*) varz byaddr nonasgn,
                          /* path to the file */
            WCHAR (*) varz byaddr optional,
                          /* returned version string */
            type DWORD byaddr optional,
                          /* in/out buffer byte count */
            WCHAR (*) varz byaddr optional,
                          /* returned language string */
            type DWORD byaddr optional)
                          /* in/out buffer byte count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFileVersionW');

 %dcl MsiGetFileVersion char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFileVersion='MsiGetFileVersionW';
 %end; %else %do;
   %MsiGetFileVersion='MsiGetFileVersionA';
 %end; /* !UNICODE */


 /* ----------------------------------------------------------------- */
 /* Error codes for installer access functions                        */
 /* have been merged into winerror.cpy                                */
 /* ----------------------------------------------------------------- */

 %end; /* _MSI_H_ */
