 /* ++                                                                */
 /*                                                                   */
 /* Copyright (C) 1996-1997 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   PDH.CPY                                                         */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Header file for the Performance Data Helper (PDH) DLL functions.*/
 /*                                                                   */
 /* --                                                                */
 %dcl #PDH_H_ char ext;
 %if #PDH_H_ = '' %then %do;
 %#PDH_H_ = 'Y';

 /* system include files required for datatype  */
 /* and constant definitions                    */
 %dcl #WINBASE_ char ext;
 %if #WINBASE_ = '' %then %do;
 %include winbase; /* necessary for data types used in this file */
 %end;
 %dcl #WINPERF_ char ext;
 %if #WINPERF_ = '' %then %do;
 %include winperf; /* necessary for the Detail Level definitions */
 %end;

 %include pdhmsg;  /* is sensible */

 define alias PDH_STATUS type LONG;

 %dcl PDH_FUNCTION char ext;
 %PDH_FUNCTION= 'returns (type PDH_STATUS byvalue optional) ' ||
                'options (byvalue linkage(stdcall) nodescriptor)';


 /* version info */
 dcl PDH_CVERSION_WIN40 type DWORD value('0400'xn);
 /* v1.1 revision of PDH -- basic log functions             */
 /* v1.2 of the PDH      -- adds variable instance counters */
 /* v1.3 of the PDH      -- adds log service control &      */
 /*                         stubs for NT5/PDH v2 fn's       */
 dcl PDH_VERSION        type DWORD value(PDH_CVERSION_WIN40 + '0003'xn);

 /* define severity masks */

 IsSuccessSeverity: proc (ErrorCode)
                    returns (bit (1) aligned byvalue)
                    options (inline reorder) internal;
   dcl ErrorCode type DWORD byvalue;
   dcl iand builtin;
   return (iand (ErrorCode, 'C0000000'xu) = '00000000'xu);
 end IsSuccessSeverity;

 IsInformationalSeverity: proc (ErrorCode)
                          returns (bit (1) aligned byvalue)
                          options (inline reorder) internal;
   dcl ErrorCode type DWORD byvalue;
   dcl iand builtin;
   return (iand (ErrorCode, 'C0000000'xu) = '40000000'xu);
 end IsInformationalSeverity;

 IsWarningSeverity: proc (ErrorCode)
                    returns (bit (1) aligned byvalue)
                    options (inline reorder) internal;
   dcl ErrorCode type DWORD byvalue;
   dcl iand builtin;
   return (iand (ErrorCode, 'C0000000'xu) = '80000000'xu);
 end IsWarningSeverity;

 IsErrorSeverity: proc (ErrorCode)
                  returns (bit (1) aligned byvalue)
                  options (inline reorder) internal;
   dcl ErrorCode type DWORD byvalue;
   dcl iand builtin;
   return (iand (ErrorCode, 'C0000000'xu) = 'C0000000'xu);
 end IsErrorSeverity;


 /* data type definitions */

 define alias HCOUNTER type HANDLE;
 define alias HQUERY   type HANDLE;
 define alias HLOG     type HANDLE;

 define structure
   1 PDH_RAW_COUNTER,
     2 CStatus     type DWORD,
     2 TimeStamp   type FILETIME,
     2 FirstValue  type LONGLONG,
     2 SecondValue type LONGLONG,
     2 MultiCount  type DWORD;
 define alias PPDH_RAW_COUNTER handle PDH_RAW_COUNTER;

 define structure
   1 PDH_RAW_COUNTER_ITEM_A,
     2 szName   type LPSTR,
     2 RawValue type PDH_RAW_COUNTER;
 define alias PPDH_RAW_COUNTER_ITEM_A handle PDH_RAW_COUNTER_ITEM_A;

 define structure
   1 PDH_RAW_COUNTER_ITEM_W,
     2 szName   type LPWSTR,
     2 RawValue type PDH_RAW_COUNTER;
 define alias PPDH_RAW_COUNTER_ITEM_W handle PDH_RAW_COUNTER_ITEM_W;

 define structure
   1 PDH_FMT_COUNTERVALUE,
     2 CStatus           type DWORD,
     2 * union,
       3 longValue       type LONG,
       3 doubleValue     float bin (53),
       3 largeValue      type LONGLONG,
       3 AnsiStringValue type LPSTR,
       3 WideStringValue type LPWSTR;
 define alias PPDH_FMT_COUNTERVALUE handle PDH_FMT_COUNTERVALUE;

 define structure
   1 PDH_FMT_COUNTERVALUE_ITEM_A,
     2 szName   type LPSTR,
     2 FmtValue type PDH_FMT_COUNTERVALUE;
 define alias PPDH_FMT_COUNTERVALUE_ITEM_A
        handle PDH_FMT_COUNTERVALUE_ITEM_A;

 define structure
   1 PDH_FMT_COUNTERVALUE_ITEM_W,
     2 szName   type LPWSTR,
     2 FmtValue type PDH_FMT_COUNTERVALUE;
 define alias PPDH_FMT_COUNTERVALUE_ITEM_W
        handle PDH_FMT_COUNTERVALUE_ITEM_W;

 define structure
   1 PDH_STATISTICS,
     2 dwFormat type DWORD,
     2 count    type DWORD,
     2 min      type PDH_FMT_COUNTERVALUE,
     2 max      type PDH_FMT_COUNTERVALUE,
     2 mean     type PDH_FMT_COUNTERVALUE;
 define alias PPDH_STATISTICS handle PDH_STATISTICS;

 define structure
   1 PDH_COUNTER_PATH_ELEMENTS_A,
     2 szMachineName    type LPSTR,
     2 szObjectName     type LPSTR,
     2 szInstanceName   type LPSTR,
     2 szParentInstance type LPSTR,
     2 dwInstanceIndex  type DWORD,
     2 szCounterName    type LPSTR;
 define alias PPDH_COUNTER_PATH_ELEMENTS_A
        handle PDH_COUNTER_PATH_ELEMENTS_A;

 define structure
   1 PDH_COUNTER_PATH_ELEMENTS_W,
     2 szMachineName    type LPWSTR,
     2 szObjectName     type LPWSTR,
     2 szInstanceName   type LPWSTR,
     2 szParentInstance type LPWSTR,
     2 dwInstanceIndex  type DWORD,
     2 szCounterName    type LPWSTR;
 define alias PPDH_COUNTER_PATH_ELEMENTS_W
       handle PDH_COUNTER_PATH_ELEMENTS_W;

 define structure
   1 PDH_DATA_ITEM_PATH_ELEMENTS_A,
     2 szMachineName  type LPSTR,
     2 ObjectGUID     type GUID,
     2 dwItemId       type DWORD,
     2 szInstanceName type LPSTR;
 define alias PPDH_DATA_ITEM_PATH_ELEMENTS_A
        handle PDH_DATA_ITEM_PATH_ELEMENTS_A;

 define structure
   1 PDH_DATA_ITEM_PATH_ELEMENTS_W,
     2 szMachineName  type LPWSTR,
     2 ObjectGUID     type GUID,
     2 dwItemId       type DWORD,
     2 szInstanceName type LPWSTR;
 define alias PPDH_DATA_ITEM_PATH_ELEMENTS_W
        handle PDH_DATA_ITEM_PATH_ELEMENTS_W;

 define structure
   1 PDH_COUNTER_INFO_A,
     2 dwLength        type DWORD,
     2 dwType          type DWORD,
     2 CVersion        type DWORD,
     2 CStatus         type DWORD,
     2 lScale          type LONG,
     2 lDefaultScale   type LONG,
     2 dwUserData      type DWORD,
     2 dwQueryUserData type DWORD,
     2 szFullPath      type LPSTR,
     2 * union,
       3 DataItemPath  type PDH_DATA_ITEM_PATH_ELEMENTS_A,
       3 CounterPath   type PDH_COUNTER_PATH_ELEMENTS_A,
       3 *,
         4 szMachineName    type LPSTR,
         4 szObjectName     type LPSTR,
         4 szInstanceName   type LPSTR,
         4 szParentInstance type LPSTR,
         4 dwInstanceIndex  type DWORD,
         4 szCounterName    type LPSTR,
     2 szExplainText   type LPSTR,
     2 DataBuffer (1)  type DWORD;
 define alias PPDH_COUNTER_INFO_A handle PDH_COUNTER_INFO_A;

 define structure
   1 PDH_COUNTER_INFO_W,
     2 dwLength        type DWORD,
     2 dwType          type DWORD,
     2 CVersion        type DWORD,
     2 CStatus         type DWORD,
     2 lScale          type LONG,
     2 lDefaultScale   type LONG,
     2 dwUserData      type DWORD,
     2 dwQueryUserData type DWORD,
     2 szFullPath      type LPWSTR,
     2 * union,
       3 DataItemPath  type PDH_DATA_ITEM_PATH_ELEMENTS_W,
       3 CounterPath   type PDH_COUNTER_PATH_ELEMENTS_W,
       3 *,
         4 szMachineName    type LPWSTR,
         4 szObjectName     type LPWSTR,
         4 szInstanceName   type LPWSTR,
         4 szParentInstance type LPWSTR,
         4 dwInstanceIndex  type DWORD,
         4 szCounterName    type LPWSTR,
     2 szExplainText   type LPWSTR,
     2 DataBuffer (1)  type DWORD;
 define alias PPDH_COUNTER_INFO_W handle PDH_COUNTER_INFO_W;

 define structure
   1 PDH_TIME_INFO,
     2 StartTime   type LONGLONG,
     2 EndTime     type LONGLONG,
     2 SampleCount type DWORD;
 define alias PPDH_TIME_INFO handle PDH_TIME_INFO;

 define structure
   1 PDH_RAW_LOG_RECORD,
     2 dwStructureSize type DWORD,
     2 dwRecordType    type DWORD,
     2 dwItems         type DWORD,
     2 RawBytes (1)    type UCHAR;
 define alias PPDH_RAW_LOG_RECORD handle PDH_RAW_LOG_RECORD;

 define structure
   1 PDH_LOG_SERVICE_QUERY_INFO_A,
     2 dwSize           type DWORD,
     2 dwFlags          type DWORD,
     2 dwLogQuota       type DWORD,
     2 szLogFileCaption type LPSTR,
     2 szDefaultDir     type LPSTR,
     2 szBaseFileName   type LPSTR,
     2 dwFileType       type DWORD,
     2 dwReserved       type DWORD,
     2 * union,
       3 *,
         4 PdlAutoNameInterval type DWORD,
         4 PdlAutoNameUnits    type DWORD,
         4 PdlCommandFilename  type LPSTR,
         4 PdlCounterList      type LPSTR,
         4 PdlAutoNameFormat   type DWORD,
         4 PdlSampleInterval   type DWORD,
         4 PdlLogStartTime     type FILETIME,
         4 PdlLogEndTime       type FILETIME,
       3 *,
         4 TlNumberOfBuffers   type DWORD,
         4 TlMinimumBuffers    type DWORD,
         4 TlMaximumBuffers    type DWORD,
         4 TlFreeBuffers       type DWORD,
         4 TlBufferSize        type DWORD,
         4 TlEventsLost        type DWORD,
         4 TlLoggerThreadId    type DWORD,
         4 TlBuffersWritten    type DWORD,
         4 TlLogHandle         type DWORD,
         4 TlLogFileName       type LPSTR;
 define alias PPDH_LOG_SERVICE_QUERY_INFO_A
        handle PDH_LOG_SERVICE_QUERY_INFO_A;

 define structure
   1 PDH_LOG_SERVICE_QUERY_INFO_W,
     2 dwSize           type DWORD,
     2 dwFlags          type DWORD,
     2 dwLogQuota       type DWORD,
     2 szLogFileCaption type LPSTR,
     2 szDefaultDir     type LPWSTR,
     2 szBaseFileName   type LPWSTR,
     2 dwFileType       type DWORD,
     2 dwReserved       type DWORD,
     2 * union,
       3 *,
         4 PdlAutoNameInterval type DWORD,
         4 PdlAutoNameUnits    type DWORD,
         4 PdlCommandFilename  type LPWSTR,
         4 PdlCounterList      type LPWSTR,
         4 PdlAutoNameFormat   type DWORD,
         4 PdlSampleInterval   type DWORD,
         4 PdlLogStartTime     type FILETIME,
         4 PdlLogEndTime       type FILETIME,
       3 *,
         4 TlNumberOfBuffers   type DWORD,
         4 TlMinimumBuffers    type DWORD,
         4 TlMaximumBuffers    type DWORD,
         4 TlFreeBuffers       type DWORD,
         4 TlBufferSize        type DWORD,
         4 TlEventsLost        type DWORD,
         4 TlLoggerThreadId    type DWORD,
         4 TlBuffersWritten    type DWORD,
         4 TlLogHandle         type DWORD,
         4 TlLogFileName       type LPWSTR;
 define alias PPDH_LOG_SERVICE_QUERY_INFO_W
        handle PDH_LOG_SERVICE_QUERY_INFO_W;


 /*  Time value constants */

 dcl MAX_TIME_VALUE type LONGLONG value('7FFFFFFFFFFFFFFF'xn);
 dcl MIN_TIME_VALUE type LONGLONG value(0);

 /* function definitions */

 dcl PdhGetDllVersion
     entry (type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDllVersion');


 /*  Query Functions */

 dcl PdhOpenQueryW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD,
            type HQUERY byaddr)
     PDH_FUNCTION external('PdhOpenQueryW');

 dcl PdhOpenQueryA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD,
            type HQUERY byaddr)
     PDH_FUNCTION external('PdhOpenQueryA');

 dcl PdhAddCounterW
     entry (type HQUERY,
            WCHAR (*) varz byaddr nonasgn,
            type DWORD,
            type HCOUNTER byaddr)
     PDH_FUNCTION external('PdhAddCounterW');

 dcl PdhAddCounterA
     entry (type HQUERY,
            char (*) varz byaddr nonasgn,
            type DWORD,
            type HCOUNTER byaddr)
     PDH_FUNCTION external('PdhAddCounterA');

 dcl PdhRemoveCounter
     entry (type HCOUNTER)
     PDH_FUNCTION external('PdhRemoveCounter');

 dcl PdhCollectQueryData
     entry (type HQUERY)
     PDH_FUNCTION external('PdhCollectQueryData');

 dcl PdhCloseQuery
     entry (type HQUERY)
     PDH_FUNCTION external('PdhCloseQuery');

 /*  Counter Functions */

 dcl PdhGetFormattedCounterValue
     entry (type HCOUNTER,
            type DWORD,
            type DWORD byaddr,
            type PDH_FMT_COUNTERVALUE byaddr)
     PDH_FUNCTION external('PdhGetFormattedCounterValue');

 dcl PdhGetFormattedCounterArrayA
     entry (type HCOUNTER,
            type DWORD,
            type DWORD byaddr,
            type DWORD byaddr,
            (*) type PDH_FMT_COUNTERVALUE_ITEM_A byaddr)
     PDH_FUNCTION external('PdhGetFormattedCounterArrayA');

 dcl PdhGetFormattedCounterArrayW
     entry (type HCOUNTER,
            type DWORD,
            type DWORD byaddr,
            type DWORD byaddr,
            (*) type PPDH_FMT_COUNTERVALUE_ITEM_W byaddr)
     PDH_FUNCTION external('PdhGetFormattedCounterArrayW');


 /* dwFormat flag values */

 dcl PDH_FMT_RAW          type DWORD value('00000010'xu);
 dcl PDH_FMT_ANSI         type DWORD value('00000020'xu);
 dcl PDH_FMT_UNICODE      type DWORD value('00000040'xu);
 dcl PDH_FMT_LONG         type DWORD value('00000100'xu);
 dcl PDH_FMT_DOUBLE       type DWORD value('00000200'xu);
 dcl PDH_FMT_LARGE        type DWORD value('00000400'xu);
 dcl PDH_FMT_NOSCALE      type DWORD value('00001000'xu);
 dcl PDH_FMT_1000         type DWORD value('00002000'xu);
 dcl PDH_FMT_NODATA       type DWORD value('00004000'xu);

 dcl PERF_DETAIL_COSTLY   type DWORD value('00010000'xu);
 dcl PERF_DETAIL_STANDARD type DWORD value('0000FFFF'xu);

 dcl PdhGetRawCounterValue
     entry (type HCOUNTER,
            type DWORD byaddr,
            type PDH_RAW_COUNTER byaddr)
     PDH_FUNCTION external('PdhGetRawCounterValue');

 dcl PdhGetRawCounterArrayA
     entry (type HCOUNTER,
            type DWORD byaddr,
            type DWORD byaddr,
            (*) type PDH_RAW_COUNTER_ITEM_A byaddr)
     PDH_FUNCTION external('PdhGetRawCounterArrayA');

 dcl PdhGetRawCounterArrayW
     entry (type HCOUNTER,
            type DWORD byaddr,
            type DWORD byaddr,
            (*) type PDH_RAW_COUNTER_ITEM_W byaddr)
     PDH_FUNCTION external('PdhGetRawCounterArrayW');

 dcl PdhCalculateCounterFromRawValue
     entry (type HCOUNTER,
            type DWORD,
            type PDH_RAW_COUNTER byaddr,
            type PDH_RAW_COUNTER byaddr optional,
            type PDH_FMT_COUNTERVALUE byaddr)
     PDH_FUNCTION external('PdhCalculateCounterFromRawValue');

 dcl PdhComputeCounterStatistics
     entry (type HCOUNTER,
            type DWORD,
            type DWORD,
            type DWORD,
            (*) type PDH_RAW_COUNTER byaddr,
            type PDH_STATISTICS byaddr)
     PDH_FUNCTION external('PdhComputeCounterStatistics');

 dcl PdhGetCounterInfoW
     entry (type HCOUNTER,
            type BOOLEAN,
            type DWORD byaddr,
            type PDH_COUNTER_INFO_W byaddr optional) /* var len at end*/
     PDH_FUNCTION external('PdhGetCounterInfoW');

 dcl PdhGetCounterInfoA
     entry (type HCOUNTER,
            type BOOLEAN,
            type DWORD byaddr,
            type PDH_COUNTER_INFO_A byaddr optional) /* var len at end*/
     PDH_FUNCTION external('PdhGetCounterInfoA');

 dcl PDH_MAX_SCALE fixed bin(31) value(7);
 dcl PDH_MIN_SCALE fixed bin(31) value(-7);

 dcl PdhSetCounterScaleFactor
     entry (type HCOUNTER,
            type LONG)
     PDH_FUNCTION external('PdhSetCounterScaleFactor');

 /*   Browsing and enumeration functions */

 dcl PdhConnectMachineW
     entry (WCHAR (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhConnectMachineW');

 dcl PdhConnectMachineA
     entry (char (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhConnectMachineA');

 dcl PdhEnumMachinesW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type LPWSTR, /* concatenation of nul-terminated strings */
            type DWORD byaddr)
     PDH_FUNCTION external('PdhEnumMachinesW');

 dcl PdhEnumMachinesA
     entry (char (*) varz byaddr nonasgn optional,
            type LPSTR,  /* concatenation of nul-terminated strings */
            type DWORD byaddr)
     PDH_FUNCTION external('PdhEnumMachinesA');

 dcl PdhEnumObjectsW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn optional,
            type LPWSTR, /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type DWORD,
            type BOOL)
     PDH_FUNCTION external('PdhEnumObjectsW');

 dcl PdhEnumObjectsA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn optional,
            type LPSTR,  /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type DWORD,
            type BOOL)
     PDH_FUNCTION external('PdhEnumObjectsA');

 dcl PdhEnumObjectItemsW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            type LPWSTR, /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type LPWSTR, /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type DWORD,
            type DWORD)
     PDH_FUNCTION external('PdhEnumObjectItemsW');

 dcl PdhEnumObjectItemsA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            type LPSTR,  /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type LPSTR,  /* concatenation of nul-terminated strings */
            type DWORD byaddr,
            type DWORD,
            type DWORD)
     PDH_FUNCTION external('PdhEnumObjectItemsA');

 dcl PdhMakeCounterPathW
     entry (type PDH_COUNTER_PATH_ELEMENTS_W byaddr,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr,
            type DWORD)
     PDH_FUNCTION external('PdhMakeCounterPathW');

 dcl PdhMakeCounterPathA
     entry (type PDH_COUNTER_PATH_ELEMENTS_A byaddr,
            char (*) varz byaddr optional,
            type DWORD byaddr,
            type DWORD)
     PDH_FUNCTION external('PdhMakeCounterPathA');

 dcl PdhParseCounterPathW
     entry (WCHAR (*) varz byaddr nonasgn,
            type PDH_COUNTER_PATH_ELEMENTS_W byaddr optional, /*+extra*/
            type DWORD byaddr,
            type DWORD)
     PDH_FUNCTION external('PdhParseCounterPathW');

 dcl PdhParseCounterPathA
     entry (char (*) varz byaddr nonasgn,
            type PDH_COUNTER_PATH_ELEMENTS_A byaddr optional, /*+extra*/
            type DWORD byaddr,
            type DWORD)
     PDH_FUNCTION external('PdhParseCounterPathA');

 dcl PdhParseInstanceNameW
     entry (WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr,
            type DWORD byaddr optional)
     PDH_FUNCTION external('PdhParseInstanceNameW');

 dcl PdhParseInstanceNameA
     entry (char (*) varz byaddr nonasgn,
            char (*) varz byaddr optional,
            type DWORD byaddr,
            char (*) varz byaddr optional,
            type DWORD byaddr,
            type DWORD byaddr optional)
     PDH_FUNCTION external('PdhParseInstanceNameA');

 dcl PdhValidatePathW
     entry (WCHAR (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhValidatePathW');

 dcl PdhValidatePathA
     entry (char (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhValidatePathA');

 dcl PdhGetDefaultPerfObjectW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDefaultPerfObjectW');

 dcl PdhGetDefaultPerfObjectA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDefaultPerfObjectA');

 dcl PdhGetDefaultPerfCounterW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDefaultPerfCounterW');

 dcl PdhGetDefaultPerfCounterA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDefaultPerfCounterA');

 define alias CounterPathCallBack
        entry (type DWORD)
        returns (type PDH_STATUS byvalue optional)
        options (byvalue linkage(stdcall) nodescriptor) limited;

 define structure
   1 PDH_BROWSE_DLG_CONFIG_W,
     /* Configuration flags (PL/I layout on Intel) */
     2 bDisableMachineSelection bit (1) unaligned,
     2 bInitializePath          bit (1) unaligned,
     2 bHideDetailBox           bit (1) unaligned,
     2 bWildCardInstances       bit (1) unaligned,
     2 bLocalCountersOnly       bit (1) unaligned,
     2 bSingleCounterPerDialog  bit (1) unaligned,
     2 bSingleCounterPerAdd     bit (1) unaligned,
     2 bIncludeInstanceIndex    bit (1) unaligned,
     2 * bit ( 7) unaligned,
     2 bIncludeCostlyObjects    bit (1) unaligned,
     2 * bit (16) unaligned,
     2 hWndOwner                type HWND,
     2 szDataSource             type LPWSTR,
     2 szReturnPathBuffer       type LPWSTR,
     2 cchReturnPathLength      type DWORD,
     2 pCallBack                type CounterPathCallBack,
     2 dwCallBackArg            type DWORD,
     2 CallBackStatus           type PDH_STATUS,
     2 dwDefaultDetailLevel     type DWORD,
     2 szDialogBoxCaption       type LPWSTR;
 define alias PPDH_BROWSE_DLG_CONFIG_W handle PDH_BROWSE_DLG_CONFIG_W;

 define structure
   1 PDH_BROWSE_DLG_CONFIG_A,
     /* Configuration flags (PL/I layout on Intel) */
     2 bDisableMachineSelection bit (1) unaligned,
     2 bInitializePath          bit (1) unaligned,
     2 bHideDetailBox           bit (1) unaligned,
     2 bWildCardInstances       bit (1) unaligned,
     2 bLocalCountersOnly       bit (1) unaligned,
     2 bSingleCounterPerDialog  bit (1) unaligned,
     2 bSingleCounterPerAdd     bit (1) unaligned,
     2 bIncludeInstanceIndex    bit (1) unaligned,
     2 * bit ( 7) unaligned,
     2 bIncludeCostlyObjects    bit (1) unaligned,
     2 * bit (16) unaligned,
     2 hWndOwner                type HWND,
     2 szDataSource             type LPSTR,
     2 szReturnPathBuffer       type LPSTR,
     2 cchReturnPathLength      type DWORD,
     2 pCallBack                type CounterPathCallBack,
     2 dwCallBackArg            type DWORD,
     2 CallBackStatus           type PDH_STATUS,
     2 dwDefaultDetailLevel     type DWORD,
     2 szDialogBoxCaption       type LPSTR;

 define alias PPDH_BROWSE_DLG_CONFIG_A handle PDH_BROWSE_DLG_CONFIG_A;

 dcl PdhBrowseCountersW
     entry (type PDH_BROWSE_DLG_CONFIG_W byaddr)
     PDH_FUNCTION external('PdhBrowseCountersW');

 dcl PdhBrowseCountersA
     entry (type PDH_BROWSE_DLG_CONFIG_A byaddr)
     PDH_FUNCTION external('PdhBrowseCountersA');

 dcl PdhExpandCounterPathW
     entry (WCHAR (*) varz byaddr nonasgn,
            type LPWSTR, /* concatenation of nul-terminated strings */
            type DWORD byaddr)
     PDH_FUNCTION external('PdhExpandCounterPathW');

 dcl PdhExpandCounterPathA
     entry (char (*) varz byaddr nonasgn,
            type LPSTR,  /* concatenation of nul-terminated strings */
            type DWORD byaddr)
     PDH_FUNCTION external('PdhExpandCounterPathA');


 /*  v2.0 functions */

 dcl PdhLookupPerfNameByIndexW
     entry (WCHAR (*) varz byaddr nonasgn,
            type DWORD,
            WCHAR (*) varz byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLookupPerfNameByIndexW');

 dcl PdhLookupPerfNameByIndexA
     entry (char (*) varz byaddr nonasgn,
            type DWORD,
            char (*) varz byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLookupPerfNameByIndexA');

 dcl PdhLookupPerfIndexByNameW
     entry (WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr nonasgn,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLookupPerfIndexByNameW');

 dcl PdhLookupPerfIndexByNameA
     entry (char (*) varz byaddr nonasgn,
            char (*) varz byaddr nonasgn,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLookupPerfIndexByNameA');

 /*   Logging Functions */

 dcl PDH_LOG_READ_ACCESS        type DWORD value('00010000'xu);
 dcl PDH_LOG_WRITE_ACCESS       type DWORD value('00020000'xu);
 dcl PDH_LOG_ACCESS_MASK        type DWORD value('000F0000'xu);

 dcl PDH_LOG_CREATE_NEW         type DWORD value('00000001'xu);
 dcl PDH_LOG_CREATE_ALWAYS      type DWORD value('00000002'xu);
 dcl PDH_LOG_OPEN_ALWAYS        type DWORD value('00000003'xu);
 dcl PDH_LOG_OPEN_EXISTING      type DWORD value('00000004'xu);
 dcl PDH_LOG_CREATE_MASK        type DWORD value('0000000F'xu);

 dcl PDH_LOG_OPT_USER_STRING    type DWORD value('01000000'xu);
 dcl PDH_LOG_OPT_MASK           type DWORD value('0F000000'xu);

 dcl PDH_LOG_TYPE_UNDEFINED     fixed bin(31) value(0);
 dcl PDH_LOG_TYPE_CSV           fixed bin(31) value(1);
 dcl PDH_LOG_TYPE_TSV           fixed bin(31) value(2);
 dcl PDH_LOG_TYPE_BINARY        fixed bin(31) value(3);
 dcl PDH_LOG_TYPE_TRACE_KERNEL  fixed bin(31) value(4);
 dcl PDH_LOG_TYPE_TRACE_GENERIC fixed bin(31) value(5);

 dcl PdhOpenLogW
     entry (WCHAR (*) varz byaddr nonasgn,
            type DWORD,
            type DWORD byaddr,
            type HQUERY,
            type DWORD,
            WCHAR (*) varz byaddr nonasgn,
            type HLOG byaddr)
     PDH_FUNCTION external('PdhOpenLogW');

 dcl PdhOpenLogA
     entry (char (*) varz byaddr nonasgn,
            type DWORD,
            type DWORD byaddr,
            type HQUERY,
            type DWORD,
            char (*) varz byaddr nonasgn,
            type HLOG byaddr)
     PDH_FUNCTION external('PdhOpenLogA');

 dcl PdhUpdateLogW
     entry (type HLOG,
            WCHAR (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhUpdateLogW');

 dcl PdhUpdateLogA
     entry (type HLOG,
            char (*) varz byaddr nonasgn)
     PDH_FUNCTION external('PdhUpdateLogA');

 dcl PdhGetLogFileSize
     entry (type HLOG,
            type LONGLONG byaddr)
     PDH_FUNCTION external('PdhGetLogFileSize');

 dcl PdhCloseLog
     entry (type HLOG,
            type DWORD)
     PDH_FUNCTION external('PdhCloseLog');

 dcl PDH_FLAGS_CLOSE_QUERY type DWORD value('00000001'xu);

 /*  Data source selection dialog */

 dcl PDH_FLAGS_FILE_BROWSER_ONLY type DWORD value('00000001'xu);

 dcl PdhSelectDataSourceW
     entry (type HWND,
            type DWORD,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhSelectDataSourceW');

 dcl PdhSelectDataSourceA
     entry (type HWND,
            type DWORD,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhSelectDataSourceA');

 dcl PdhIsRealTimeQuery
     entry (type HQUERY)
     options (byvalue linkage(stdcall) nodescriptor) /* cdecl ??? */
     external('PdhIsRealTimeQuery');

 dcl PdhSetQueryTimeRange
     entry (type HQUERY,
            type PDH_TIME_INFO byaddr)
     PDH_FUNCTION external('PdhSetQueryTimeRange');

 dcl PdhGetDataSourceTimeRangeW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD byaddr,
            type PDH_TIME_INFO byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDataSourceTimeRangeW');

 dcl PdhGetDataSourceTimeRangeA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD byaddr,
            type PDH_TIME_INFO byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhGetDataSourceTimeRangeA');

 dcl PdhCollectQueryDataEx
     entry (type HQUERY,
            type DWORD,
            type HANDLE)
     PDH_FUNCTION external('PdhCollectQueryDataEx');

 dcl PdhFormatFromRawValue
     entry (type DWORD,
            type DWORD,
            type LONGLONG byaddr,
            type PDH_RAW_COUNTER byaddr,
            type PDH_RAW_COUNTER byaddr optional,
            type PDH_FMT_COUNTERVALUE byaddr)
     PDH_FUNCTION external('PdhFormatFromRawValue');

 dcl PdhGetCounterTimeBase
     entry (type HCOUNTER,
            type LONGLONG byaddr)
     PDH_FUNCTION external('PdhGetCounterTimeBase');

 dcl PdhEncodeWmiPathA
     entry (char (*) varz byaddr nonasgn,
            type PDH_DATA_ITEM_PATH_ELEMENTS_A byaddr,
            type DWORD byaddr,
            type LANGID,
            type DWORD)
     PDH_FUNCTION external('PdhEncodeWmiPathA');

 dcl PdhEncodeWmiPathW
     entry (WCHAR (*) varz byaddr nonasgn,
            type PDH_DATA_ITEM_PATH_ELEMENTS_W byaddr,
            type DWORD byaddr,
            type LANGID,
            type DWORD)
     PDH_FUNCTION external('PdhEncodeWmiPathW');

 dcl PdhDecodeWmiPathA
     entry (type PDH_DATA_ITEM_PATH_ELEMENTS_A byaddr,
            type PDH_COUNTER_PATH_ELEMENTS_A byaddr,
            type DWORD byaddr,
            type LANGID,
            type DWORD)
     PDH_FUNCTION external('PdhDecodeWmiPathA');

 dcl PdhDecodeWmiPathW
     entry (type PDH_DATA_ITEM_PATH_ELEMENTS_W byaddr,
            type PDH_COUNTER_PATH_ELEMENTS_W byaddr,
            type DWORD byaddr,
            type LANGID,
            type DWORD)
     PDH_FUNCTION external('PdhDecodeWmiPathW');

 dcl PdhReadRawLogRecord
     entry (type HLOG,
            type FILETIME,
            type PDH_RAW_LOG_RECORD byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhReadRawLogRecord');

 /* flags for the log service api's */
 dcl PDH_LOGSVC_NO_WAIT          type DWORD value('80000000'xu);
 dcl PDH_LOGSVC_ALL_QUERIES      type DWORD value('00000001'xu);
 dcl PDH_LOGSVC_TRACE_LOG        type DWORD value('00000002'xu);

 dcl PDH_LOGSVC_CMD_START        type DWORD value('00000010'xu);
 dcl PDH_LOGSVC_CMD_STOP         type DWORD value('00000020'xu);

 dcl PDH_LOGSVC_CTRL_ADD         type DWORD value('00000100'xu);
 dcl PDH_LOGSVC_CTRL_REMOVE      type DWORD value('00000200'xu);
 dcl PDH_LOGSVC_CTRL_INFO        type DWORD value('00000400'xu);

 dcl PDH_LOGSVC_STATUS_RUNNING   type DWORD value('00001000'xu);
 dcl PDH_LOGSVC_STATUS_STOPPED   type DWORD value('00002000'xu);
 dcl PDH_LOGSVC_STATUS_PAUSED    type DWORD value('00004000'xu);
 dcl PDH_LOGSVC_STATUS_ERROR     type DWORD value('00008000'xu);
 dcl PDH_LOGSVC_STATUS_PENDING   type DWORD value('00010000'xu);

 dcl PDH_LOGSVC_NAME_UNDEFINED   fixed bin(31) value(-1 );
 dcl PDH_LOGSVC_NAME_MMDDHH      type DWORD value(0);
 dcl PDH_LOGSVC_NAME_NNNNNN      type DWORD value(1);
 dcl PDH_LOGSVC_NAME_YYDDD       type DWORD value(2);
 dcl PDH_LOGSVC_NAME_YYMM        type DWORD value(3);
 dcl PDH_LOGSVC_NAME_YYMMDD      type DWORD value(4);
 dcl PDH_LOGSVC_NAME_YYMMDDHH    type DWORD value(5);

 dcl PDH_LOGSVC_RENAME_UNDEFINED fixed bin(31) value(-1 );
 dcl PDH_LOGSVC_RENAME_HOURS     type DWORD value(0);
 dcl PDH_LOGSVC_RENAME_DAYS      type DWORD value(1);
 dcl PDH_LOGSVC_RENAME_MONTHS    type DWORD value(2);
 dcl PDH_LOGSVC_RENAME_KBYTES    type DWORD value(3);
 dcl PDH_LOGSVC_RENAME_MBYTES    type DWORD value(4);

 dcl PdhLogServiceCommandA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            type DWORD,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLogServiceCommandA');

 dcl PdhLogServiceCommandW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            type DWORD,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLogServiceCommandW');

 dcl PdhLogServiceControlA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            type DWORD,
            type PDH_LOG_SERVICE_QUERY_INFO_A byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLogServiceControlA');

 dcl PdhLogServiceControlW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            type DWORD,
            type PDH_LOG_SERVICE_QUERY_INFO_W byaddr,
            type DWORD byaddr)
     PDH_FUNCTION external('PdhLogServiceControlW');

 /*   Unicode/ANSI compatibility section */

 %dcl PdhOpenQuery                 char ext;
 %dcl PdhAddCounter                char ext;
 %dcl PdhGetCounterInfo            char ext;
 %dcl PDH_COUNTER_INFO             char ext;
 %dcl PPDH_COUNTER_INFO            char ext;
 %dcl PdhConnectMachine            char ext;
 %dcl PdhEnumMachines              char ext;
 %dcl PdhEnumObjects               char ext;
 %dcl PdhEnumObjectItems           char ext;
 %dcl PdhMakeCounterPath           char ext;
 %dcl PDH_COUNTER_PATH_ELEMENTS    char ext;
 %dcl PPDH_COUNTER_PATH_ELEMENTS   char ext;
 %dcl PdhParseCounterPath          char ext;
 %dcl PdhParseInstanceName         char ext;
 %dcl PdhValidatePath              char ext;
 %dcl PdhGetDefaultPerfObject      char ext;
 %dcl PdhGetDefaultPerfCounter     char ext;
 %dcl PdhBrowseCounters            char ext;
 %dcl PDH_BROWSE_DLG_CONFIG        char ext;
 %dcl PPDH_BROWSE_DLG_CONFIG       char ext;
 %dcl PdhExpandCounterPath         char ext;
 /* v2.0 functions */
 %dcl PDH_FMT_COUNTERVALUE_ITEM    char ext;
 %dcl PPDH_FMT_COUNTERVALUE_ITEM   char ext;
 %dcl PDH_RAW_COUNTER_ITEM         char ext;
 %dcl PPDH_RAW_COUNTER_ITEM        char ext;
 %dcl PdhGetFormattedCounterArray  char ext;
 %dcl PdhGetRawCounterArray        char ext;
 %dcl PdhLookupPerfNameByIndex     char ext;
 %dcl PdhLookupIndexByPerfName     char ext;
 %dcl PdhOpenLog                   char ext;
 %dcl PdhUpdateLog                 char ext;
 %dcl PdhSelectDataSource          char ext;
 %dcl PdhGetDataSourceTimeRange    char ext;
 %dcl PDH_DATA_ITEM_PATH_ELEMENTS  char ext;
 %dcl PPDH_DATA_ITEM_PATH_ELEMENTS char ext;
 %dcl PdhEncodeWmiPath             char ext;
 %dcl PdhDecodeWmiPath             char ext;
 %dcl PDH_LOG_SERVICE_QUERY_INFO   char ext;
 %dcl PPDH_LOG_SERVICE_QUERY_INFO  char ext;
 %dcl PdhLogServiceControl         char ext;
 %dcl PdhLogServiceQuery           char ext;

 %dcl UNICODE char ext;
 %if UNICODE ^= '' %then  %do;
 /* start of UNICODE definitions */
 %PdhOpenQuery                 = 'PdhOpenQueryW';
 %PdhAddCounter	               = 'PdhAddCounterW';
 %PdhGetCounterInfo            = 'PdhGetCounterInfoW';
 %PDH_COUNTER_INFO	            = 'PDH_COUNTER_INFO_W';
 %PPDH_COUNTER_INFO	         = 'PPDH_COUNTER_INFO_W';
 %PdhConnectMachine            = 'PdhConnectMachineW';
 %PdhEnumMachines              = 'PdhEnumMachinesW';
 %PdhEnumObjects               = 'PdhEnumObjectsW';
 %PdhEnumObjectItems           = 'PdhEnumObjectItemsW';
 %PdhMakeCounterPath           = 'PdhMakeCounterPathW';
 %PDH_COUNTER_PATH_ELEMENTS    = 'PDH_COUNTER_PATH_ELEMENTS_W';
 %PPDH_COUNTER_PATH_ELEMENTS   = 'PPDH_COUNTER_PATH_ELEMENTS_W';
 %PdhParseCounterPath          = 'PdhParseCounterPathW';
 %PdhParseInstanceName         = 'PdhParseInstanceNameW';
 %PdhValidatePath              = 'PdhValidatePathW';
 %PdhGetDefaultPerfObject      = 'PdhGetDefaultPerfObjectW';
 %PdhGetDefaultPerfCounter     = 'PdhGetDefaultPerfCounterW';
 %PdhBrowseCounters            = 'PdhBrowseCountersW';
 %PDH_BROWSE_DLG_CONFIG        = 'PDH_BROWSE_DLG_CONFIG_W';
 %PPDH_BROWSE_DLG_CONFIG       = 'PPDH_BROWSE_DLG_CONFIG_W';
 %PdhExpandCounterPath         = 'PdhExpandCounterPathW';
 /* v2.0 functions */
 %PDH_FMT_COUNTERVALUE_ITEM    = 'PDH_FMT_COUNTERVALUE_ITEM_W';
 %PPDH_FMT_COUNTERVALUE_ITEM   = 'PPDH_FMT_COUNTERVALUE_ITEM_W';
 %PDH_RAW_COUNTER_ITEM         = 'PDH_RAW_COUNTER_ITEM_W';
 %PPDH_RAW_COUNTER_ITEM        = 'PPDH_RAW_COUNTER_ITEM_W';
 %PdhGetFormattedCounterArray  = 'PdhGetFormattedCounterArrayW';
 %PdhGetRawCounterArray        = 'PdhGetRawCounterArrayW';
 %PdhLookupPerfNameByIndex     = 'PdhLookupPerfNameByIndexW';
 %PdhLookupIndexByPerfName     = 'PdhLookupIndexByPerfNameW';
 %PdhOpenLog                   = 'PdhOpenLogW';
 %PdhUpdateLog                 = 'PdhUpdateLogW';
 %PdhSelectDataSource          = 'PdhSelectDataSourceW';
 %PdhGetDataSourceTimeRange    = 'PdhGetDataSourceTimeRangeW';
 %PDH_DATA_ITEM_PATH_ELEMENTS  = 'PDH_DATA_ITEM_PATH_ELEMENTS_W';
 %PPDH_DATA_ITEM_PATH_ELEMENTS = 'PPDH_DATA_ITEM_PATH_ELEMENTS_W';
 %PdhEncodeWmiPath             = 'PdhEncodeWmiPathW';
 %PdhDecodeWmiPath             = 'PdhDecodeWmiPathW';
 %PDH_LOG_SERVICE_QUERY_INFO   = 'PDH_LOG_SERVICE_QUERY_INFO_W';
 %PPDH_LOG_SERVICE_QUERY_INFO  = 'PPDH_LOG_SERVICE_QUERY_INFO_W';
 %PdhLogServiceControl         = 'PdhLogServiceControlW';
 %PdhLogServiceQuery           = 'PdhLogServiceQueryW';
 /* end of UNICODE definitions */
 %end; %else %do;
 /* start of ANSI definitions */
 %PdhOpenQuery                 = 'PdhOpenQueryA';
 %PdhAddCounter                = 'PdhAddCounterA';
 %PdhGetCounterInfo            = 'PdhGetCounterInfoA';
 %PDH_COUNTER_INFO             = 'PDH_COUNTER_INFO_A';
 %PPDH_COUNTER_INFO            = 'PPDH_COUNTER_INFO_A';
 %PdhConnectMachine            = 'PdhConnectMachineA';
 %PdhEnumMachines              = 'PdhEnumMachinesA';
 %PdhEnumObjects               = 'PdhEnumObjectsA';
 %PdhEnumObjectItems           = 'PdhEnumObjectItemsA';
 %PdhMakeCounterPath           = 'PdhMakeCounterPathA';
 %PDH_COUNTER_PATH_ELEMENTS    = 'PDH_COUNTER_PATH_ELEMENTS_A';
 %PPDH_COUNTER_PATH_ELEMENTS   = 'PPDH_COUNTER_PATH_ELEMENTS_A';
 %PdhParseCounterPath          = 'PdhParseCounterPathA';
 %PdhParseInstanceName         = 'PdhParseInstanceNameA';
 %PdhValidatePath              = 'PdhValidatePathA';
 %PdhGetDefaultPerfObject      = 'PdhGetDefaultPerfObjectA';
 %PdhGetDefaultPerfCounter     = 'PdhGetDefaultPerfCounterA';
 %PdhBrowseCounters            = 'PdhBrowseCountersA';
 %PDH_BROWSE_DLG_CONFIG        = 'PDH_BROWSE_DLG_CONFIG_A';
 %PPDH_BROWSE_DLG_CONFIG       = 'PPDH_BROWSE_DLG_CONFIG_A';
 %PdhExpandCounterPath         = 'PdhExpandCounterPathA';
 /* v2.0 functions */
 %PDH_FMT_COUNTERVALUE_ITEM    = 'PDH_FMT_COUNTERVALUE_ITEM_A';
 %PPDH_FMT_COUNTERVALUE_ITEM   = 'PPDH_FMT_COUNTERVALUE_ITEM_A';
 %PDH_RAW_COUNTER_ITEM         = 'PDH_RAW_COUNTER_ITEM_A';
 %PPDH_RAW_COUNTER_ITEM        = 'PPDH_RAW_COUNTER_ITEM_A';
 %PdhGetFormattedCounterArray  = 'PdhGetFormattedCounterArrayA';
 %PdhGetRawCounterArray        = 'PdhGetRawCounterArrayA';
 %PdhLookupPerfNameByIndex     = 'PdhLookupPerfNameByIndexA';
 %PdhLookupIndexByPerfName     = 'PdhLookupIndexByPerfNameA';
 %PdhOpenLog                   = 'PdhOpenLogA';
 %PdhUpdateLog                 = 'PdhUpdateLogA';
 %PdhSelectDataSource          = 'PdhSelectDataSourceA';
 %PdhGetDataSourceTimeRange    = 'PdhGetDataSourceTimeRangeA';
 %PDH_DATA_ITEM_PATH_ELEMENTS  = 'PDH_DATA_ITEM_PATH_ELEMENTS_A';
 %PPDH_DATA_ITEM_PATH_ELEMENTS = 'PPDH_DATA_ITEM_PATH_ELEMENTS_A';
 %PdhEncodeWmiPath             = 'PdhEncodeWmiPathA';
 %PdhDecodeWmiPath             = 'PdhDecodeWmiPathA';
 %PDH_LOG_SERVICE_QUERY_INFO   = 'PDH_LOG_SERVICE_QUERY_INFO_A';
 %PPDH_LOG_SERVICE_QUERY_INFO  = 'PPDH_LOG_SERVICE_QUERY_INFO_A';
 %PdhLogServiceControl         = 'PdhLogServiceControlA';
 %PdhLogServiceQuery           = 'PdhLogServiceQueryA';
 /* end of ANSI definitions */
 %end; /* UNICODE */

 %end; /*_PDH_H_ */
