 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1998 - 1999  Microsoft Corporation                  */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*     qos.cpy - QoS definitions for NDIS components.                */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*     This module defines the type of objects that can go into the  */
 /*     ProviderSpecific buffer in the QOS structure.                 */
 /*                                                                   */
 /* Revision History:                                                 */
 /*                                                                   */
 /* --                                                                */

 %dcl #_QOSSP_H_ char ext;
 %if #_QOSSP_H_ = '' %then %do;
 %#_QOSSP_H_ = 'Y';

 %dcl #WINSOCK2API_ char ext;
 %if #WINSOCK2API_ = '' %then %do;
   %include winsock2;
 %end;

 /* Definition of object Types                                        */
 /*                                                                   */
 /* define the values for ObjectType above - RSVP Objects ids start   */
 /* at an offset from zero to allow for ATM objects that might be     */
 /* defined in the lower number range.                                */

 dcl RSVP_OBJECT_ID_BASE      type ULONG value(1000);
 dcl RSVP_OBJECT_STATUS_INFO  type ULONG
     value('00000000'xu + RSVP_OBJECT_ID_BASE);
     /* RSVP_STATUS_INFO structure passed */
 dcl RSVP_OBJECT_RESERVE_INFO type ULONG
     value('00000001'xu + RSVP_OBJECT_ID_BASE);
     /* RSVP_RESERVE_INFO structure passed */
 dcl RSVP_OBJECT_ADSPEC       type ULONG
     value('00000002'xu + RSVP_OBJECT_ID_BASE);
     /* RSVP_ADSPEC structure passed */
 dcl RSVP_OBJECT_POLICY_INFO  type ULONG
     value('00000003'xu + RSVP_OBJECT_ID_BASE);
     /* RSVP POLICY ELEMENT(S) retrieved */


 /* IPV4 addressing for RSVP FILTERSPECS */
 define structure
   1 IN_ADDR_IPV4 union,
     2 Addr          type ULONG,
     2 AddrBytes (4) type UCHAR;
 define alias LPIN_ADDR_IPV4 handle IN_ADDR_IPV4;

 /* IPV6 addressing for RSVP FILTERSPECS */
 define structure
   1 IN_ADDR_IPV6,
     2 Addr (16)     type UCHAR; /*IPV6 address */
 define alias  LPIN_ADDR_IPV6 handle IN_ADDR_IPV6;
 define alias LPCIN_ADDR_IPV6 handle IN_ADDR_IPV6; /* nonasgn */


 /* IPV4 addressing for RSVP FILTERSPECS */
 define structure
   1 RSVP_FILTERSPEC_V4,
     2 Address       type IN_ADDR_IPV4,
     2 Unused        type USHORT,
     2 Port          type USHORT;
 define alias LPRSVP_FILTERSPEC_V4 handle RSVP_FILTERSPEC_V4;

 define structure
   1 RSVP_FILTERSPEC_V6,
     2 Address       type IN_ADDR_IPV6,
     2 UnUsed        type USHORT,
     2 Port          type USHORT;
 define alias LPRSVP_FILTERSPEC_V6 handle RSVP_FILTERSPEC_V6;


 define structure
   1 RSVP_FILTERSPEC_V6_FLOW,
     2 Address       type IN_ADDR_IPV6,
     2 UnUsed        type UCHAR,
     2 FlowLabel (3) type UCHAR;
 define alias LPRSVP_FILTERSPEC_V6_FLOW handle RSVP_FILTERSPEC_V6_FLOW;

 define structure
   1 RSVP_FILTERSPEC_V4_GPI,
     2 Address       type IN_ADDR_IPV4,
     2 GeneralPortId type ULONG;
 define alias LPRSVP_FILTERSPEC_V4_GPI handle RSVP_FILTERSPEC_V4_GPI;

 define structure
   1 RSVP_FILTERSPEC_V6_GPI,
     2 Address       type IN_ADDR_IPV6,
     2 GeneralPortId type ULONG;
 define alias LPRSVP_FILTERSPEC_V6_GPI handle RSVP_FILTERSPEC_V6_GPI;


 /* FILTERSPEC TYPES used in making reservations. */
 define ordinal FilterType
   ( FILTERSPECV4      value(1),
     FILTERSPECV6,
     FILTERSPECV6_FLOW,
     FILTERSPECV4_GPI,
     FILTERSPECV6_GPI,
     FILTERSPEC_END
   ) prec (31);


 define structure
   1 RSVP_FILTERSPEC,
     2 Type type FilterType,
     2 *    union,
       3 FilterSpecV4     type RSVP_FILTERSPEC_V4,
       3 FilterSpecV6     type RSVP_FILTERSPEC_V6,
       3 FilterSpecV6Flow type RSVP_FILTERSPEC_V6_FLOW,
       3 FilterSpecV4Gpi  type RSVP_FILTERSPEC_V4_GPI,
       3 FilterSpecV6Gpi  type RSVP_FILTERSPEC_V6_GPI;
 define alias LPRSVP_FILTERSPEC handle RSVP_FILTERSPEC;

 /* FLOWDESCRIPTOR Structure used for specifying one or more */
 /* Filters per Flowspec.                                    */

 define structure
   1 FLOWDESCRIPTOR,
     2 FlowSpec   type FLOWSPEC,
     2 NumFilters type ULONG,
     2 FilterList type LPRSVP_FILTERSPEC;
 define alias LPFLOWDESCRIPTOR handle FLOWDESCRIPTOR;

 /* RSVP_POLICY contains undefined policy data. RSVP transports this */
 /* data on behalf of the Policy Control component.                  */
 define structure
   1 RSVP_POLICY,
     2 Len      type USHORT, /* total length of this element object */
     2 Type     type USHORT,
     2 Info (4) type UCHAR;
 define alias  LPRSVP_POLICY handle RSVP_POLICY;
 define alias LPCRSVP_POLICY handle RSVP_POLICY; /* nonasgn */
 dcl RSVP_POLICY_HDR_LEN fixed bin(31)
     value (size (: USHORT:) + size (: USHORT :));

 /* RSVP_POLICY_INFO contains undefined policy element(s) retrieved */
 /* from RSVP.                                                      */
 define structure
   1 RSVP_POLICY_INFO,
     2 ObjectHdr         type QOS_OBJECT_HDR,
     2 NumPolicyElement  type ULONG,
       /* count of the * number of policy elements */
     2 PolicyElement (1) type RSVP_POLICY;
       /* a list of the policy elements retrieved */
 define alias LPRSVP_POLICY_INFO handle RSVP_POLICY_INFO;

 /* RSVP_RESERVE_INFO Structure used for storing RSVP specific        */
 /* information for fine tuning interaction via the Winsock2          */
 /* Generic QoS API via the provider specific buffer. This structure  */
 /* includes the QOS_OBJECT_HDR structure directly                    */

 define structure
   1 RSVP_RESERVE_INFO,
     2 ObjectHdr         type QOS_OBJECT_HDR,
       /* type and length of this object */
     2 Style             type ULONG,
       /* RSVP Style (FF,WF,SE) */
     2 ConfirmRequest    type ULONG,
       /* Non Zero for Confirm Request (receive only) */
     2 PolicyElementList type LPRSVP_POLICY_INFO,
       /* Points to the set of policy elements */
     2 NumFlowDesc       type ULONG,
       /* Number of FlowDesc */
     2 FlowDescList      type LPFLOWDESCRIPTOR;
       /* Points to the FlowDesc list */
 define alias  LPRSVP_RESERVE_INFO handle RSVP_RESERVE_INFO;
 define alias LPCRSVP_RESERVE_INFO handle RSVP_RESERVE_INFO; /*nonasgn*/


 /* definitions for the ulStyle in the previous structure */
 dcl RSVP_DEFAULT_STYLE         type ULONG value('00000000'xu);
 dcl RSVP_WILDCARD_STYLE        type ULONG value('00000001'xu);
 dcl RSVP_FIXED_FILTER_STYLE    type ULONG value('00000002'xu);
 dcl RSVP_SHARED_EXPLICIT_STYLE type ULONG value('00000003'xu);

 /* RSVP_STATUS_INFO Structure used for storing RSVP specific         */
 /* error of status indications.  This also serves as a header        */
 /* for additional objects in the provider specific buffer when       */
 /* interacting via Winsock2 Generic QoS API. This structure includes */
 /* the QOS_OBJECT_COUNT and QOS_OBJECT_HDR structures directly It    */
 /* is expected to be the first structure in the provider specific    */
 /* structure since it includes the QOS_OBJECT_COUNT                  */

 define structure
   1 RSVP_STATUS_INFO,
     2 ObjectHdr       type QOS_OBJECT_HDR,
       /* Object Hdr */
     2 StatusCode      type ULONG,
       /* Error or Status Information see * Winsock2.cpy */
     2 ExtendedStatus1 type ULONG,
       /* Provider specific status extension */
     2 ExtendedStatus2 type ULONG;
       /* Provider specific status extension */
 define alias  LPRSVP_STATUS_INFO handle RSVP_STATUS_INFO;
 define alias LPCRSVP_STATUS_INFO handle RSVP_STATUS_INFO; /* nonasgn */


 /* QOS_DestAddr structure -- used for WSAIoctl(SIO_SET_QOS) when we  */
 /* do not want to issue a connect for a sending socket.              */
 /* The destination address is required so we can generate the        */
 /* session information for RSVP signalling.                          */

 define structure
   1 QOS_DESTADDR,
     2 ObjectHdr           type QOS_OBJECT_HDR,
       /* Object header */
     2 SocketAddress       handle sockaddr,
       /* Destination socket address */
     2 SocketAddressLength type ULONG;
       /* Length of the address structure */
 define alias  LPQOS_DESTADDR handle QOS_DESTADDR;
 define alias LPCQOS_DESTADDR handle QOS_DESTADDR; /* nonasgn */


 /* this structure defines the "General Characterization Parameters" */
 /* contained in the RSVP Adspec object                              */
 define structure
   1 AD_GENERAL_PARAMS,
     2 IntServAwareHopCount  type ULONG,
       /* number of hops that conform to   */
       /* Integrated Services requirements */
     2 PathBandwidthEstimate type ULONG,
       /* minimum bandwidth available from */
       /* sender to receiver */
     2 MinimumLatency        type ULONG,
       /* sum of minimum latency of the packet */
       /* forwarding process in routers (in usec)*/
     2 PathMTU               type ULONG,
       /* max transmission unit end to end that */
       /* will not incur fragmentation          */
     2 Flags                 type ULONG;
       /* used to hold break bits.*/
 define alias LPAD_GENERAL_PARAMS handle AD_GENERAL_PARAMS;

 /* Minimum Latency may be set to this "undefined" value */
 dcl INDETERMINATE_LATENCY type ULONG value('FFFFFFFF'xu);

 /* This Flag is used to indicate the existence of a network element  */
 /* not supporting  QoS control services somewhere in the data path.  */
 /* If this bit is set in the specific service override then it       */
 /* indicates that that service was not supported at at least one hop.*/
 dcl AD_FLAG_BREAK_BIT type ULONG value('00000001'xu);

 /* this structure describes the Guaranteed service parameters */
 define structure
   1 AD_GUARANTEED,
     2 CTotal type ULONG,
     2 DTotal type ULONG,
     2 CSum   type ULONG,
     2 DSum   type ULONG;
 define alias LPAD_GUARANTEED handle AD_GUARANTEED;

 /* this structure describes the format of the parameter buffer that  */
 /* can be included in the Service_Type structure below.              */
 /* This structure allows an application to include any valid Int     */
 /* Serv service parameter in the Buffer value, after providing the   */
 /* Int Serv parameter id in the ParameterId field.                   */
 define structure
   1 PARAM_BUFFER,
     2 ParameterId type ULONG, /* Int Server parameter ID */
     2 Length      type ULONG, /* total length of this structure */
                               /* ( 8 bytes + length of Buffer ) */
     2 Buffer (1)  type UCHAR; /* Paramter itself */
 define alias LPPARAM_BUFFER handle PARAM_BUFFER;

 /* this structure contains the service types supported */
 define structure
 1 #CONTROL_SERVICE# union,
     2 Guaranteed type AD_GUARANTEED,
     2 ParamBuffer (1) type PARAM_BUFFER;/* allows for other servi
 ces down * the road */

 define structure
   1 CONTROL_SERVICE,
     /* the length of this entire structure including the following   */
     /* buffer. This length value can be added to the ptr to the      */
     /* structure to get the ptr to the next SERVICE_TYPE structure in*/
     /* the list, until the NumberOfServices count has been exhausted.*/
     2 Length    type ULONG,
     2 Service   type SERVICETYPE,
     2 Overrides type AD_GENERAL_PARAMS,

     /* service specific information ( controlled load has no service */
     /* specific info here )                                          */
     2 *         union,
       3 Guaranteed      type AD_GUARANTEED,
       3 ParamBuffer (1) type PARAM_BUFFER;
         /* allows for other services down the road */
 define alias LPCONTROL_SERVICE handle CONTROL_SERVICE;

 /* This structure defines the information which is carried in the    */
 /* Rsvp Adspec.  This Rsvp object typically indicates which service  */
 /* types are available ( Controlled Load and/or Guaranteed Service ),*/
 /* if a non-Rsvp hop has been encountered by the Path message, and   */
 /* the minumum MTU along the path. The services array indicates      */
 /* which services are supported                                      */
 define structure
   1 RSVP_ADSPEC,
     2 ObjectHdr        type QOS_OBJECT_HDR,
     2 GeneralParams    type AD_GENERAL_PARAMS,
       /* contains the general characterization paramters */
     2 NumberOfServices type ULONG,
       /* count of the number of services */
     2 Services (1) type CONTROL_SERVICE;
       /* a list of the services supported/requested */
 define alias LPRSVP_ADSPEC handle RSVP_ADSPEC;


 /* Opcode for the SIO_CHK_QOS ioctl                                  */
 /* (specific for the Microsoft QOS Service Provider                  */
 /*                                                                   */
 /* Bascially:                                                        */
 /*                                                                   */
 /* SIO_CHK_QOS = _WSAIORW(IOC_VENDOR,1)                              */
 /*             = mIOC_IN | mIOC_OUT | mIOC_VENDOR | mCOMPANY |       */
 /*               ioctl_code                                          */
 /*    where                                                          */
 /*         mIOC_IN     = 0x80000000                                  */
 /*         mIOC_OUT    = 0x40000000                                  */
 /*         mIOC_VENDOR = 0x04000000                                  */
 /*         mCOMPANY    = 0x18000000                                  */
 /*         ioctl_code  = 0x00000001                                  */
 /*                                                                   */
 /* See WSAIoctl man page for details.                                */


 dcl mIOC_IN     type ULONG value('80000000'xu);
 dcl mIOC_OUT    type ULONG value('40000000'xu);
 dcl mIOC_VENDOR type ULONG value('04000000'xu);
 dcl mCOMPANY    type ULONG value('18000000'xu);
 dcl ioctl_code  type ULONG value('00000001'xu);

 dcl SIO_CHK_QOS type ULONG
     value (ior (mIOC_IN, mIOC_OUT, mIOC_VENDOR, mCOMPANY, ioctl_code));


 /* The following may be specified in the input buffer */
 /* of the SIO_CHK_IOCTL ioctl call                    */

 dcl QOSSPBASE                    fixed bin(31) value(50000);

 dcl ALLOWED_TO_SEND_DATA         fixed bin(31) value(QOSSPBASE+1);
     /* query the SBM/BEST_EFFORT limit                          */
     /*   -- result is based on a comparison between             */
     /*      the user-specified token rate and the               */
     /*      SBM/BEST_EFFORT bandwidth parameters in             */
     /*      the system                                          */
     /*   -- result is a YES(1) or NO(0) answer                  */
     /*      in the output buffer                                */

 dcl ABLE_TO_RECV_RSVP            fixed bin(31) value(QOSSPBASE+2);
     /* query the SBM/BEST_EFFORT limit                          */
     /*   -- result is based on a comparison between             */
     /*      the user-specified token rate and the               */
     /*      SBM/BEST_EFFORT bandwidth parameters in             */
     /*      the system                                          */
     /*   -- result is a YES(1) or NO(0) answer                  */
     /*      in the output buffer                                */

 dcl LINE_RATE                    fixed bin(31) value(QOSSPBASE+3);
     /* query the interface capacity                             */
     /*   -- result returned in the output buffer                */
     /*      in kilo-bits per second                             */

 dcl LOCAL_TRAFFIC_CONTROL        fixed bin(31) value(QOSSPBASE+4);
     /* check if Kernel Traffic Control is available or not      */
     /*   -- 0 if not available                                  */
     /*   -- 1 if avaiable                                       */
     /*   -- INFO_NOT_AVAILABLE if there is no way to check      */

 dcl LOCAL_QOSABILITY             fixed bin(31) value(QOSSPBASE+5);
     /* The followings are for                                   */
     /* the Discovery of Local QOSability                        */
     /*   -- 0 if no local QOS support                           */
     /*   -- 1 if local QOS support available                    */
     /*   -- INFO_NOT_AVAILABLE if there is no way to check      */

 dcl END_TO_END_QOSABILITY        fixed bin(31) value(QOSSPBASE+6);
     /* The followings are for                                   */
     /* the Discovery of End-to-End QOSability                   */
     /*   -- 0 if no end-to-end QOS support                      */
     /*   -- 1 if end-to-end QOS support available               */
     /*   -- INFO_NOT_AVAILABLE if there is no way to check      */

 dcl INFO_NOT_AVAILABLE           type ULONG value('FFFFFFFF'xu);
     /* used when LINE_RATE is not known */


 dcl QUERY_PS_SIZE                type ULONG value('FFFFFFFF'xu);
     /* used in SIO_GET_QOS to query the */
     /* required buffer size */

 dcl ANY_DEST_ADDR                type ULONG value('FFFFFFFF'xu);
     /* for QOS_OBJECT_DEST_ADDR */


 /* The followings are for Guaranteed_Service+Latency */
 /* -> ISSLOW_Traffic_Class Mapping                   */

 dcl MODERATELY_DELAY_SENSITIVE   type ULONG value('FFFFFFFD'xu);

 dcl HIGHLY_DELAY_SENSITIVE       type ULONG value('FFFFFFFE'xu);

 /* QOSSP Error Code/Value */

 dcl QOSSP_ERR_BASE          fixed bin(31) value(QOSSPBASE+6000);

 /* No error */
 dcl GQOS_NO_ERRORCODE       fixed bin(31) value(0);

 dcl GQOS_NO_ERRORVALUE      fixed bin(31) value(0);

 /* Unknown error */
 dcl GQOS_ERRORCODE_UNKNOWN  fixed bin(31) value('FFFFFFFF'xn);

 dcl GQOS_ERRORVALUE_UNKNOWN fixed bin(31) value('FFFFFFFF'xn);

 /* Admission (Resource) Error */
 dcl GQOS_NET_ADMISSION      fixed bin(31) value(QOSSP_ERR_BASE+100);

 dcl GQOS_OTHER              fixed bin(31) value(QOSSP_ERR_BASE+100+1);
 dcl GQOS_DELAYBND           fixed bin(31) value(QOSSP_ERR_BASE+100+2);
 dcl GQOS_BANDWIDTH          fixed bin(31) value(QOSSP_ERR_BASE+100+3);
 dcl GQOS_MTU                fixed bin(31) value(QOSSP_ERR_BASE+100+4);
 dcl GQOS_FLOW_RATE          fixed bin(31) value(QOSSP_ERR_BASE+100+5);
 dcl GQOS_PEAK_RATE          fixed bin(31) value(QOSSP_ERR_BASE+100+6);

 /* Policy Error */
 dcl GQOS_NET_POLICY         fixed bin(31) value(QOSSP_ERR_BASE+200);

 dcl GQOS_POLICY_ERROR_UNKNOWN
                             fixed bin(31) value(QOSSP_ERR_BASE+200+0);

 dcl GQOS_POLICY_GLOBAL_DEF_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+1);
 dcl GQOS_POLICY_GLOBAL_GRP_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+2);
 dcl GQOS_POLICY_GLOBAL_USER_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+3);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+4);
 dcl GQOS_POLICY_SUBNET_DEF_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+5);
 dcl GQOS_POLICY_SUBNET_GRP_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+6);
 dcl GQOS_POLICY_SUBNET_USER_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+7);
 dcl GQOS_POLICY_SUBNET_UNK_USER_FLOW_COUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+200+8);

 dcl GQOS_POLICY_GLOBAL_DEF_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+9);
 dcl GQOS_POLICY_GLOBAL_GRP_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+10);
 dcl GQOS_POLICY_GLOBAL_USER_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+11);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+12);
 dcl GQOS_POLICY_SUBNET_DEF_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+13);
 dcl GQOS_POLICY_SUBNET_GRP_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+14);
 dcl GQOS_POLICY_SUBNET_USER_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+15);
 dcl GQOS_POLICY_SUBNET_UNK_USER_FLOW_DURATION
                             fixed bin(31) value(QOSSP_ERR_BASE+200+16);

 dcl GQOS_POLICY_GLOBAL_DEF_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+17);
 dcl GQOS_POLICY_GLOBAL_GRP_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+18);
 dcl GQOS_POLICY_GLOBAL_USER_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+19);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+20);
 dcl GQOS_POLICY_SUBNET_DEF_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+21);
 dcl GQOS_POLICY_SUBNET_GRP_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+22);
 dcl GQOS_POLICY_SUBNET_USER_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+23);
 dcl GQOS_POLICY_SUBNET_UNK_USER_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+24);

 dcl GQOS_POLICY_GLOBAL_DEF_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+25);
 dcl GQOS_POLICY_GLOBAL_GRP_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+26);
 dcl GQOS_POLICY_GLOBAL_USER_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+27);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+28);
 dcl GQOS_POLICY_SUBNET_DEF_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+29);
 dcl GQOS_POLICY_SUBNET_GRP_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+30);
 dcl GQOS_POLICY_SUBNET_USER_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+31);
 dcl GQOS_POLICY_SUBNET_UNK_USER_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+32);

 dcl GQOS_POLICY_GLOBAL_DEF_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+33);
 dcl GQOS_POLICY_GLOBAL_GRP_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+34);
 dcl GQOS_POLICY_GLOBAL_USER_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+35);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+36);
 dcl GQOS_POLICY_SUBNET_DEF_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+37);
 dcl GQOS_POLICY_SUBNET_GRP_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+38);
 dcl GQOS_POLICY_SUBNET_USER_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+39);
 dcl GQOS_POLICY_SUBNET_UNK_USER_SUM_FLOW_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+40);

 dcl GQOS_POLICY_GLOBAL_DEF_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+41);
 dcl GQOS_POLICY_GLOBAL_GRP_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+42);
 dcl GQOS_POLICY_GLOBAL_USER_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+43);
 dcl GQOS_POLICY_GLOBAL_UNK_USER_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+44);
 dcl GQOS_POLICY_SUBNET_DEF_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+45);
 dcl GQOS_POLICY_SUBNET_GRP_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+46);
 dcl GQOS_POLICY_SUBNET_USER_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+47);
 dcl GQOS_POLICY_SUBNET_UNK_USER_SUM_PEAK_RATE
                             fixed bin(31) value(QOSSP_ERR_BASE+200+48);

 dcl GQOS_POLICY_UNKNOWN_USER
                             fixed bin(31) value(QOSSP_ERR_BASE+200+49);
 dcl GQOS_POLICY_NO_PRIVILEGES
                             fixed bin(31) value(QOSSP_ERR_BASE+200+50);
 dcl GQOS_POLICY_EXPIRED_USER_TOKEN
                             fixed bin(31) value(QOSSP_ERR_BASE+200+51);
 dcl GQOS_POLICY_NO_RESOURCES
                             fixed bin(31) value(QOSSP_ERR_BASE+200+52);
 dcl GQOS_POLICY_PRE_EMPTED
                             fixed bin(31) value(QOSSP_ERR_BASE+200+53);
 dcl GQOS_POLICY_USER_CHANGED
                             fixed bin(31) value(QOSSP_ERR_BASE+200+54);
 dcl GQOS_POLICY_NO_ACCEPTS  fixed bin(31) value(QOSSP_ERR_BASE+200+55);
 dcl GQOS_POLICY_NO_MEMORY   fixed bin(31) value(QOSSP_ERR_BASE+200+56);
 dcl GQOS_POLICY_CRAZY_FLOWSPEC
                             fixed bin(31) value(QOSSP_ERR_BASE+200+57);

 dcl GQOS_POLICY_ERROR_USERID
                             fixed bin(31) value(QOSSP_ERR_BASE+200+99);

 /* RSVP Error */
 dcl GQOS_RSVP               fixed bin(31) value(QOSSP_ERR_BASE+300);

 dcl GQOS_NO_PATH            fixed bin(31) value(QOSSP_ERR_BASE+300+1);
 dcl GQOS_NO_SENDER          fixed bin(31) value(QOSSP_ERR_BASE+300+2);
 dcl GQOS_BAD_STYLE          fixed bin(31) value(QOSSP_ERR_BASE+300+3);
 dcl GQOS_UNKNOWN_STYLE      fixed bin(31) value(QOSSP_ERR_BASE+300+4);
 dcl GQOS_BAD_DSTPORT        fixed bin(31) value(QOSSP_ERR_BASE+300+5);
 dcl GQOS_BAD_SNDPORT        fixed bin(31) value(QOSSP_ERR_BASE+300+6);
 dcl GQOS_AMBIG_FILTER       fixed bin(31) value(QOSSP_ERR_BASE+300+7);
 dcl GQOS_PREEMPTED          fixed bin(31) value(QOSSP_ERR_BASE+300+8);
 dcl GQOS_UNKN_OBJ_CLASS     fixed bin(31) value(QOSSP_ERR_BASE+300+9);
 dcl GQOS_UNKNOWN_CTYPE      fixed bin(31) value(QOSSP_ERR_BASE+300+10);
 dcl GQOS_INVALID            fixed bin(31) value(QOSSP_ERR_BASE+300+11);

 /* API Error */
 dcl GQOS_API                fixed bin(31) value(QOSSP_ERR_BASE+400);

 dcl GQOS_API_BADSEND        fixed bin(31) value(QOSSP_ERR_BASE+400+1);
     /* Sender addr not my interface */
 dcl GQOS_API_BADRECV        fixed bin(31) value(QOSSP_ERR_BASE+400+2);
     /* Recv addr not my interface */
 dcl GQOS_API_BADSPORT       fixed bin(31) value(QOSSP_ERR_BASE+400+3);
     /* Sport !=0 but Dport == 0 */

 /* TC System Error */
 dcl GQOS_KERNEL_TC_SYS      fixed bin(31) value(QOSSP_ERR_BASE+500);

 dcl GQOS_TC_GENERIC         fixed bin(31) value(QOSSP_ERR_BASE+500+1);
 dcl GQOS_TC_INVALID         fixed bin(31) value(QOSSP_ERR_BASE+500+2);
 dcl GQOS_NO_MEMORY          fixed bin(31) value(QOSSP_ERR_BASE+500+3);
 dcl GQOS_BAD_ADDRESSTYPE    fixed bin(31) value(QOSSP_ERR_BASE+500+4);
 dcl GQOS_BAD_DUPLICATE      fixed bin(31) value(QOSSP_ERR_BASE+500+5);
 dcl GQOS_CONFLICT           fixed bin(31) value(QOSSP_ERR_BASE+500+6);
 dcl GQOS_NOTREADY           fixed bin(31) value(QOSSP_ERR_BASE+500+7);
 dcl GQOS_WOULDBLOCK         fixed bin(31) value(QOSSP_ERR_BASE+500+8);
 dcl GQOS_INCOMPATIBLE       fixed bin(31) value(QOSSP_ERR_BASE+500+9);
 dcl GQOS_BAD_SDMODE         fixed bin(31) value(QOSSP_ERR_BASE+500+10);
 dcl GQOS_BAD_QOSPRIORITY    fixed bin(31) value(QOSSP_ERR_BASE+500+11);
 dcl GQOS_BAD_TRAFFICCLASS   fixed bin(31) value(QOSSP_ERR_BASE+500+12);
 dcl GQOS_NO_SYS_RESOURCES   fixed bin(31) value(QOSSP_ERR_BASE+500+13);

 /* RSVP System Error */
 dcl GQOS_RSVP_SYS           fixed bin(31) value(QOSSP_ERR_BASE+600);

 dcl GQOS_OTHER_SYS          fixed bin(31) value(QOSSP_ERR_BASE+600+1);
 dcl GQOS_MEMORY_SYS         fixed bin(31) value(QOSSP_ERR_BASE+600+2);
 dcl GQOS_API_SYS            fixed bin(31) value(QOSSP_ERR_BASE+600+3);
 dcl GQOS_SETQOS_NO_LOCAL_APPS
                             fixed bin(31) value(QOSSP_ERR_BASE+600+4);

 /* TC Error */
 dcl GQOS_KERNEL_TC          fixed bin(31) value(QOSSP_ERR_BASE+700);

 dcl GQOS_CONFLICT_SERV      fixed bin(31) value(QOSSP_ERR_BASE+700+1);
 dcl GQOS_NO_SERV            fixed bin(31) value(QOSSP_ERR_BASE+700+2);
 dcl GQOS_BAD_FLOWSPEC       fixed bin(31) value(QOSSP_ERR_BASE+700+3);
 dcl GQOS_BAD_TSPEC          fixed bin(31) value(QOSSP_ERR_BASE+700+4);
 dcl GQOS_BAD_ADSPEC         fixed bin(31) value(QOSSP_ERR_BASE+700+5);

 /* GQOS API ERRORS */

 /* WSAIoctl Error */

 dcl GQOS_IOCTL_SYSTEMFAILURE
                             fixed bin(31) value(QOSSP_ERR_BASE+800+1);
 dcl GQOS_IOCTL_NOBYTESRETURNED
                             fixed bin(31) value(QOSSP_ERR_BASE+800+2);
 dcl GQOS_IOCTL_INVALIDSOCKET
                             fixed bin(31) value(QOSSP_ERR_BASE+800+3);

 /* SIO_SET_QOS error */

 dcl GQOS_SETQOS_BADINBUFFER fixed bin(31) value(QOSSP_ERR_BASE+810+1);
 dcl GQOS_SETQOS_BADFLOWSPEC fixed bin(31) value(QOSSP_ERR_BASE+810+2);
 dcl GQOS_SETQOS_COLLISION
                             fixed bin(31) value(QOSSP_ERR_BASE+810+3);
 dcl GQOS_SETQOS_BADPROVSPECBUF
                             fixed bin(31) value(QOSSP_ERR_BASE+810+4);
 dcl GQOS_SETQOS_ILLEGALOP   fixed bin(31) value(QOSSP_ERR_BASE+810+5);
 dcl GQOS_SETQOS_INVALIDADDRESS
                             fixed bin(31) value(QOSSP_ERR_BASE+810+6);
 dcl GQOS_SETQOS_OUTOFMEMORY fixed bin(31) value(QOSSP_ERR_BASE+810+7);
 dcl GQOS_SETQOS_EXCEPTION   fixed bin(31) value(QOSSP_ERR_BASE+810+8);
 dcl GQOS_SETQOS_BADADDRLEN  fixed bin(31) value(QOSSP_ERR_BASE+810+9);
 dcl GQOS_SETQOS_NOSOCKNAME  fixed bin(31) value(QOSSP_ERR_BASE+810+10);
 dcl GQOS_SETQOS_IPTOSFAIL   fixed bin(31) value(QOSSP_ERR_BASE+810+11);
 dcl GQOS_SETQOS_OPENSESSIONFAIL
                             fixed bin(31) value(QOSSP_ERR_BASE+810+12);
 dcl GQOS_SETQOS_SENDFAIL    fixed bin(31) value(QOSSP_ERR_BASE+810+13);
 dcl GQOS_SETQOS_RECVFAIL    fixed bin(31) value(QOSSP_ERR_BASE+810+14);
 dcl GQOS_SETQOS_BADPOLICYOBJECT
                             fixed bin(31) value(QOSSP_ERR_BASE+810+15);
 dcl GQOS_SETQOS_UNKNOWNFILTEROBJ
                             fixed bin(31) value(QOSSP_ERR_BASE+810+16);
 dcl GQOS_SETQOS_BADFILTERTYPE
                             fixed bin(31) value(QOSSP_ERR_BASE+810+17);
 dcl GQOS_SETQOS_BADFILTERCOUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+810+18);
 dcl GQOS_SETQOS_BADOBJLENGTH
                             fixed bin(31) value(QOSSP_ERR_BASE+810+19);
 dcl GQOS_SETQOS_BADFLOWCOUNT
                             fixed bin(31) value(QOSSP_ERR_BASE+810+20);
 dcl GQOS_SETQOS_UNKNOWNPSOBJ
                             fixed bin(31) value(QOSSP_ERR_BASE+810+21);
 dcl GQOS_SETQOS_BADPOLICYOBJ
                             fixed bin(31) value(QOSSP_ERR_BASE+810+22);
 dcl GQOS_SETQOS_BADFLOWDESC fixed bin(31) value(QOSSP_ERR_BASE+810+23);
 dcl GQOS_SETQOS_BADPROVSPECOBJ
                             fixed bin(31) value(QOSSP_ERR_BASE+810+24);
 dcl GQOS_SETQOS_NOLOOPBACK  fixed bin(31) value(QOSSP_ERR_BASE+810+25);
 dcl GQOS_SETQOS_MODENOTSUPPORTED
                             fixed bin(31) value(QOSSP_ERR_BASE+810+26);
 dcl GQOS_SETQOS_MISSINGFLOWDESC
                             fixed bin(31) value(QOSSP_ERR_BASE+810+27);

 /* SIO_GET_QOS error */

 dcl GQOS_GETQOS_BADOUTBUFFER
                             fixed bin(31) value(QOSSP_ERR_BASE+840+1);
 dcl GQOS_GETQOS_SYSTEMFAILURE
                             fixed bin(31) value(QOSSP_ERR_BASE+840+2);
 dcl GQOS_GETQOS_EXCEPTION   fixed bin(31) value(QOSSP_ERR_BASE+840+3);
 dcl GQOS_GETQOS_INTERNALFAILURE
                             fixed bin(31) value(QOSSP_ERR_BASE+840+4);

 /* SIO_CHK_QOS error */

 dcl GQOS_CHKQOS_BADINBUFFER fixed bin(31) value(QOSSP_ERR_BASE+850+1);
 dcl GQOS_CHKQOS_BADOUTBUFFER
                             fixed bin(31) value(QOSSP_ERR_BASE+850+2);
 dcl GQOS_CHKQOS_SYSTEMFAILURE
                             fixed bin(31) value(QOSSP_ERR_BASE+850+3);
 dcl GQOS_CHKQOS_INTERNALFAILURE
                             fixed bin(31) value(QOSSP_ERR_BASE+850+4);
 dcl GQOS_CHKQOS_BADPARAMETER
                             fixed bin(31) value(QOSSP_ERR_BASE+850+5);
 dcl GQOS_CHKQOS_EXCEPTION   fixed bin(31) value(QOSSP_ERR_BASE+850+6);

 %end; /* __QOSSP_H_ */
