 /*********************************************************************/
 /* Module Name: REXX.CPY                                             */
 /*                                                                   */
 /* REXX Common Definitions File                                      */
 /*                                                                   */
 /* Copyright  1989, 1991 IBM Corporation                             */
 /* Licensed Materials - Property of IBM                              */
 /*                                                                   */
 /* *******************************************************************/
 /*                                                                   */
 /*                                                                   */
 /*   define:               To include:                               */
 /*                                                                   */
 /*   INCL_REXXSAA          Complete Rexx support                     */
 /*   INCL_RXSUBCOM         Rexx subcommand handler support           */
 /*   INCL_RXSHV            Rexx shared variable pool support         */
 /*   INCL_RXFUNC           Rexx external function support            */
 /*   INCL_RXSYSEXIT        Rexx system exit support                  */
 /*   INCL_RXMACRO          Rexx macro space support                  */
 /*   INCL_RXARI            Rexx asynchronous Trace/Halt support      */
 /*   INCL_RXQUEUE          Rexx queue interface support              */
 /*   INCL_RXOBJECT         Object REXX extensions                    */
 /*                                                                   */
 /*********************************************************************/

 %dcl REXXSAA_INCLUDED char ext;
 %if REXXSAA_INCLUDED = '' %then %do;
   %REXXSAA_INCLUDED = 'Y';

 %dcl INCL_REXXSAA char ext;
 %dcl INCL_RXSUBCOM char ext;
 %dcl INCL_RXSHV char ext;
 %dcl INCL_RXFUNC char ext;
 %dcl INCL_RXSYSEXIT char ext;
 %dcl INCL_RXMACRO char ext;
 %dcl INCL_RXARI char ext;
 %dcl INCL_RXQUEUE char ext;
 %dcl INCL_RXOBJECT char ext;

 /* Pull in WINBASE.H if necessary */
 %dcl #WINBASE_ char ext;
 %if #WINBASE_ = '' %then %do;
   %include winbase;
 %end;

 %dcl REXXAPI char;
 %REXXAPI = 'options (byvalue linkage(system) nodescriptor)';

 define alias APIRET type ULONG;
 define alias PID    type DWORD;
 define alias TID    type DWORD;
 define alias PFN    ptr; /* entryaddr(func) */

 /*********************************************************************/
 /*                                                                   */
 /*                            Common                                 */
 /*                                                                   */
 /*********************************************************************/

 %if INCL_REXXSAA ^= '' %then %do;
   %INCL_RXSUBCOM  = 'Y';
   %INCL_RXSHV     = 'Y';
   %INCL_RXFUNC    = 'Y';
   %INCL_RXSYSEXIT = 'Y';
   %INCL_RXMACRO   = 'Y';
   %INCL_RXARI     = 'Y';
   %INCL_RXQUEUE   = 'Y';
 %end;
 /* INCL_REXXSAA */


   /***    Structure for external interface string (RXSTRING) */

   define structure
     1 RXSTRING,
       2 strlength type ULONG,  /* length  of string */
       2 strptr    type PCH;    /* pointer to string */

   define alias PRXSTRING handle RXSTRING; /* pointer to a RXSTRING   */

   dcl RXAUTOBUFLEN fixed bin(31) value(256);

   /***    Structure for system exit block (RXSYSEXIT) 32-bit */

   define structure
     1 RXSYSEXIT,
       2 sysexit_name type PSZ,  /* subcom environment for sysexit */
       2 sysexit_code type LONG; /* sysexit function code */

   define alias PRXSYSEXIT handle RXSYSEXIT; /* pointer to a RXSYSEXIT*/

   /*** Call type codes for use on interpreter startup                */
   dcl RXCOMMAND    fixed bin(31) value(0), /* called as Command      */
       RXSUBROUTINE fixed bin(31) value(1), /* called as Subroutine   */
       RXFUNCTION   fixed bin(31) value(2); /* called as Function     */

 /***    Subcommand Interface defines*/
 %if INCL_RXSUBCOM ^= '' %then %do;
   /***    Drop Authority for RXSUBCOM interface */

   dcl RXSUBCOM_DROPPABLE fixed bin(15)
                        value('00'xn), /* handler to be dropped by all*/
       RXSUBCOM_NONDROP fixed bin(15)
                        value('01'xn); /* process with same PID as the*/
                                       /* registrant may drop environ */

   /***    Return Codes from RXSUBCOM interface */

   dcl RXSUBCOM_ISREG   fixed bin(15)
                        value('01'xn), /* Subcommand is registered    */
       RXSUBCOM_ERROR   fixed bin(15)
                        value('01'xn), /* Subcommand Ended in Error   */
       RXSUBCOM_FAILURE fixed bin(15)
                        value('02'xn), /* Subcommand Ended in Failure */
       RXSUBCOM_BADENTRY fixed bin(31)
                        value(1001),   /* Invalid Entry Conditions    */
       RXSUBCOM_NOEMEM  fixed bin(31)
                        value(1002),   /* Insuff stor to complete req */
       RXSUBCOM_BADTYPE fixed bin(31)
                        value(1003),   /* Bad registration type.      */
       RXSUBCOM_NOTINIT fixed bin(31)
                        value(1004),   /* API system not initialized. */
       RXSUBCOM_OK      fixed bin(31)
                        value(0),      /* Function Complete           */
       RXSUBCOM_DUP     fixed bin(31)  /* Duplicate Environment Name- */
                        value(10),     /* but Registration Completed  */
       RXSUBCOM_MAXREG  fixed bin(31)
                        value(20),     /* Cannot register more hdlrs  */
       RXSUBCOM_NOTREG  fixed bin(31)
                        value(30),     /* Name Not Registered         */
       RXSUBCOM_NOCANDROP fixed bin(31)
                         value(40),    /* Name not droppable          */
       RXSUBCOM_LOADERR fixed bin(31)
                        value(50),     /* Could not load function     */
       RXSUBCOM_NOPROC  fixed bin(31)
                        value(127);    /* RXSUBCOM routine - not found*/
 %end; /* INCL_RXSUBCOM for defines */

 /***    Shared Variable Pool Interface defines */
 %if INCL_RXSHV ^= '' %then %do;
   /***    Function Codes for Variable Pool Interface (shvcode)   */

   dcl ( RXSHV_SET   value('00'xn) /* Set var from given value     */
       , RXSHV_FETCH value('01'xn) /* Copy value of var to buffer  */
       , RXSHV_DROPV value('02'xn) /* Drop variable                */
       , RXSHV_SYSET value('03'xn) /* Symbolic name Set variable   */
       , RXSHV_SYFET value('04'xn) /* Symbolic name Fetch variable */
       , RXSHV_SYDRO value('05'xn) /* Symbolic name Drop variable  */
       , RXSHV_NEXTV value('06'xn) /* Fetch "next" variable        */
       , RXSHV_PRIV  value('07'xn) /* Fetch private information    */
       , RXSHV_EXIT  value('08'xn) /* Set function exit value      */
       ) type UCHAR;

   /***    Return Codes for Variable Pool Interface */

   dcl RXSHV_NOAVL fixed bin(31) value('90'xn);
                                       /* Interface not available */

   /***    Return Code Flags for Variable Pool Interface (shvret) */

   dcl ( RXSHV_OK    value('00'xn) /* Execution was OK             */
       , RXSHV_NEWV  value('01'xn) /* Variable did not exist       */
       , RXSHV_LVAR  value('02'xn) /* Last var trans via SHVNEXTV  */
       , RXSHV_TRUNC value('04'xn) /* Truncation occurred-Fetch    */
       , RXSHV_BADN  value('08'xn) /* Invalid variable name        */
       , RXSHV_MEMFL value('10'xn) /* Out of memory failure        */
       , RXSHV_BADF  value('80'xn) /* Invalid funct code (shvcode) */
       ) type UCHAR;

   /***    Structure of Shared Variable Request Block (SHVBLOCK) */

   define structure
     1 SHVBLOCK,
          2 shvnext     ptr,           /* pointer to the next block   */
          2 shvname     type RXSTRING, /* Pointer to the name buffer  */
          2 shvvalue    type RXSTRING, /* Pointer to the value buffer */
          2 shvnamelen  type ULONG,    /* Length of the name value    */
          2 shvvaluelen type ULONG,    /* Length of the fetch value   */
          2 shvcode     type UCHAR,    /* Function code for this block*/
          2 shvret      type UCHAR;    /* Individual Return Code Flags*/

   define alias PSHVBLOCK handle SHVBLOCK;
 %end; /* INCL_RXSHV for defines */

 /***    External Function Interface */
 %if INCL_RXFUNC ^= '' %then %do;
   /***    Registration Type Identifiers for Available Function Table */
   dcl RXFUNC_DYNALINK  fixed bin(31)
                        value(1), /* Function Available in DLL        */
       RXFUNC_CALLENTRY fixed bin(31)
                        value(2); /* Registered as mem entry point    */


   /***    Return Codes from RxFunction interface */

   dcl ( RXFUNC_OK        value( 0)   /* REXX-API Call Successful     */
       , RXFUNC_DEFINED   value(10)   /* Function Defined in AFT      */
       , RXFUNC_NOMEM     value(20)   /* Not Enough Mem to Add        */
       , RXFUNC_NOTREG    value(30)   /* Funct Not Registered in AFT  */
       , RXFUNC_MODNOTFND value(40)   /* Funct Dll Module Not Found   */
       , RXFUNC_ENTNOTFND value(50)   /* Funct Entry Point Not Found  */
       , RXFUNC_NOTINIT   value(60)   /* API not initialized          */
       , RXFUNC_BADTYPE   value(70)   /* Bad function type            */
       , RXFUNC_NOEMEM    value(1002) /* Failure in memory manager    */
       ) fixed bin (31);
 %end;  /* INCL_RXFUNC for defines */

 /***   System Exits defines */
 %if INCL_RXSYSEXIT ^= '' %then %do;
   /***    Drop Authority for Rexx Exit interface */

   dcl RXEXIT_DROPPABLE fixed bin(15)
                         value('00'xn),/* handler to be dropped by all*/
       RXEXIT_NONDROP   fixed bin(15)
                         value('01'xn);/* process with same PID as the*/
                                       /* registrant may drop environ */

   /***    Exit return actions */

   dcl RXEXIT_HANDLED     fixed bin(31)
                          value(0),    /* Exit handled exit event     */
       RXEXIT_NOT_HANDLED fixed bin(31)
                          value(1),    /* Exit passes on exit event   */
       RXEXIT_RAISE_ERROR fixed bin(31)
                          value(-1);   /* Exit handler error occurred */

   /***    Return Codes from RXEXIT interface */

   dcl RXEXIT_ISREG     fixed bin(15)
                        value('01'xn), /* Exit is registered          */
       RXEXIT_ERROR     fixed bin(15)
                        value('01'xn), /* Exit Ended in Error         */
       RXEXIT_FAILURE   fixed bin(15)
                        value('02'xn), /* Exit Ended in Failure       */
       RXEXIT_BADENTRY  fixed bin(31)
                        value(1001),   /* Invalid Entry Conditions    */
       RXEXIT_NOEMEM    fixed bin(31)
                        value(1002),   /* Insuff stor to complete req */
       RXEXIT_BADTYPE   fixed bin(31)
                        value(1003),   /* Bad registration type       */
       RXEXIT_NOTINIT   fixed bin(31)
                        value(1004),   /* API system not initialized  */
       RXEXIT_OK        fixed bin(31)
                        value(0),      /* Function Complete           */
       RXEXIT_DUP       fixed bin(31)  /* Duplicate Exit Name         */
                        value(10),     /* but Registration Completed  */
       RXEXIT_MAXREG    fixed bin(31)  /* Cannot register more        */
                        value(20),     /* handlers                    */
       RXEXIT_NOTREG    fixed bin(31)
                        value(30),     /* Name Not Registered         */
       RXEXIT_NOCANDROP fixed bin(31)
                        value(40),     /* Name not droppable          */
       RXEXIT_LOADERR   fixed bin(31)
                        value(50),     /* Could not load function     */
       RXEXIT_NOPROC    fixed bin(31)
                        value(127);    /* RXEXIT routine - not found  */


   /* System Exit function and sub-function definitions */

   dcl ( RXENDLST   value(0)  /* End of exit list           */
       , RXFNC      value(2)  /* Process external functions */
         , RXFNCCAL value(1)  /* subcode value              */
       , RXCMD      value(3)  /* Process host commands      */
         , RXCMDHST value(1)  /* subcode value              */
       , RXMSQ      value(4)  /* Manipulate queue.          */
         , RXMSQPLL value(1)  /* Pull a line from queue     */
         , RXMSQPSH value(2)  /* Place a line on queue      */
         , RXMSQSIZ value(3)  /* Return no of lines on queue*/
         , RXMSQNAM value(20) /* Set active queue name      */
       , RXSIO      value(5)  /* Session I/O.               */
         , RXSIOSAY value(1)  /* SAY a line to STDOUT       */
         , RXSIOTRC value(2)  /* Trace output               */
         , RXSIOTRD value(3)  /* Read from char stream      */
         , RXSIODTR value(4)  /* DEBUG read from char stream*/
         , RXSIOTLL value(5)  /* Return linelength(N/A OS/2)*/
       , RXHLT      value(7)  /* Halt processing.           */
         , RXHLTCLR value(1)  /* Clear HALT indicator       */
         , RXHLTTST value(2)  /* Test HALT indicator        */
       , RXTRC      value(8)  /* Test ext trace indicator.  */
         , RXTRCTST value(1)  /* subcode value.             */
       , RXINI      value(9)  /* Initialization processing. */
         , RXINIEXT value(1)  /* subcode value.             */
       , RXTER      value(10) /* Termination processing.    */
         , RXTEREXT value(1)  /* subcode value.             */
 %if INCL_RXOBJECT ^= '' %then %do;
       , RXDBG      value(11) /* Test ext trace indic before instr */
         , RXDBGTST        value(1) /* subcode value.             */
         /* return codes for the debug exit                        */
         , RXDBGOFF        value(0) /* set trace off              */
         , RXDBGSTEPIN     value(1) /* set trace on (normal)      */
         , RXDBGSTEPOVER   value(2) /* set trace on but step over */
         , RXDBGSTEPOUT    value(3) /* set step out               */
         , RXDBGENDSTEP    value(4) /* leaving a subroutine       */
         , RXDBGENTERSUB   value(5) /* entering a subroutine      */
         , RXDBGLEAVESUB   value(6) /* leaving a subroutine       */
         , RXDBGLOCATELINE value(7) /* locate current line        */
         , RXDBGSTEPAGAIN  value(8) /* repeat debug pause         */
       , RXNOOFEXITS value(12)  /* 1 + largest exit number*/
 %end; %else %do;
       , RXNOOFEXITS value(11)  /* 1 + largest exit number*/
 %end; /* INCL_RXOBJECT */
       ) fixed bin (31);

   define alias PEXIT type PUCHAR;
 %end; /* INCL_RXSYSEXIT for defines */

 /***    Asynchronous Request Interface defines */
 %if INCL_RXARI ^= '' %then %do;
   /***    Return Codes from Asynchronous Request interface */

   dcl RXARI_OK        fixed bin(31)
                       value(0),   /* Interface completed             */
       RXARI_NOT_FOUND fixed bin(31)
                       value(1),   /* Target program not found        */
       RXARI_PROCESSING_ERROR fixed bin(31)
                       value(2);   /* Error processing request        */
 %end; /* INCL_RXARI for defines */

 /***    Macro Space Interface defines */
 %if INCL_RXMACRO ^= '' %then %do;

   /***    Registration Search Order Flags */

   dcl RXMACRO_SEARCH_BEFORE fixed bin(31)
                             value(1), /* Beginning of search order  */
       RXMACRO_SEARCH_AFTER  fixed bin(31)
                             value(2); /* End of search order        */

   /***    Return Codes from RxMacroSpace interface */

   dcl RXMACRO_OK                 fixed bin(31)
                             value(0), /* Macro interface completed   */
       RXMACRO_NO_STORAGE         fixed bin(31)
                             value(1), /* Not Enough Storage Available*/
       RXMACRO_NOT_FOUND          fixed bin(31)
                             value(2), /* Requested function not found*/
       RXMACRO_EXTENSION_REQUIRED fixed bin(31)
                             value(3), /* File ext required for save  */
       RXMACRO_ALREADY_EXISTS     fixed bin(31)
                             value(4), /* Macro functions exist       */
       RXMACRO_FILE_ERROR         fixed bin(31)
                             value(5), /* File I/O error in save/load */
       RXMACRO_SIGNATURE_ERROR    fixed bin(31)
                             value(6), /* Incorrect format for load   */
       RXMACRO_SOURCE_NOT_FOUND   fixed bin(31)
                             value(7), /* Requested cannot be found   */
       RXMACRO_INVALID_POSITION   fixed bin(31)
                             value(8), /* Invalid search order pos    */
       RXMACRO_NOT_INIT           fixed bin(31)
                             value(9); /* API not initialized         */
 %end; /* INCL_RXMACRO for defines */

 /***    Main Entry Point to the REXXSAA Interpreter */

   dcl RexxStart entry
          (type LONG byvalue,         /* Num of args passed to rexx   */
           (*) type RXSTRING byaddr,  /* Array of args passed to rexx */
           char (*) varyingz byaddr,  /* [d:][path] filename[.ext]    */
           (2) type RXSTRING byaddr   /* Loc. of rexx proc in memory  */
               optional,
           char (*) varyingz byaddr,  /* ASCIIZ initial environment   */
           type LONG byvalue,         /* type (command,subrtn,funct)  */
           (*) type RXSYSEXIT byaddr  /* SysExit env names & codes    */
               optional,
           type SHORT byaddr,         /* Ret code if numeric          */
           type RXSTRING byaddr)      /* Ret value from the rexx proc */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxStart');

   dcl RexxWaitForTermination entry ()
       REXXAPI ext ('RexxWaitForTermination');

   dcl RexxDidRexxTerminate entry ()
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxDidRexxTerminate');

   dcl RexxSetProcessMessages entry (type BOOL)
       returns( byvalue optional type BOOL )
       REXXAPI ext ('RexxSetProcessMessages');


 /***    Subcommand Interface */
 %if INCL_RXSUBCOM ^= '' %then %do;
   /***   RexxRegisterSubcomDll -- Register a DLL entry point         */
   /***   as a Subcommand handler */

   dcl RexxRegisterSubcomDll entry
          (char (*) varyingz byaddr,  /* Name of subcom handler       */
           char (*) varyingz byaddr,  /* Name of DLL                  */
           char (*) varyingz byaddr,  /* Name of procedure in DLL     */
           ptr byvalue,               /* User area                    */
           TYPE ULONG byvalue)        /* drop authority               */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterSubcomDll');

   /***   RexxRegisterSubcomExe -- Register an EXE entry point        */
   /***   as a Subcommand handler */

   dcl RexxRegisterSubcomExe entry
          (char (*) varyingz byaddr,  /* Name of subcom handler       */
           type PFN,                  /* address of handler in EXE    */
           ptr byvalue)               /* User area                    */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterSubcomExe');

   /***    RexxQuerySubcom - Query an environment for Existence */

   dcl RexxQuerySubcom entry
          (char (*) varyingz byaddr,  /* Name of the Environment      */
           char (*) varyingz byaddr optional, /* DLL Module Name      */
           type USHORT byaddr,        /* Store for existance code     */
           PTR byvalue)               /* User area                    */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxQuerySubcom');

   /***    RexxDeregisterSubcom - Drop registration of a Subcommand  */
   /***    environment */

   dcl RexxDeregisterSubcom entry
          (char (*) varyingz byaddr,  /* Name of the Environment      */
           char (*) varyingz byaddr optional) /* Name of DLL          */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxDeregisterSubcom');

 %end; /* INCL_RXSUBCOM */


 /***    Shared Variable Pool Interface */
 %if INCL_RXSHV ^= '' %then %do;
   /***    RexxVariablePool - Request Variable Pool Service */

   dcl RexxVariablePool entry
          (type SHVBLOCK byaddr)           /* list of SHVBLOCKs */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxVariablePool');
 %end; /* INCL_RXSHV */


 /***    External Function Interface */
 %if INCL_RXFUNC ^= '' %then %do;
   /***    RexxRegisterFunctionDll - Register a function in the AFT   */

   dcl RexxRegisterFunctionDll entry
          (char (*) varyingz byaddr,  /* Name of function to add      */
           char (*) varyingz byaddr,  /* Dll file name (if in dll)    */
           char (*) varyingz byaddr)  /* Entry point name             */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterFunctionDll');


   /***    RexxRegisterFunctionExe - Register a function in the AFT   */

   dcl RexxRegisterFunctionExe entry
          (char (*) varyingz byaddr,  /* Name of function to add      */
           type PFN)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterFunctionExe');


   /***    RexxDeregisterFunction - Delete a function from the AFT    */

   dcl RexxDeregisterFunction entry
          (char (*) varyingz byaddr)  /* Name of function to remove   */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxDeregisterFunction');


   /***    RexxQueryFunction - Scan the AFT for a function */

   dcl RexxQueryFunction entry
          (char (*) varyingz byaddr)  /* Name of function to find     */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxQueryFunction');
 %end; /* INCL_RXFUNC */


 /***   System Exits */
 %if INCL_RXSYSEXIT ^= '' %then %do;
   /***    Subfunction RXFNCCAL - External Function Calls */

 define structure
   1 RXFNC_FLAGS, /* flags reversed - different bit ordering */
     3 *           BIT (5) UNALIGNED,
     3 rxffsub     BIT UNALIGNED,    /* Called as a subroutine        */
     3 rxffnfnd    BIT UNALIGNED,    /* Function not found.           */
     3 rxfferr     BIT UNALIGNED,    /* Invalid call to routine.      */
     3 *           BIT (24) UNALIGNED;
 define alias PRXFNC_FLAGS handle RXFNC_FLAGS;

 define structure
   1 RXFNCCAL_PARM,
     3 rxfnc_flags type RXFNC_FLAGS, /* function flags                */
     3 rxfnc_name  type PUCHAR,      /* Pointer to function name.     */
     3 rxfnc_namel type USHORT,      /* Length of function name.      */
     3 rxfnc_que   type PUCHAR,      /* Current queue name.           */
     3 rxfnc_quel  type USHORT,      /* Length of queue name.         */
     3 rxfnc_argc  type USHORT,      /* Number of args in list.       */
     3 rxfnc_argv  type PRXSTRING,   /* Pointer to argument list.     */
     3 rxfnc_retc  type RXSTRING;    /* Return value.                 */
 define alias PRXFNCCAL_PARM handle RXFNCCAL_PARM;

 /***    Subfunction RXCMDHST -- Process Host Commands       */

 define structure
   1 RXCMD_FLAGS,  /* flags reversed - different bit ordering */
     3 *           BIT(6) UNALIGNED,
     3 rxfcerr     BIT UNALIGNED,        /* Command ERROR occurred.   */
     3 rxfcfail    BIT UNALIGNED,        /* Command failed.           */
     3 *           BIT(24) UNALIGNED;
 define alias PRXCMD_FLAGS handle RXCMD_FLAGS;


 define structure
   1 RXCMDHST_PARM,
     3 rxcmd_flags    type RXCMD_FLAGS,  /* error/failure flags       */
     3 rxcmd_address  type PUCHAR,       /* Pointer to address name   */
     3 rxcmd_addressl type SHORT,        /* Length of address name    */
     3 rxcmd_dll      type PUCHAR,       /* dll name for command      */
     3 rxcmd_dll_len  type USHORT,       /* Length of dll name        */
     3 rxcmd_command  type RXSTRING,     /* The command string        */
     3 rxcmd_retc     type RXSTRING;     /* Pointer to return buffer  */
 define alias PRXCMDHST_PARM handle RXCMDHST_PARM;


 /***     Subfunction RXMSQPLL -- Pull Entry from Queue */

 define structure
   1 RXMSQPLL_PARM,
     3 rxmsq_retc type RXSTRING;        /* Pointer to dequeued entry  */
                                        /* buffer. User allocated.    */
 define alias PRXMSQPLL_PARM handle RXMSQPLL_PARM;


 /***    Subfunction RXMSQPSH -- Push Entry on Queue */

 define structure
   1 RXMSQ_FLAGS, /* flags reversed - different bit ordering */
     3 *          BIT(7) UNALIGNED,
     3 rxfmlifo   BIT UNALIGNED,        /* Stack entry LIFO if set    */
     3 *          BIT(24) UNALIGNED;
 define alias PRXMSQ_FLAGS handle RXMSQ_FLAGS;


 define structure
   1 RXMSQPSH_PARM,
     3 rxmsq_flags type RXMSQ_FLAGS,    /* LIFO/FIFO flag             */
     3 rxmsq_value type RXSTRING;       /* The entry to be pushed.    */
 define alias PRXMSQPSH_PARM handle RXMSQPSH_PARM;


 /***    Subfunction RXMSQSIZ -- Return the Current Queue Size */

 define structure
   1 RXMSQSIZ_PARM,
     3 rxmsq_size type ULONG;           /* Number of Lines in Queue   */
 define alias PRXMSQSIZ_PARM handle RXMSQSIZ_PARM;


 /***    Subfunction RXMSQNAM -- Set Current Queue Name */

 define structure
   1 RXMSQNAM_PARM,
     3 rxmsq_name type RXSTRING;   /* RXSTRING containing queue name. */
 define alias PRXMSQNAM_PARM handle RXMSQNAM_PARM;


 /***    Subfunction RXSIOSAY -- Perform SAY Clause */

 define structure
   1 RXSIOSAY_PARM,
     3 rxsio_string type RXSTRING;      /* String to display.         */
 define alias PRXSIOSAY_PARM handle RXSIOSAY_PARM;

 /***    Subfunction RXSIOTRC -- Write Trace Output */

 define structure
   1 RXSIOTRC_PARM,
     3 rxsio_string type RXSTRING;      /* Trace line to display.     */
 define alias PRXSIOTRC_PARM handle RXSIOTRC_PARM;

 /***    Subfunction RXSIOTRD -- Read Input from the Terminal */

 define structure
   1 RXSIOTRD_PARM,
     3 rxsiotrd_retc type RXSTRING;     /* RXSTRING for output.       */
 define alias PRXSIOTRD_PARM handle RXSIOTRD_PARM;

 /***    Subfunction RXSIODTR -- Read Debug Input from the Terminal */

 define structure
   1 RXSIODTR_PARM,
     3 rxsiodtr_retc type RXSTRING;     /* RXSTRING for output.       */
 define alias PRXSIODTR_PARM handle RXSIODTR_PARM;


 /***    Subfunction RXHSTTST -- Test for HALT Condition */

 define structure
   1 RXHLT_FLAGS, /* flags reversed - different bit ordering */
     3 *          BIT(7) UNALIGNED,
     3 rxfhhalt   BIT UNALIGNED,        /* Set if HALT occurred.      */
     3 *          BIT(24) UNALIGNED;
 define alias PRXHLT_FLAGS handle RXHLT_FLAGS;

 define structure
   1 RXHLTTST_PARM,
     3 rxhlt_flags type RXHLT_FLAGS;    /* Set if HALT occurred       */
 define alias PRXHLTTST_PARM handle RXHLTTST_PARM;


 /***    Subfunction RXTRCTST -- Test for TRACE Condition */

 define structure
   1 RXTRC_FLAGS, /* flags reversed - different bit ordering */
     3 *          BIT(7) UNALIGNED,
     3 rxftrace   BIT UNALIGNED,        /* Set to run external trace */
     3 *          BIT(24) UNALIGNED;
 define alias PRXTRC_FLAGS handle RXTRC_FLAGS;

 define structure
   1 RXTRCTST_PARM,
     3 rxtrc_flags type RXTRC_FLAGS;    /* Set to run external trace */
 define alias PRXTRCTST_PARM handle RXTRCTST_PARM;

 %if INCL_RXOBJECT ^= '' %then %do;
 define structure
   1 RXDBG_FLAGS,
     3 rxftrace  fixed bin (31);        /* set to run external trace */
 define alias PRXDBG_FLAGS handle RXDBG_FLAGS;

 define structure
   1 RXDBGTST_PARM,
     3 rxdbg_flags    type RXDBG_FLAGS,  /* run external trace before */
     3 rxdbg_line     type ULONG,
     3 rxdbg_filename type RXSTRING,
     3 rxdbg_routine  type RXSTRING;
 define alias PRXDBGTST_PARM handle RXDBGTST_PARM;

 %end; /* INCL_RXOBJECT */

 /***      RexxRegisterExitDll - Register a system exit. */
   dcl RexxRegisterExitDll entry
          (char (*) varyingz byaddr,    /* Name of the exit handler   */
           char (*) varyingz byaddr,    /* Name of the DLL            */
           char (*) varyingz byaddr,    /* Name of the procedure      */
           ptr byvalue,                 /* User area                  */
           type ULONG)                  /* Drop authority             */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterExitDll');


 /***      RexxRegisterExitExe - Register a system exit. */

   dcl RexxRegisterExitExe entry
          (char (*) varyingz byaddr,    /* Name of the exit handler   */
           type PFN,                    /* Address of exit handler    */
           ptr byvalue)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxRegisterExitExe');


 /***    RexxDeregisterExit - Drop registration of a system exit. */

   dcl RexxDeregisterExit entry
          (char (*) varyingz byaddr,    /* Exit name                  */
           char (*) varyingz byaddr optional)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxDeregisterExit');


 /***    RexxQueryExit - Query an exit for existance. */

   dcl RexxQueryExit entry
          (char (*) varyingz byaddr,    /* Exit name                  */
           char (*) varyingz byaddr optional, /* DLL Module name.     */
           type USHORT byvalue,         /* Existence flag.            */
           ptr byaddr)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxQueryExit');

 %end; /* INCL_RXSYSEXIT */


 /***    Asynchronous Request Interface */
 %if INCL_RXARI ^= '' %then %do;

 /***    RexxSetHalt - Request Program Halt */

   dcl RexxSetHalt entry
          (type PID,                   /* Process Id                  */
           type TID)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxSetHalt');


 /***    RexxSetTrace - Request Program Trace */

   dcl RexxSetTrace entry
          (type PID,                   /* Process Id                  */
           type TID)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxSetTrace');


 /***    RexxResetTrace - Turn Off Program Trace */

   dcl RexxResetTrace entry
          (type PID,                   /* Process Id                  */
           type TID)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxResetTrace');

 %end; /* INCL_RXARI */


 /***    Macro Space Interface */
 %if INCL_RXMACRO ^= '' %then %do;

 /***    RexxAddMacro - Register a function in the Macro Space        */

   dcl RexxAddMacro entry
          (char (*) varyingz byaddr,   /* Function to add or change   */
           char (*) varyingz byaddr,   /* Name of file to get function*/
           type ULONG)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxAddMacro');

 /***    RexxDropMacro - Remove a function from the Macro Space       */

   dcl RexxDropMacro entry
          (char (*) varyingz byaddr)   /* Name of function to remove  */
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxDropMacro');

 /***    RexxSaveMacroSpace - Save Macro Space functions to a file    */

   dcl RexxSaveMacroSpace entry
          (type ULONG,                 /* Argument count (0==save all)*/
           POINTER,                    /* List of funct names to save */
           char (*) varyingz byaddr)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxSaveMacroSpace');

 /***    RexxLoadMacroSpace - Load Macro Space functions from a file  */

   dcl RexxLoadMacroSpace entry
          (type ULONG,                 /* Argument count (0==load all)*/
           POINTER,                    /* List of funct names to load */
           char (*) varyingz byaddr)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxLoadMacroSpace');

 /***    RexxQueryMacro - Find a function's search-order position     */

   dcl RexxQueryMacro entry
          (char (*) varyingz byaddr,   /* Function to search for      */
           type PUSHORT)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxQueryMacro');

 /***    RexxReorderMacro - Change a function's search-order position */

   dcl RexxReorderMacro entry
          (char (*) varyingz byaddr,   /* Name of funct change order  */
           type ULONG)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxReorderMacro');

 /***    RexxClearMacroSpace - Remove all functions from a MacroSpace */

   dcl RexxClearMacroSpace entry
          (/* No Arguments */)
       returns( byvalue optional type APIRET )
       REXXAPI ext ('RexxClearMacroSpace');

 %end; /* INCL_RXMACRO */

 /***    Queueing Services */
 %if INCL_RXQUEUE ^= '' %then %do;

 /***    Request flags for External Data Queue access */

 dcl ( RXQUEUE_FIFO      value (0) /* Access queue first-in-first-out */
     , RXQUEUE_LIFO      value (1) /* Access queue last-in-first-out  */
     ) fixed bin (31);

 dcl ( RXQUEUE_NOWAIT    value (0) /* Wait for data if queue empty    */
     , RXQUEUE_WAIT      value (1) /* Don't wait on an empty queue    */
     ) fixed bin (31);

 /***    Return Codes from RxQueue interface */

 dcl ( RXQUEUE_OK         value (   0) /* Successful return           */
     , RXQUEUE_NOTINIT    value (1000) /* Queues not initialized      */

     , RXQUEUE_STORAGE    value (   1) /* Ret info buf not big enough */
     , RXQUEUE_SIZE       value (   2) /* Data size > 64K-64          */
     , RXQUEUE_DUP        value (   3) /* Attempt-duplicate queue name*/
     , RXQUEUE_NOEMEM     value (1002) /* failure in memory manager   */
     , RXQUEUE_BADQNAME   value (   5) /* Not a valid queue name      */
     , RXQUEUE_PRIORITY   value (   6) /* Not accessed as LIFO|FIFO   */
     , RXQUEUE_BADWAITFLAG value(   7) /* Not accessed as WAIT|NOWAIT */
     , RXQUEUE_EMPTY      value (   8) /* No data in queue            */
     , RXQUEUE_NOTREG     value (   9) /* Queue does not exist        */
     , RXQUEUE_ACCESS     value (  10) /* Queue busy and wait active  */
     , RXQUEUE_MAXREG     value (  11) /* No memory to create a queue */
     , RXQUEUE_MEMFAIL    value (  12) /* Failure in memory management*/
     ) type ULONG;

 /***    RexxCreateQueue - Create an External Data Queue */

 dcl RexxCreateQueue entry
          ( char (*) varz byaddr,      /* Name of queue created       */
            type ULONG,                /* Size of buf for ret name    */
            char (*) varz byaddr,      /* Requested name for queue    */
            type ULONG byaddr)         /* Duplicate name flag.        */
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxCreateQueue');


 /***    RexxDeleteQueue - Delete an External Data Queue */

 dcl RexxDeleteQueue entry
          ( char (*) varz byaddr)      /* Name of queue to be deleted */
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxDeleteQueue');


 /*** RexxQueryQueue - Query an External Data Queue for number of     */
 /***                  entries                                        */

 dcl RexxQueryQueue entry
          ( char (*) varz byaddr,      /* Name of queue to query      */
            type ULONG byaddr)         /* Place to put element count  */
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxQueryQueue');


 /***    RexxAddQueue - Add an entry to an External Data Queue */

 dcl RexxAddQueue entry
          ( char (*) varz byaddr,      /* Name of queue to add to     */
            type RXSTRING byaddr,      /* Data string to add          */
            type ULONG )               /* Queue type (FIFO|LIFO)      */
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxAddQueue');


 /***    RexxPullQueue - Retrieve data from an External Data Queue */

 dcl RexxPullQueue entry
          ( char (*) varz byaddr,      /* Name of queue to add to     */
            type RXSTRING byaddr,      /* RXSTRING to receive data    */
            type SYSTEMTIME byaddr,    /* Store for data date/time    */
            type ULONG )               /* wait status (WAIT|NOWAIT)   */
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxPullQueue');

 %end; /* INCL_RXQUEUE */


 dcl RexxShutDownAPI entry             /* request from toronto */
          (/* No Arguments */)
       returns( byvalue optional type ULONG )
       REXXAPI ext ('RexxShutDownAPI');

 %end; /* REXXSAA_INCLUDED */
