 /* tcl.cpy --                                                        */
 /*                                                                   */
 /* This header file describes the externally-visible facilities      */
 /* of the Tcl interpreter.                                           */
 /*                                                                   */
 /* Copyright (c) 1987-1994 The Regents of the University of          */
 /*                         California.                               */
 /* Copyright (c) 1993-1996 Lucent Technologies.                      */
 /* Copyright (c) 1994-1998 Sun Microsystems, Inc.                    */
 /* Copyright (c) 1998-2000 by Scriptics Corporation.                 */
 /* Copyright (c) 1998-2000 Mark Yudkin (PL/I translation)            */

 %dcl #TCL char ext;
 %if #TCL = '' %then %do;
 %#TCL = 'Y';

 %dcl TCL_MEM_DEBUG     char ext; /* set to Y for debug routines      */

 %dcl TCL_NO_DEPRECATED char ext; /* set to Y for deprecated features */

 /* The following defines are used to indicate the various release    */
 /* levels.                                                           */

 dcl TCL_ALPHA_RELEASE fixed bin(31) value(0);
 dcl TCL_BETA_RELEASE  fixed bin(31) value(1);
 dcl TCL_FINAL_RELEASE fixed bin(31) value(2);

 /* When version numbers change here, must also go into the following */
 /* files and update the version numbers:                             */
 /*                                                                   */
 /* library/init.tcl                                                  */
 /* unix/configure.in                                                 */
 /* unix/pkginfo                                                      */
 /* win/makefile.bc                                                   */
 /* win/makefile.vc                                                   */
 /*                                                                   */
 /* The release level should be  0 for alpha, 1 for beta, and 2 for   */
 /* final/patch.  The release serial value is the number that follows */
 /*                                                                   */
 /* the "a", "b", or "p" in the patch level; for example, if the      */
 /* patch level is 7.6b2, TCL_RELEASE_SERIAL is 2.  It restarts at 1  */
 /* whenever the release level is changed, except for the final       */
 /* release which is 0 (the first patch will start at 1).             */

 dcl TCL_MAJOR_VERSION  fixed bin(31) value(8);
 dcl TCL_MINOR_VERSION  fixed bin(31) value(3);
 dcl TCL_RELEASE_LEVEL  fixed bin(31) value(TCL_FINAL_RELEASE);
 dcl TCL_RELEASE_SERIAL fixed bin(31) value(3);

 dcl TCL_VERSION        char value('8.3');
 dcl TCL_PATCH_LEVEL    char value('8.3.3');

 /* The following definitions set up the proper options for Windows   */
 /* compilers.  We use this method because there is no autoconf       */
 /* equivalent.                                                       */

 %dcl USE_TCLALLOC fixed ext;

 %dcl #_WIN32__ char ext;
 %if system() = ' WIN ' ! system() = ' OS2 ' %then %do;
   %#_WIN32__ = 'Y';
 %end;
 %dcl #WIN32__ char ext;
 %dcl WIN32 char ext;
 %if #_WIN32__ = '' %then %do;
   %if (#WIN32__^='') | (WIN32^='') %then %do;
     %#_WIN32__ = 'Y';
   %end;
 %end;

 %if #_WIN32__ ^= '' %then %do;
   %if USE_TCLALLOC = 0 %then %do;
     %USE_TCLALLOC = 1;
   %end;
 %end; /* #_WIN32__ */

 /* Special macro to define mutexes, that doesn't do anything */
 /* if we are not using threads.                              */

 %dcl TCL_THREADS char;

 %TCL_DECLARE_MUTEX: proc (name) returns (char) stmt;
   dcl name char;
   if TCL_THREADS ^= '' then
     return ('dcl ' || name || ' type TCL_MUTEX static');
   else
     return ('');
 %end TCL_DECLARE_MUTEX;
 %dcl TCL_DECLARE_MUTEX entry;

 /* Macros that eliminate the overhead of the thread synchronization  */
 /* functions when compiling without thread support.                  */
 %if TCL_THREADS = '' %then %do;

 Tcl_MutexLock: proc (mutexPtr) options (inline) internal;
   dcl mutexPtr type Tcl_Mutex byaddr;
 end Tcl_MutexLock;

 Tcl_MutexUnlock: proc (mutexPtr) options (inline) internal;
   dcl mutexPtr type Tcl_Mutex byaddr;
 end Tcl_MutexUnlock;

 Tcl_MutexFinalize: proc (mutexPtr) options (inline) internal;
   dcl mutexPtr type Tcl_Mutex byaddr;
 end Tcl_MutexFinalize;

 Tcl_ConditionNotify: proc (condPtr) options (inline) internal;
   dcl condPtr type Tcl_Condition byaddr;
 end Tcl_ConditionNotify;

 Tcl_ConditionWait: proc (condPtr, mutexPtr, timePtr)
                    options (inline) internal;
   dcl condPtr  type Tcl_Condition byaddr,
       mutexPtr type Tcl_Mutex byaddr,
       timePtr  type Tcl_Time byaddr optional;
 end Tcl_ConditionWait;

 Tcl_ConditionFinalize: proc (condPtr) options (inline) internal;
   dcl condPtr type Tcl_Condition byaddr;
 end Tcl_ConditionFinalize;

 %end; /* TCL_THREADS */


 /* Miscellaneous PL/I-specific declarations. */
 dcl $tcl_string char (32767) varz based; /* for macros */
 dcl $tcl_array (32767) pointer based; /* for macros */
 %$tcl_p2z: proc (p, i) returns (char);
   dcl (p, i) char;
   /* tcl_p2z (p) turns a "char *" into a PL/I string */
   /* tcl_p2z (p, i) turns a "char **" (array) into a PL/I string */
   if parmset (i) then
     return (p || ' -> $tcl_array (' || i || ') -> $tcl_string');
   else
     return (p || ' -> $tcl_string');
 %end $tcl_p2z;
 %dcl $tcl_p2z entry;

 $Tcl_Entry_Constant: proc (p) returns (entry limited
                      options (nodescriptor byvalue linkage(cdecl)))
                      options (inline) internal;
   dcl p fixed bin (31);
   dcl e entry limited options (nodescriptor byvalue linkage(cdecl));
   dcl (entryaddr, ptrvalue) builtin;
   entryaddr (e) = ptrvalue (p);
   return (e);
 end $Tcl_Entry_Constant;

 %dcl TCL_NULL_ENTRY char ext; /* for "NULL" entries */
 %TCL_NULL_ENTRY = '$Tcl_Entry_Constant (0)';

 %dcl #CLIENTDATA char ext;
 %if #CLIENTDATA = '' %then %do;
 %#CLIENTDATA = 'Y';
 define alias ClientData pointer;
 %end;

 /* Data structures defined opaquely in this module. The definitions  */
 /* below just provide dummy types. A few fields are made visible in  */
 /* Tcl_Interp structures, namely those used for returning a string   */
 /* result from commands. Direct access to the result field is        */
 /* discouraged in Tcl 8.0.                                           */
 /* The interpreter result is either an object or a string, and the   */
 /* two values are kept consistent unless some code sets              */
 /* interp=>result directly. Programmers should use either the        */
 /* procedure Tcl_GetObjResult() or Tcl_GetStringResult() to read the */
 /* interpreter's result. See the SetResult man page for details.     */
 /*                                                                   */
 /* Note: any change to the Tcl_Interp definition below must be       */
 /* mirrored in the "real" definition in tclInt.h.                    */
 /*                                                                   */
 /* Note: Tcl_ObjCmdProc procedures do not directly set result and    */
 /* free Proc.                                                        */
 /* Instead, they set a Tcl_Obj member in the "real" structure that   */
 /* can be accessed with Tcl_GetObjResult() and Tcl_SetObjResult().   */

 define structure
   1 Tcl_Interp,
     2 result pointer, /* If the last command returned a string       */
                       /* result, this points to it.                  */
     2 freeProc entry(ptr) options (nodescriptor byvalue
                                    linkage(cdecl)) limited,
                       /* Tcl_FreeProc  */
     /* TCL_STATIC  means the string result is statically allocated.  */
     /* TCL_DYNAMIC means it was allocated with ckalloc and should    */
     /* be freed with ckfree. Other values give the address of a      */
     /* procedure to invoke to free the result. Tcl_Eval must free    */
     /* it before executing next command.                             */
     2 errorLine fixed bin(31); /* When TCL_ERROR is returned, this   */
     /* gives the line number within the command where * the error    */
     /* occurred (1 if first line).                                   */

 define alias Tcl_AsyncHandler       pointer;
 define alias Tcl_Channel            pointer;
 define alias Tcl_Command            pointer;
 define alias Tcl_Condition          pointer;
 define alias Tcl_EncodingState      pointer;
 define alias Tcl_Encoding           pointer;
 define alias Tcl_Mutex              pointer;
 define alias Tcl_Pid                pointer;
 define alias Tcl_RegExp             pointer;
 define alias Tcl_ThreadDataKey      pointer;
 define alias Tcl_ThreadId           pointer;
 define alias Tcl_TimerToken         pointer;
 define alias Tcl_Trace              pointer;
 define alias Tcl_Var                pointer;
 define alias Tcl_ChannelTypeVersion pointer;

 /* Definition of the interface to procedures implementing threads.   */
 /* A procedure following this definition is given to each call of    */
 /* 'Tcl_CreateThread' and will be called as the main fuction of      */
 /* the new thread created by that call.                              */

 define alias Tcl_ThreadCreateProc
        entry (type ClientData)
        returns (fixed bin (32) unsigned byvalue optional)
        options (byvalue nodescriptor linkage(stdcall)) limited;

 /* Threading function return types used for abstracting away platform*/
 /* differences when writing a Tcl_ThreadCreateProc. See the NewThread*/
 /* function in generic/tclThreadTest.c for its usage.                */

 define alias Tcl_ThreadCreateType
        entry options (byvalue nodescriptor linkage(stdcall)) limited;

 %TCL_THREAD_CREATE_RETURN: proc returns (char) stmt;
   return ('return (0)');
 %end TCL_THREAD_CREATE_RETURN;
 %dcl TCL_THREAD_CREATE_RETURN entry;


 /* Definition of values for default stacksize and the possible flags */
 /* to be given to Tcl_CreateThread.                                  */

 dcl TCL_THREAD_STACK_DEFAULT fixed bin (31) value (0);
                              /* Use default size for stack */
 dcl TCL_THREAD_NOFLAGS       fixed bin (31) value ('0000'xn);
                              /* Standard flags, default behaviour */
 dcl TCL_THREAD_JOINABLE      fixed bin (31) value ('0001'xn);
                              /* Mark the thread as joinable */


 /* Flag values passed to Tcl_GetRegExpFromObj. */

 dcl TCL_REG_BASIC    fixed bin (31) value ('0000000'xn);
                      /* BREs (convenience) */
 dcl TCL_REG_EXTENDED fixed bin (31) value ('000001'xn);
                      /* EREs */
 dcl TCL_REG_ADVF     fixed bin (31) value ('000002'xn);
                      /* advanced features in EREs */
 dcl TCL_REG_ADVANCED fixed bin (31) value ('000003'xn);
                      /* AREs (which are also EREs) */
 dcl TCL_REG_QUOTE    fixed bin (31) value ('000004'xn);
                      /* no special characters, none */
 dcl TCL_REG_NOCASE   fixed bin (31) value ('000010'xn);
                      /* ignore case */
 dcl TCL_REG_NOSUB    fixed bin (31) value ('000020'xn);
                      /* don't care about subexpressions */
 dcl TCL_REG_EXPANDED fixed bin (31) value ('000040'xn);
                      /* expanded format, white space & comments */
 dcl TCL_REG_NLSTOP   fixed bin (31) value ('000100'xn);
                      /* \n doesn't match . or [^ ] */
 dcl TCL_REG_NLANCH   fixed bin (31) value ('000200'xn);
                      /* ^ matches after \n, $ before */
 dcl TCL_REG_NEWLINE  fixed bin (31) value ('000300'xn);
                      /* newlines are line terminators */
 dcl TCL_REG_CANMATCH fixed bin (31) value ('001000'xn);
                      /* report details on partial/limited matches */

 /* The following flag is experimental and only intended for use */
 /* by Expect. It will probably go away in a later release.      */

 dcl TCL_REG_BOSONLY  fixed bin (31) value ('002000'xn);
                      /* prepend \A to pattern so it only matches */
                      /* at the beginning of the string.          */

 /* Flags values passed to Tcl_RegExpExecObj. */

 dcl TCL_REG_NOTBOL   fixed bin (31) value ('0001'xn);
                      /* Beginning of string does not match ^.  */
 dcl TCL_REG_NOTEOL   fixed bin (31) value ('0002'xn);
                      /* End of string does not match $. */

 define structure
   1 Tcl_RegExpIndices,
     2 start fixed bin (31),
             /* character offset of first character in match */
     2 end   fixed bin (31);
             /* character offset of first character after the match. */

 define structure
   1 Tcl_RegExpInfo,
     2 nsubs       fixed bin (31),
               /* number of subexpressions in the compiled expression */
     2 matches     handle Tcl_RegExpIndices,
               /* array of nsubs match offset pairs */
     2 extendStart fixed bin (31),
               /* The offset at which a subsequent match might begin. */
     2 reserved    fixed bin (31);
               /* Reserved for later use. */


 /* When a TCL command returns, the interpreter contains a result     */
 /* from the command. Programmers are strongly encouraged to use one  */
 /* of the procedures Tcl_GetObjResult() or Tcl_GetStringResult() to  */
 /* read the interpreter's result. See the SetResult man page for     */
 /* details. Besides this result, the command procedure returns an    */
 /* integer code, which is one of the following:                      */
 /*                                                                   */
 /* TCL_OK       Command completed normally. The interpreter's        */
 /*              result contains the command's result.                */
 /* TCL_ERROR    The command couldn't be completed successfully.      */
 /*              The interpreter's result describes what went wrong.  */
 /* TCL_RETURN   The command requests that the current procedure      */
 /*              return. The interpreter's result contains the        */
 /*              procedure's return value.                            */
 /* TCL_BREAK    The command requests that the innermost loop         */
 /*              be exited. the interpreter's result is meaningless.  */
 /* TCL_CONTINUE Go on to the next iteration of the current loop.     */
 /*              the interpreter's result is meaningless.             */

 dcl TCL_OK       fixed bin(31) value(0);
 dcl TCL_ERROR    fixed bin(31) value(1);
 dcl TCL_RETURN   fixed bin(31) value(2);
 dcl TCL_BREAK    fixed bin(31) value(3);
 dcl TCL_CONTINUE fixed bin(31) value(4);

 dcl TCL_RESULT_SIZE fixed bin(31) value(200);

 /* Argument descriptors for math function callbacks in expressions: */

 define ordinal Tcl_ValueType (TCL_INT,
                               TCL_DOUBLE,
                               TCL_EITHER) prec(31);
 define structure
   1 Tcl_Value,
     2 type   type Tcl_ValueType,/* Indicates intValue or doubleValue */
                                 /* is valid, or both.                */
     2 intValue    fixed bin (31), /* Integer value.                  */
     2 doubleValue float bin (53); /* Double-precision float value.   */


 /* Procedure types defined by Tcl: */
 define alias Tcl_AppInitProc
        entry (handle Tcl_Interp)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_AsyncProc
        entry (type ClientData,
               handle Tcl_Interp,
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_ChannelProc
        entry (type ClientData,
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_CloseProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_CmdDeleteProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_CmdProc
        entry (type ClientData,
               handle Tcl_Interp,
               fixed bin (31),
               pointer) /* to (*) pointer to char (*) varz */
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_CmdTraceProc
        entry (type ClientData,
               handle Tcl_Interp,
               fixed bin (31),
               pointer, /* to (*) pointer to char (*) varz */,
               type Tcl_CmdProc,
               type ClientData,
               fixed bin (31),
               pointer) /* to (*) pointer to char (*) varz */
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DupInternalRepProc
        entry (handle Tcl_Obj,
               handle Tcl_Obj)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EncodingConvertProc
        entry (type ClientData,
               pointer, /* to char (*) varz */,
               fixed bin (31),
               fixed bin (31),
               type Tcl_EncodingState byaddr,
               pointer, /* to char (*) varz */,
               fixed bin (31),
               fixed bin (31) byaddr,
               fixed bin (31) byaddr,
               fixed bin (31) byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EncodingFreeProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EventProc
        entry (handle Tcl_Event,
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EventCheckProc
        entry (type ClientData,
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EventDeleteProc
        entry (handle Tcl_Event,
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_EventSetupProc
        entry (type ClientData,
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_ExitProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_FileProc
        entry (type ClientData,
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_FileFreeProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_FreeInternalRepProc
        entry (handle Tcl_Obj)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_FreeProc
        entry (pointer)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_IdleProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_InterpDeleteProc
        entry (type ClientData,
               handle Tcl_Interp)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_MathProc
        entry (type ClientData,
               handle Tcl_Interp,
               handle Tcl_Value,
               handle Tcl_Value)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_NamespaceDeleteProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_ObjCmdProc
        entry (type ClientData,
               handle Tcl_Interp,
               fixed bin (31),
               pointer) /* to (*) handle Tcl_Obj */
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_PackageInitProc
        entry (handle Tcl_Interp)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_PanicProc
        entry (pointer list)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_TcpAcceptProc
        entry (type ClientData,
               type Tcl_Channel,
               pointer, /* to char (*) varz byaddr */
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_TimerProc
        entry (type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_SetFromAnyProc
        entry (type Tcl_Interp,
               handle Tcl_Obj)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_UpdateStringProc
        entry (handle Tcl_Obj)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_VarTraceProc
        entry (type ClientData,
               type Tcl_Interp,
               pointer, /* to char (*) varz byaddr */
               pointer, /* to char (*) varz byaddr */
               fixed bin (31))
        returns (pointer byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_CreateFileHandlerProc
        entry (fixed bin (31),
               fixed bin (31),
               type Tcl_FileProc,
               type ClientData)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DeleteFileHandlerProc
        entry (fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;


 /* The following structure represents a type of object, which is a   */
 /* particular internal representation for an object plus a set of    */
 /* procedures that provide standard operations on objects of that    */
 /* type.                                                             */

 define structure
   1 Tcl_ObjType,
     2 name pointer, /* Name of the type, e.g. "int". */
     2 freeIntRepProc   type Tcl_FreeInternalRepProc,
                     /* Called to free any storage for the type's */
                     /* internal rep. NULL if the internal rep    */
                     /* does not need freeing.                    */
     2 dupIntRepProc    type Tcl_DupInternalRepProc,
                     /* Called to create a new object as a copy   */
                     /* of an existing object.                    */
     2 updateStringProc type Tcl_UpdateStringProc,
                     /* Called to update the string rep from the  */
                     /* type's internal representation.           */
     2 setFromAnyProc   type Tcl_SetFromAnyProc;
                     /* Called to convert the object's internal   */
                     /* rep to this type. Frees the internal rep  */
                     /* of the old type. Returns TCL_ERROR on     */
                     /* failure.                                  */

 /* One of the following structures exists for each object in the Tcl */
 /* system. An object stores a value as either a string,              */
 /* some internal representation, or both.                            */

 define structure
   1 Tcl_Obj,
     2 refCount fixed bin(31),/* When 0 the object will be freed.     */
     2 bytes pointer, /* This points to the first byte of the         */
                      /* object's string representation. The array    */
                      /* must be followed by a null byte (i.e., at    */
                      /* offset length) but may also contain          */
                      /* embedded null characters. The array's        */
                      /* storage is allocated by ckalloc. NULL        */
                      /* means the string rep is invalid and must     */
                      /* be regenerated from the internal rep.        */
                      /* Clients should use Tcl_GetStringFromObj      */
                      /* to get a pointer to the byte array as a      */
                      /* readonly value.                              */
     2 length fixed bin(31),/* The number of bytes at *bytes, not     */
                            /* including the terminating null.        */
     2 typePtr handle Tcl_ObjType,/* Denotes the object's type. Always*/
                       /* corresponds to the type of the object's     */
                       /* internal rep. NULL indicates the object     */
                       /* has no internal rep (has no type).          */
     2 internalRep union,
       3 longValue fixed bin(31),   /* - an long integer value */
       3 doubleValue float bin (53),/* - a double-precision float */
       3 otherValuePtr pointer,     /* - another, type-specific value */
       3 twoPtrValue,
         4 ptr1 pointer,
         4 ptr2 pointer;

 /* Macros to increment and decrement a Tcl_Obj's reference count, and*/
 /* to test whether an object is shared (i.e. has reference count > 1)*/
 /* Note: clients should use Tcl_DecrRefCount() when they are         */
 /* finished using an object, and should never call TclFreeObj()      */
 /* directly. TclFreeObj() is only defined and made public in tcl.inc */
 /* to support Tcl_DecrRefCount's macro definition.                   */
 /* Note also that Tcl_DecrRefCount() refers to the parameter         */
 /* "obj" twice. This means that you should avoid calling it with an  */
 /* expression that is expensive to compute or has side effects.      */

 %if 0 = 1 %then %do;
 dcl Tcl_IncrRefCount entry (handle Tcl_Obj )
                      options (byvalue nodescriptor linkage(cdecl))
                      external('Tcl_IncrRefCount');
 dcl Tcl_DecrRefCount entry (handle Tcl_Obj )
                      options (byvalue nodescriptor linkage(cdecl))
                      external('Tcl_DecrRefCount');
 dcl Tcl_IsShared     entry (handle Tcl_Obj )
                      returns (fixed bin(31) byvalue)
                      options (byvalue nodescriptor linkage(cdecl))
                      external('Tcl_IsShared');
 %end;

 %Tcl_IncrRefCount: proc (objPtr) returns (char);
   dcl objPtr char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('call Tcl_DbIncrRefCount (' || objPtr ||
             ', sourcefile(), sourceline())');
   end; else do;
     return (objPtr || ' => refCount += 1;');
   end;
 %end Tcl_IncrRefCount;

 %Tcl_DecrRefCount: proc (objPtr) returns (char);
   dcl objPtr char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('call Tcl_DbDecrRefCount (' || objPtr ||
             ', sourcefile(), sourceline())');
   end; else do;
     return ('do; ' ||
             objPtr || ' => refCount -= 1;' ||
             'if ' || objPtr || ' <= 0 then ' ||
                'call TclFreeObj (' || objPtr || ');' ||
             'end;');
   end;
 %end Tcl_DecrRefCount;

 %Tcl_IsShared: proc (objPtr) returns (char);
   dcl objPtr char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('call Tcl_DbIsShared (' || objPtr ||
             ', sourcefile(), sourceline())');
   end; else do;
     return ('binary (' || objPtr || ' => refCount > 1, 31)');
   end;
 %end Tcl_IsShared;

 %dcl (Tcl_IncrRefCount, Tcl_DecrRefCount, Tcl_IsShared) entry;

 /* Macros and definitions that help to debug the use of Tcl objects. */
 /* When TCL_MEM_DEBUG is defined, the Tcl_New* declarations are      */
 /* overridden to call debugging versions of the object creation      */
 /* procedures.                                                       */

 %if TCL_MEM_DEBUG ^= '' %then %do;
 %Tcl_NewBooleanObj: proc (val) returns (char);
   dcl val char;
   return ('Tcl_DbNewBooleanObj (' || val ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewBooleanObj;

 %Tcl_NewByteArrayObj: proc (bytes, len) returns (char);
   dcl (bytes, len) char;
   return ('Tcl_DbNewByteArrayObj (' || bytes || ', ' || len ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewByteArrayObj;

 %Tcl_NewDoubleObj: proc (val) returns (char);
   dcl val char;
   return ('Tcl_DbNewDoubleObj (' || val ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewDoubleObj;

 %Tcl_NewIntObj: proc (val) returns (char);
   dcl val char;
   return ('Tcl_DbNewIntObj (' || val ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewIntObj;

 %Tcl_NewListObj: proc (val) returns (char);
   dcl val char;
   return ('Tcl_DbNewListObj (' || val ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewListObj;

 %Tcl_NewLongObj: proc (val) returns (char);
   dcl val char;
   return ('Tcl_DbNewLongObj (' || val ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewLongObj;

 %Tcl_NewObj: proc () returns (char);
   return ('Tcl_DbNewObj (sourcefile(), sourceline())');
 %end Tcl_NewObj;

 %Tcl_NewStringObj: proc (bytes, len) returns (char);
   dcl (bytes, len) char;
   return ('Tcl_DbNewStringObj (' || bytes || ', ' || len ||
           ', sourcefile(), sourceline())');
 %end Tcl_NewStringObj;

 %dcl (Tcl_NewBooleanObj, Tcl_NewDoubleObj, Tcl_NewIntObj,
       Tcl_NewListObj, Tcl_NewLongObj, Tcl_NewObj,
       Tcl_NewStringObj) entry;
 %end; /* TCL_MEM_DEBUG */

 /* The following structure contains the state needed by              */
 /* Tcl_SaveResult.  No-one outside of Tcl should access any of these */
 /* fields.  This structure is typically allocated on the stack.      */

 define structure
   1 Tcl_SavedResult,
     2 result       pointer, /* to char(*) varz */
     2 freeProc     type Tcl_FreeProc,
     2 objResultPtr handle Tcl_Obj,
     2 appendResult pointer, /* to char(*) varz */
     2 appendAvl    fixed bin (31),
     2 appendUsed   fixed bin (31),
     2 resultSpace  char (TCL_RESULT_SIZE) varz;

 /* The following definitions support Tcl's namespace facility.       */
 /* Note: the first five fields must match exactly the fields in a    */
 /* Namespace structure (see tcl.inc).                                */

 define structure
   1 Tcl_Namespace,
     2 name       pointer,/* The namespace's name within its parent */
                          /* namespace. This contains no ::'s. The  */
                          /* name of the global namespace is ""     */
                          /* although "::" is an synonym.           */
     2 fullName   pointer,/* The namespace's fully qualified name.  */
                          /* This starts with ::. */
     2 clientData type ClientData,
                  /* Arbitrary value associated with this namespace. */
     2 deleteProc type Tcl_NamespaceDeleteProc,
                  /* Procedure invoked when deleting the             */
                  /* namespace to, e.g., free clientData.            */
     2 parentPtr  handle Tcl_Namespace;
                  /* Points to the namespace that contains this one. */
                  /* NULL if this is the global namespace.           */

 /* The following structure represents a call frame, or activation    */
 /* record. A call frame defines a naming context for a procedure     */
 /* call: its local scope (for local variables) and its namespace     */
 /* scope (used for non-local variables; often the global ::          */
 /* namespace). A call frame can also define the naming context for a */
 /* namespace eval or namespace inscope command: the namespace in     */
 /* which the command's code should execute. The Tcl_CallFrame        */
 /* structures exist only while procedures or namespace               */
 /* eval/inscope's are being executed, and provide a Tcl call stack.  */
 /*                                                                   */
 /* A call frame is initialized and pushed using Tcl_PushCallFrame    */
 /* and popped using Tcl_PopCallFrame. Storage for a Tcl_CallFrame    */
 /* must be provided by the Tcl_PushCallFrame caller, and callers     */
 /* typically allocate them on the C call stack for efficiency. For   */
 /* this reason, Tcl_CallFrame is defined as a structure and not as   */
 /* an opaque token. However, most Tcl_CallFrame fields are hidden    */
 /* since applications should not access them directly; others are    */
 /* declared as "dummyX".                                             */
 /*                                                                   */
 /* WARNING!! The structure definition must be kept consistent with   */
 /* the CallFrame structure in tclInt.h. If you change one, change    */
 /* the other.                                                        */

 define structure
   1 Tcl_CallFrame,
    2 nsPtr pointer,
    2 dummy1 fixed bin(31),
    2 dummy2 fixed bin(31),
    2 dummy3 pointer,
    2 dummy4 pointer,
    2 dummy5 pointer,
    2 dummy6 fixed bin(31),
    2 dummy7 pointer,
    2 dummy8 pointer,
    2 dummy9 fixed bin(31),
    2 dummy10 pointer;

 /* Information about commands that is returned by Tcl_GetCommandInfo */
 /* and passed to Tcl_SetCommandInfo. objProc is an objc/objv         */
 /* object-based command procedure while proc is a traditional Tcl    */
 /* argc/argv string-based procedure. Tcl_CreateObjCommand and        */
 /* Tcl_CreateCommand ensure that both objProc and proc are non-NULL  */
 /* and can be called to execute the command. However, it may be      */
 /* faster to call one instead of the other. The member               */
 /* isNativeObjectProc is set to 1 if an object-based procedure was   */
 /* registered by Tcl_CreateObjCommand, and to 0 if a string-based    */
 /* procedure was registered by Tcl_CreateCommand. The other          */
 /* procedure is typically set to a compatibility wrapper that does   */
 /* string-to-object or object-to-string argument conversions then    */
 /* calls the other procedure.                                        */


 define structure
   1 Tcl_CmdInfo,
    2 isNativeObjectProc fixed bin(31),/* 1 if objProc was registered */
                  /* by a call to Tcl_CreateObjCommand, 0 otherwise.  */
                  /* Tcl_SetCmdInfo does not modify this field.       */
    2 objProc type Tcl_ObjCmdProc,/* Command's object-based procedure.*/
    2 objClientData type ClientData,/* ClientData for object proc.    */
    2 proc type Tcl_CmdProc, /* Command's string-based procedure.     */
    2 clientData type ClientData,/* ClientData for string proc.       */
    2 deleteProc type Tcl_CmdDeleteProc,
                        /* Procedure to call when command is deleted. */
    2 deleteData type ClientData,/* Value to pass to deleteProc       */
                                 /* (usually the same as clientData). */
    2 namespacePtr handle Tcl_Namespace; /* Points to the namespace   */
                  /* that contains this command. Note that            */
                  /* Tcl_SetCmdInfo will not change a command's name  */
                  /* space, use Tcl_RenameCommand to do that.         */

 /* The structure defined below is used to hold dynamic strings.      */
 /* The only field that clients should use is the string field, and   */
 /* they should never modify it.                                      */

 dcl TCL_DSTRING_STATIC_SIZE fixed bin(31) value(200);
 define structure
   1 Tcl_DString,
    2 string pointer, /* Points to beginning of string: either        */
                      /* staticSpace below or a malloced array.       */
    2 length fixed bin(31), /* Number of non-NULL characters in the   */
                            /* string.                                */
    2 spaceAvl fixed bin(31),/* Total number of bytes available for   */
                      /* the string and its terminating NULL char.    */
    2 staticSpace char(TCL_DSTRING_STATIC_SIZE-1) varyingz;
                /* Space to use in common case where string is small. */

 %Tcl_DStringLength: proc (dsVal) returns (char);
   dcl dsVal char;
   return (dsVal || '.length');
 %end Tcl_DStringLength;
 %dcl Tcl_DStringLength entry;

 %Tcl_DStringValue: proc (dsVal) returns (char);
   dcl dsVal char;
   return (dsVal || '.string -> $tcl_string');
 %end Tcl_DStringValue;
 %dcl Tcl_DStringValue entry;

 %dcl Tcl_DStringTrunc char ext;
 %Tcl_DStringTrunc='Tcl_DStringSetLength';

 /* Definitions for the maximum number of digits of precision that    */
 /* maybe specified in the "tcl_precision" variable, and the number   */
 /* of characters of buffer space required by Tcl_PrintDouble.        */

 dcl TCL_MAX_PREC        fixed bin (31) value(17);
 dcl TCL_DOUBLE_SPACE    fixed bin (31) value(TCL_MAX_PREC+10);

 /* Definition for a number of bytes of buffer space sufficient to    */
 /* hold the string representation of an integer in base 10           */
 /* (assuming the existence of 64-bit integers).                      */

 dcl TCL_INTEGER_SPACE   fixed bin(31) value(24);

 /* Flag that may be passed to Tcl_ConvertElement to force it not to  */
 /* output braces (careful!  if you change this flag be sure to       */
 /* change the definitions at the front of tclUtil.c).                */

 dcl TCL_DONT_USE_BRACES fixed bin(31) value(1);

 /* Flag that may be passed to Tcl_GetIndexFromObj to force it to     */
 /* disallow abbreviated strings.                                     */

 dcl TCL_EXACT           fixed bin(31) value(1);

 /* Flag values passed to Tcl_RecordAndEval and/or Tcl_EvalObj.       */
 /* WARNING: these bit choices must not conflict with the bit choices */
 /* for evalFlag bits in tclInt.h!!                                   */

 dcl TCL_NO_EVAL         fixed bin(31) value('10000'xn);
 dcl TCL_EVAL_GLOBAL     fixed bin(31) value('20000'xn);
 dcl TCL_EVAL_DIRECT     fixed bin(31) value('40000'xn);

 /* Special freeProc values that may be passed to Tcl_SetResult       */
 /* (see the man page for details).                                   */

 /* PL/I has no technique for defining a constant of an entry types   */
 /* with a specific value (ENTRYADDR pseudo-variable is required).    */
 $Tcl_FreeProc_Constant: proc (p) returns (type Tcl_FreeProc)
                         options (inline) internal;
   dcl p pointer byvalue;
   dcl e type Tcl_FreeProc;
   dcl entryaddr builtin;
   entryaddr (e) = p;
   return (e);
 end $Tcl_FreeProc_Constant;

 %dcl TCL_VOLATILE char ext;
 %TCL_VOLATILE = '$Tcl_FreeProc_Constant (ptrvalue (1))';
 %dcl TCL_STATIC  char ext;
 %TCL_STATIC   = '$Tcl_FreeProc_Constant (ptrvalue (0))';
 %dcl TCL_DYNAMIC char ext;
 %TCL_DYNAMIC  = '$Tcl_FreeProc_Constant (ptrvalue (3))';


 /* Flag values passed to variable-related procedures. */

 dcl TCL_GLOBAL_ONLY      fixed bin(31) value(1);
 dcl TCL_NAMESPACE_ONLY   fixed bin(31) value(2);
 dcl TCL_APPEND_VALUE     fixed bin(31) value(4);
 dcl TCL_LIST_ELEMENT     fixed bin(31) value(8);
 dcl TCL_TRACE_READS      fixed bin(31) value('10'xn);
 dcl TCL_TRACE_WRITES     fixed bin(31) value('20'xn);
 dcl TCL_TRACE_UNSETS     fixed bin(31) value('40'xn);
 dcl TCL_TRACE_DESTROYED  fixed bin(31) value('80'xn);
 dcl TCL_INTERP_DESTROYED fixed bin(31) value('100'xn);
 dcl TCL_LEAVE_ERR_MSG    fixed bin(31) value('200'xn);
 dcl TCL_TRACE_ARRAY      fixed bin(31) value('800'xn);

 /* The TCL_PARSE_PART1 flag is deprecated and has no effect.         */
 /* The part1 is now always parsed whenever the part2 is NULL.        */
 /* (This is to avoid a common error when converting code to          */
 /*  use the new object based APIs and forgetting to give the flag)   */

 %if TCL_NO_DEPRECATED ^= '' %then %do;
 dcl TCL_PARSE_PART1      fixed bin(31) value('400'xn);
 %end;

 /* Types for linked variables: */

 dcl TCL_LINK_INT         fixed bin(31) value(1);
 dcl TCL_LINK_DOUBLE      fixed bin(31) value(2);
 dcl TCL_LINK_BOOLEAN     fixed bin(31) value(3);
 dcl TCL_LINK_STRING      fixed bin(31) value(4);
 dcl TCL_LINK_READ_ONLY   fixed bin(31) value('80'xn);

 /* The following declarations either map ckalloc and ckfree to       */
 /* alloc and plifree, or they map them to procedures with all sorts  */
 /* of debugging hooks defined in tclCkalloc.c.                       */

 %if TCL_MEM_DEBUG ^= '' %then %do;

 %Tcl_Alloc: proc (x) returns (char);
   dcl x char;
   return ('Tcl_DbCkalloc (' || x ||
           ', sourcefile(), sourceline())');
 %end Tcl_Alloc;

 %Tcl_Free: proc (x) returns (char);
   dcl x char;
   return ('Tcl_DbCkfree (' || x ||
           ', sourcefile(), sourceline())');
 %end Tcl_Free;

 %Tcl_Realloc: proc (x, y) returns (char);
   dcl (x, y) char;
   return ('Tcl_DbCkrealloc (' || x || ',' || y ||
           ', sourcefile(), sourceline())');
 %end Tcl_Realloc;

 /* activate Tcl_Alloc, Tcl_Free and Tcl_Realloc */
 /* after declaration of externals (see below).  */

 %end;

 %ckalloc: proc (x) returns (char);
   dcl x char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('Tcl_DbCkalloc (' || x ||
             ', sourcefile(), sourceline())');
   end; else if USE_TCLALLOC ^= 0 then do;
     return ('Tcl_Alloc (' || x || ')');
   end; else do;
     return ('alloc (' || x || ')');
   end;
 %end ckalloc;

 %ckfree: proc (x) returns (char);
   dcl x char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('Tcl_DbCkfree (' || x ||
             ', sourcefile(), sourceline())');
   end; else if USE_TCLALLOC ^= 0 then do;
     return ('Tcl_Free (' || x || ')');
   end; else do;
     return ('plifree (' || x || ')');
   end;
 %end ckfree;

 %ckrealloc: proc (x, y) returns (char);
   dcl (x, y) char;
   if TCL_MEM_DEBUG ^= '' then do;
     return ('Tcl_DbCkrealloc (' || x || ',' || y ||
             ', sourcefile(), sourceline())');
   end; else if USE_TCLALLOC ^= 0 then do;
     return ('Tcl_Realloc (' || x || ',' || y || ')');
   end; else do;
     note ('ckrealloc unusable, no such routine in PL/I', 8);
     return ('realloc (' || x || ',' || y || ')');
   end;
 %end ckrealloc;

 %dcl (ckalloc, ckfree, ckrealloc) entry;


 %if TCL_MEM_DEBUG ^= '' %then %do;

 Tcl_InitMemory: proc (x)
                 options(inline) internal;
   dcl x handle Tcl_Interp byvalue;
 end Tcl_InitMemory;

 Tcl_DumpActiveMemory: proc (x)
                       returns (fixed bin (31) optional byvalue)
                       options(inline) internal;
   dcl x char (*) varz byaddr nonasgn;
   return (0);
 end Tcl_DumpActiveMemory;

 Tcl_ValidateAllMemory: proc (x,y)
                        options(inline) internal;
   dcl x char (*) varz byaddr nonasgn;
   dcl y fixed bin(31) byvalue;
 end Tcl_ValidateAllMemory;

 %end; /* TCL_MEM_DEBUG */

 /* Structure definition for an entry in a hash table.                */
 /* No-one outside Tcl should access any of these fields directly,    */
 /* use the macros defined below.                                     */

 define structure
   1 Tcl_HashEntry,
    2 nextPtr handle Tcl_HashEntry,/* Pointer to next entry in this   */
                        /* hash bucket, or NULL for end of chain.     */
    2 tablePtr handle Tcl_HashTable,
                        /* Pointer to table containing entry.         */
    2 bucketPtr pointer,/* Pointer to bucket that points to           */
                        /* first entry in this entry's chain:         */
                        /* used for deleting the entry.               */
    2 clientData type ClientData,/* Application stores something here */
                                 /* with Tcl_SetHashValue.            */
    2 key union,
      3 oneWordValue pointer,/* One-word value for key.               */
      3 words (1) fixed bin(31),/* Multiple integer words for         */
                        /* key. The actual size will be as large      */
                        /* as necessary for this table's keys.        */
      3 string char(3) varz;/* String for key. The actual size will   */
                            /* be as large as needed to hold the key. */

 /* Structure definition for a hash table. Must be in tcl.inc so      */
 /* clients can allocate space for these structures, but clients      */
 /* should never access any fields in this structure.                 */

 dcl TCL_SMALL_HASH_TABLE fixed bin(31) value(4);
 define structure
   1 Tcl_HashTable,
    2 buckets pointer,  /* Pointer to bucket array. Each element      */
                        /* points to first entry in bucket's hash     */
                        /* chain, or NULL.                            */
    2 staticBuckets (TCL_SMALL_HASH_TABLE) handle Tcl_HashEntry,
                        /* Bucket array used for small tables         */
                        /* (to avoid mallocs and frees).              */
    2 numBuckets fixed bin(31),/* Total number of buckets allocated.  */
    2 numEntries fixed bin(31),/* Total number of entries present     */
                               /* in table.                           */
    2 rebuildSize fixed bin(31),/* Enlarge table when numEntries gets */
                                /* to be this large.                  */
    2 downShift fixed bin(31),/* Shift count used in hashing          */
                              /* function. Designed to use high-      */
                              /* order bits of randomized keys.       */
    2 mask      fixed bin(31),/* Mask value used in hashing function. */
    2 keyType   fixed bin(31),/* Type of keys used in this table.     */
                              /* It's either TCL_STRING_KEYS,         */
                              /* TCL_ONE_WORD_KEYS, or an integer     */
                              /* giving the number of ints that is    */
                              /* the size of the key.                 */
    2 findProc   entry (handle Tcl_HashTable, /* prefer byaddr */
                        pointer) /* type as per keyType */
                 returns (handle Tcl_HashEntry byvalue optional)
                 options (byvalue nodescriptor linkage(cdecl)) limited,
    2 createProc entry (handle Tcl_HashTable, /* prefer byaddr */
                        pointer, /* type as per keyType */
                        fixed bin (31) byaddr)
                 returns (handle Tcl_HashEntry byvalue optional)
                 options (byvalue nodescriptor linkage(cdecl)) limited;

 /* Structure definition for information used to keep track of        */
 /* searches through hash tables:                                     */

 define structure
   1 Tcl_HashSearch,
    2 tablePtr handle Tcl_HashTable, /* Table being searched.         */
    2 nextIndex fixed bin(31),       /* Index of next bucket to be    */
                                     /* enumerated after present one. */
    2 nextEntryPtr handle Tcl_HashEntry;/* Next entry to be enumerated*/
                                     /* in the the current bucket.    */

 /* Acceptable key types for hash tables: */

 dcl TCL_STRING_KEYS   fixed bin(31) value(0);
 dcl TCL_ONE_WORD_KEYS fixed bin(31) value(1);

 /* Routines for clients to use to access fields of hash entries: */

 Tcl_GetHashValue: proc (h) returns (pointer)
                   options (inline) internal;
   dcl h type Tcl_HashEntry byaddr;
   return (h.clientData);
 end Tcl_GetHashValue;

 Tcl_SetHashValue: proc (h, value)
                   options (inline) internal;
   dcl h     type Tcl_HashEntry byaddr,
       value type ClientData byvalue;
   h.clientData = value;
 end Tcl_SetHashValue;

 Tcl_GetHashKey: proc (tablePtr, h) returns (pointer)
                 options (inline) internal;
   dcl tablePtr type Tcl_HashTable byaddr,
       h        type Tcl_HashEntry byaddr;
   if tablePtr.keyType = TCL_ONE_WORD_KEYS then
     return (h.key.oneWordValue);
   else
     return (addr (h.key.string));
 end Tcl_GetHashKey;

 /* Routines for clients to use to invoke find and create */
 /* procedures for hash tables:                           */

 Tcl_FindHashEntry: proc (tablePtr, key)
                    returns (handle Tcl_HashEntry)
                    options (inline) internal;
   dcl tablePtr type Tcl_HashTable byaddr,
       key      pointer byvalue;
   return (tablePtr.findProc (handle (tablePtr), key ));
 end Tcl_FindHashEntry;

 Tcl_CreateHashEntry: proc (tablePtr, key, newPtr)
                      returns (handle Tcl_HashEntry)
                      options (inline) internal;
   dcl tablePtr type Tcl_HashTable byaddr,
       key      pointer byvalue,
       newPtr   fixed bin (31) byvalue;
   return (tablePtr.createProc (handle (tablePtr), key, newPtr));
 end Tcl_CreateHashEntry;

 /* Flag values to pass to Tcl_DoOneEvent to disable searches         */
 /* for some kinds of events:                                         */

 dcl TCL_DONT_WAIT     fixed bin (31) value (raise2(1,1));
 dcl TCL_WINDOW_EVENTS fixed bin (31) value (raise2(1,2));
 dcl TCL_FILE_EVENTS   fixed bin (31) value (raise2(1,3));
 dcl TCL_TIMER_EVENTS  fixed bin (31) value (raise2(1,4));
 dcl TCL_IDLE_EVENTS   fixed bin (31) value (raise2(1,5));
 dcl TCL_ALL_EVENTS    fixed bin (31) value (inot(TCL_DONT_WAIT));

 /* The following structure defines a generic event for the Tcl event */
 /* system. These are the things that are queued in calls to          */
 /* Tcl_Queue Event and serviced later by Tcl_DoOneEvent. There can   */
 /* be many different kinds of events with different fields,          */
 /* corresponding to window events, timer events, etc.  The structure */
 /* for a particular event consists of a Tcl_Event header followed by */
 /* additional information specific to that event.                    */

 define structure
   1 Tcl_Event,
    2 proc    type Tcl_EventProc,
              /* Procedure to call to service this event. */
    2 nextPtr handle Tcl_Event;
              /* Next in list of pending events, or NULL. */


 /* Positions to pass to Tcl_QueueEvent: */

 define ordinal Tcl_QueuePosition (TCL_QUEUE_TAIL,
                                   TCL_QUEUE_HEAD,
                                   TCL_QUEUE_MARK) prec(31);

 /* Values to pass to Tcl_SetServiceMode to specify the behavior of   */
 /* notifier event routines.                                          */

 dcl TCL_SERVICE_NONE fixed bin(31) value(0);
 dcl TCL_SERVICE_ALL  fixed bin(31) value(1);

 /* The following structure keeps is used to hold a time value,       */
 /* either as an absolute time (the number of seconds from the epoch) */
 /* or as an elapsed time.                                            */
 /* On Unix systems the epoch is Midnight Jan 1, 1970 GMT.            */
 /* On Macintosh systems the epoch is Midnight Jan 1, 1904 GMT.       */

 define structure
   1 Tcl_Time,
    2 sec  fixed bin(31),/* Seconds. */
    2 usec fixed bin(31);/* Microseconds. */

 define alias Tcl_SetTimerProc
        entry (type Tcl_Time byaddr)
        options (byvalue nodescriptor linkage(cdecl)) limited;
 define alias Tcl_WaitForEventProc
        entry (type Tcl_Time byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 /* Bits to pass to Tcl_CreateFileHandler and Tcl_CreateChannelHandler*/
 /* to indicate what sorts of events are of interest:                 */

 dcl TCL_READABLE  fixed bin (31) value (raise2(1,1));
 dcl TCL_WRITABLE  fixed bin (31) value (raise2(1,2));
 dcl TCL_EXCEPTION fixed bin (31) value (raise2(1,3));

 /* Flag values to pass to Tcl_OpenCommandChannel to indicate the     */
 /* disposition of the stdio handles. TCL_STDIN, TCL_STDOUT,          */
 /* TCL_STDERR, are also used in Tcl_GetStdChannel.                   */

 dcl TCL_STDIN        fixed bin (31) value (raise2(1,1));
 dcl TCL_STDOUT       fixed bin (31) value (raise2(1,2));
 dcl TCL_STDERR       fixed bin (31) value (raise2(1,3));
 dcl TCL_ENFORCE_MODE fixed bin (31) value (raise2(1,4));

 /* Bits passed to Tcl_DriverClose2Proc to indicate which side of a   */
 /* channel should be closed.                                         */

 dcl TCL_CLOSE_READ   fixed bin (31) value (raise2(1,1));
 dcl TCL_CLOSE_WRITE  fixed bin (31) value (raise2(1,2));

 /* Value to use as the closeProc for a channel that supports the     */
 /* close2Proc interface. (For explanation of PL/I coding see above.) */

 $Tcl_CloseProc_Constant: proc (p) returns (type Tcl_CloseProc)
                          options (inline) internal;
   dcl p pointer byvalue;
   dcl e type Tcl_CloseProc;
   dcl entryaddr builtin;
   entryaddr (e) = p;
   return (e);
 end $Tcl_CloseProc_Constant;

 %dcl TCL_CLOSE2PROC char ext;
 %TCL_CLOSE2PROC = '$Tcl_CloseProc_Constant (ptrvalue (1))';

 /* Channel version tag.  This was introduced in 8.3.2/8.4. */

 dcl TCL_CHANNEL_VERSION_1 type Tcl_ChannelTypeVersion
                           value (pointervalue ('01'xn));
 dcl TCL_CHANNEL_VERSION_2 type Tcl_ChannelTypeVersion
                           value (pointervalue ('02'xn));

 /* Defines for the various operations in a channel type: */

 define alias Tcl_DriverBlockModeProc
        entry (type ClientData,
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverCloseProc
        entry (type ClientData,
               handle Tcl_Interp)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverClose2Proc
        entry (type ClientData,
               handle Tcl_Interp,
               fixed bin(31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverInputProc
        entry (type ClientData,
               char (*) varz byaddr,
               fixed bin (31),
               fixed bin (31) byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverOutputProc
        entry (type ClientData,
               char (*) varz byaddr,
               fixed bin (31),
               fixed bin (31) byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverSeekProc
        entry (type ClientData,
               fixed bin (31),
               fixed bin (31),
               fixed bin (31) byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverSetOptionProc
        entry (type ClientData,
               handle Tcl_Interp,
               char (*) varz byaddr,
               char (*) varz byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverGetOptionProc
        entry (type ClientData,
               handle Tcl_Interp,
               char (*) varz byaddr,
               type Tcl_Dstring byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverWatchProc
        entry (type ClientData,
               fixed bin (31))
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverGetHandleProc
        entry (type ClientData,
               fixed bin (31),
               type ClientData byaddr)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverFlushProc
        entry (type ClientData)
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 define alias Tcl_DriverHandlerProc
        entry (type ClientData,
               fixed bin (31))
        returns (fixed bin (31) byvalue optional)
        options (byvalue nodescriptor linkage(cdecl)) limited;

 /* Enum for different end of line translation and recognition modes. */

 define ordinal Tcl_EolTranslation
     (TCL_TRANSLATE_AUTO,  /* Eol == \r), \n and \r\n. */
      TCL_TRANSLATE_CR,    /* Eol == \r.               */
      TCL_TRANSLATE_LF,    /* Eol == \n.               */
      TCL_TRANSLATE_CRLF)  /* Eol == \r\n.             */
     prec(31);

 /* Tcl_ChannelType:                                                  */
 /*                                                                   */
 /* One such structure exists for each type (kind) of channel.        */
 /* It collects together in one place all the functions that are      */
 /* part of the specific channel type.                                */

 define structure
   1 Tcl_ChannelType,
    2 typeName      pointer, /* The name of the channel type in Tcl   */
                             /* commands This storage is owned by     */
                             /* channel type.                         */
    2 version       type Tcl_ChannelTypeVersion,
                             /* Version of the channel type. */
    2 closeProc     type Tcl_DriverCloseProc,
      /* Procedure to call to close the channel. */
    2 inputProc     type Tcl_DriverInputProc,
      /* Procedure to call for input on channel. */
    2 outputProc    type Tcl_DriverOutputProc,
      /* Procedure to call for output on channel. */
    2 seekProc      type Tcl_DriverSeekProc,
      /* Procedure to call to seek on the channel. May be NULL. */
    2 setOptionProc type Tcl_DriverSetOptionProc,
      /* Set an option on a channel. */
    2 getOptionProc type Tcl_DriverGetOptionProc,
      /* Get an option from a channel. */
    2 watchProc     type Tcl_DriverWatchProc,
      /* Set up the notifier to watch for events on this channel. */
    2 getHandleProc type Tcl_DriverGetHandleProc,
      /* Get an OS handle from the channel or NULL if not supported. */
    2 close2Proc    type Tcl_DriverClose2Proc,
      /* Procedure to call to close the channel if the device   */
      /* supports closing the read & write sides independently. */
    2 blockModeProc type Tcl_DriverBlockModeProc,
      /* Set blocking mode for the raw channel. May be NULL. */
    /* Only valid in TCL_CHANNEL_VERSION_2 channels */
    2 flushProc     type Tcl_DriverFlushProc,
      /* Procedure to call to flush a channel. May be NULL. */
    2 handlerProc   type Tcl_DriverHandlerProc;
      /* Procedure to call to handle a channel event.      */
      /* This will be passed up the stacked channel chain. */

 /* The following flags determine whether the blockModeProc above  */
 /* should set the channel into blocking or nonblocking mode.      */
 /* They are passed as arguments to the blockModeProc procedure in */
 /* the above structure.                                           */

 dcl TCL_MODE_BLOCKING    fixed bin(31) value(0);
     /* Put channel into blocking mode. */
 dcl TCL_MODE_NONBLOCKING fixed bin(31) value(1);
     /* Put channel into nonblocking mode. */

 /* Enum for different types of file paths. */

 define ordinal Tcl_PathType (TCL_PATH_ABSOLUTE,
                              TCL_PATH_RELATIVE,
                              TCL_PATH_VOLUME_RELATIVE) prec(31);

 /* The following structure represents the Notifier functions that */
 /* you can override with the Tcl_SetNotifier call.                */

 define structure
   1 Tcl_NotifierProcs,
     3 setTimerProc          type Tcl_SetTimerProc,
     3 waitForEventProc      type Tcl_WaitForEventProc,
     3 createFileHandlerProc type Tcl_CreateFileHandlerProc,
     3 deleteFileHandlerProc type Tcl_DeleteFileHandlerProc;

 /* The following structure represents a user-defined encoding.  */
 /* It collects together all the functions that are used by the  */
 /* specific encoding.                                           */

 define structure
   1 Tcl_EncodingType,
     2 encodingName pointer, /* The name of the encoding,          */
                             /* e.g. "euc-jp". This name is the    */
                             /* unique key for this encoding type. */
     2 toUtfProc    type Tcl_EncodingConvertProc,
       /* Procedure to convert from external encoding into UTF-8.  */
     2 fromUtfProc  type Tcl_EncodingConvertProc,
       /* Procedure to convert from UTF-8 into external encoding.  */
     2 freeProc     type Tcl_EncodingFreeProc,
       /* If non-NULL, proc to call when this encoding is deleted. */
     2 clientData   type ClientData,
       /* Arbitrary value associated with encoding type.           */
       /*  Passed to conversion procedures.                        */
     2 nullSize     fixed bin (31);
       /* Number of zero bytes that signify end-of-string in this  */
       /* encoding. This number is used to determine the source    */
       /* string length when the srcLen argument is negative.      */
       /* Must be 1 or 2.                                          */

 /* The following definitions are used as values for the conversion   */
 /* control flags argument when converting text from one character    */
 /* set to another:                                                   */
 /*                                                                   */
 /* TCL_ENCODING_START: Signifies that the source buffer is the first */
 /*          block in a (potentially multi-block) input               */
 /*          stream.  Tells the conversion procedure to               */
 /*          reset to an initial state and perform any                */
 /*          initialization that needs to occur before the            */
 /*          first byte is converted.  If the source                  */
 /*          buffer contains the entire input stream to be            */
 /*          converted, this flag should be set.                      */
 /*                                                                   */
 /* TCL_ENCODING_END:    Signifies that the source buffer is the last */
 /*          block in a (potentially multi-block) input               */
 /*          stream.  Tells the conversion routine to                 */
 /*          perform any finalization that needs to occur             */
 /*          after the last byte is converted and then to             */
 /*          reset to an initial state.  If the source                */
 /*          buffer contains the entire input stream to be            */
 /*          converted, this flag should be set.                      */
 /*                                                                   */
 /* TCL_ENCODING_STOPONERROR:  If set, then the converter will return */
 /*          immediately upon encountering an invalid                 */
 /*          byte sequence or a source character that has             */
 /*          no mapping in the target encoding.  If clear,            */
 /*          then the converter will skip the problem,                */
 /*          substituting one or more "close" characters              */
 /*          in the destination buffer and then continue              */
 /*          to sonvert the source.                                   */


 dcl TCL_ENCODING_START       fixed bin (31) value('01'xn);
 dcl TCL_ENCODING_END         fixed bin (31) value('02'xn);
 dcl TCL_ENCODING_STOPONERROR fixed bin (31) value('04'xn);


 /*----------------------------------------------------------------   */
 /* The following data structures and declarations are for the new    */
 /* Tcl parser. This stuff should all move to tcl.cpy eventually.     */
 /*----------------------------------------------------------------   */


 /* For each word of a command, and for each piece of a word such as a*/
 /* variable reference, one of the following structures is created to */
 /* describe the token.                                               */


 define structure
   1 Tcl_Token,
     2 type  fixed bin(31), /* Type of token, such as TCL_TOKEN_WORD, */
                            /* see below for valid types.             */
     2 start pointer,       /* First character in token.              */
     2 size  fixed bin(31), /* Number of bytes in token.              */
     2 numComponents fixed bin(31);
       /* If this token is composed of other tokens, this field tells */
       /* how many of them there are (including components of         */
       /* components, etc.). The component tokens immediately follow  */
       /* this one.                                                   */

 /* Type values defined for Tcl_Token structures.  These values are   */
 /* defined as mask bits so that it's easy to check for collections   */
 /* of types.                                                         */
 /*                                                                   */
 /* TCL_TOKEN_WORD -     The token describes one word of a command,   */
 /*          from the first non-blank character of                    */
 /*          the word (which may be " or {) up to but                 */
 /*          not including the space, semicolon, or                   */
 /*          bracket that terminates the word.                        */
 /*          NumComponents counts the total number of                 */
 /*          sub-tokens that make up the word.  This                  */
 /*          includes, for example, sub-tokens of                     */
 /*          TCL_TOKEN_VARIABLE tokens.                               */
 /* TCL_TOKEN_SIMPLE_WORD - This token is just like TCL_TOKEN_WORD    */
 /*          except that the word is guaranteed to                    */
 /*          consist of a single TCL_TOKEN_TEXT                       */
 /*          sub-token.                                               */
 /* TCL_TOKEN_TEXT -     The token describes a range of literal       */
 /*          text that is part of a word.                             */
 /*          NumComponents is always 0.                               */
 /* TCL_TOKEN_BS -    The token describes a backslash sequence        */
 /*          that must be collapsed.  NumComponents                   */
 /*          is always 0.                                             */
 /* TCL_TOKEN_COMMAND -     The token describes a command whose result*/
 /*          must be substituted into the word.  The                  */
 /*          token includes the enclosing brackets.                   */
 /*          NumComponents is always 0.                               */
 /* TCL_TOKEN_VARIABLE -    The token describes a variable            */
 /*          substitution, including the dollar sign,                 */
 /*          variable name, and array index (if there                 */
 /*          is one) up through the right                             */
 /*          parentheses.  NumComponents tells how                    */
 /*          many additional tokens follow to                         */
 /*          represent the variable name.  The first                  */
 /*          token will be a TCL_TOKEN_TEXT token                     */
 /*          that describes the variable name.  If                    */
 /*          the variable is an array reference then                  */
 /*          there will be one or more additional                     */
 /*          tokens, of type TCL_TOKEN_TEXT,                          */
 /*          TCL_TOKEN_BS, TCL_TOKEN_COMMAND, and                     */
 /*          TCL_TOKEN_VARIABLE, that describe the                    */
 /*          array index; numComponents counts the                    */
 /*          total number of nested tokens that make                  */
 /*          up the variable reference, including                     */
 /*          sub-tokens of TCL_TOKEN_VARIABLE tokens.                 */
 /* TCL_TOKEN_SUB_EXPR -    The token describes one subexpression of  */
 /*          an expression, from the first non-blank                  */
 /*          character of the subexpression up to but not             */
 /*          including the space, brace, or bracket                   */
 /*          that terminates the subexpression.                       */
 /*          NumComponents counts the total number of                 */
 /*          following subtokens that make up the                     */
 /*          subexpression; this includes all subtokens               */
 /*          for any nested TCL_TOKEN_SUB_EXPR tokens.                */
 /*          For example, a numeric value used as a                   */
 /*          primitive operand is described by a                      */
 /*          TCL_TOKEN_SUB_EXPR token followed by a                   */
 /*          TCL_TOKEN_TEXT token. A binary subexpression             */
 /*          is described by a TCL_TOKEN_SUB_EXPR token               */
 /*          followed by the   TCL_TOKEN_OPERATOR token               */
 /*          for the operator, then TCL_TOKEN_SUB_EXPR                */
 /*          tokens for the left then the right operands.             */
 /* TCL_TOKEN_OPERATOR -  The token describes one expression operator.*/
 /*          An operator might be the name of a math                  */
 /*          function such as "abs". A TCL_TOKEN_OPERATOR             */
 /*          token is always preceeded by one                         */
 /*          TCL_TOKEN_SUB_EXPR token for the operator's              */
 /*          subexpression, and is followed by zero or                */
 /*          more TCL_TOKEN_SUB_EXPR tokens for the                   */
 /*          operator's operands. NumComponents is                    */
 /*          always 0.                                                */


 dcl TCL_TOKEN_WORD        fixed bin (31) value(1);
 dcl TCL_TOKEN_SIMPLE_WORD fixed bin (31) value(2);
 dcl TCL_TOKEN_TEXT        fixed bin (31) value(4);
 dcl TCL_TOKEN_BS          fixed bin (31) value(8);
 dcl TCL_TOKEN_COMMAND     fixed bin (31) value(16);
 dcl TCL_TOKEN_VARIABLE    fixed bin (31) value(32);
 dcl TCL_TOKEN_SUB_EXPR    fixed bin (31) value(64);
 dcl TCL_TOKEN_OPERATOR    fixed bin (31) value(128);


 /* Parsing error types.  On any parsing error, one of these values  */
 /* will be stored in the error field of the Tcl_Parse structure     */
 /* defined below.                                                   */


 dcl TCL_PARSE_SUCCESS            fixed bin (31) value(0);
 dcl TCL_PARSE_QUOTE_EXTRA        fixed bin (31) value(1);
 dcl TCL_PARSE_BRACE_EXTRA        fixed bin (31) value(2);
 dcl TCL_PARSE_MISSING_BRACE      fixed bin (31) value(3);
 dcl TCL_PARSE_MISSING_BRACKET    fixed bin (31) value(4);
 dcl TCL_PARSE_MISSING_PAREN      fixed bin (31) value(5);
 dcl TCL_PARSE_MISSING_QUOTE      fixed bin (31) value(6);
 dcl TCL_PARSE_MISSING_VAR_BRACE  fixed bin (31) value(7);
 dcl TCL_PARSE_SYNTAX             fixed bin (31) value(8);
 dcl TCL_PARSE_BAD_NUMBER         fixed bin (31) value(9);


 /* A structure of the following type is filled in by Tcl_ParseCommand*/
 /* and describes a single command parsed from an input string.       */


 dcl NUM_STATIC_TOKENS fixed bin (31) value(20);

 define structure
   1 Tcl_Parse,
     2 commentStart pointer,
       /* Pointer to # that begins the first of                       */
       /* one or more comments preceding the command.                 */
     2 commentSize  fixed bin(31),
       /* Number of bytes in comments (up through newline character   */
       /* that terminates the last comment). If there were no         */
       /* comments, this field is 0.                                  */
     2 commandStart pointer,
       /* First character in first word of command.                   */
     2 commandSize  fixed bin(31),
       /* Number of bytes in command, including first character of    */
       /* first word, up through the terminating newline, close       */
       /* bracket, or semicolon.                                      */
     2 numWords     fixed bin(31),
       /* Total number of words in command.  May be 0.                */
     2 tokenPtr     handle Tcl_Token,
       /* Pointer to first token representing the words of the        */
       /* command. Initially points to staticTokens, but may change   */
       /* to point to malloc-ed space if command exceeds space in     */
       /* staticTokens.                                               */
     2 numTokens    fixed bin(31),
       /* Total number of tokens in command.                          */
     2 tokensAvailable fixed bin(31),
       /* Total number of tokens available at *tokenPtr.              */
     2 errorType    fixed bin(31),
       /* One of the parsing error types defined above.               */

     /* The fields below are intended only for the private use of the */
     /* parser.  They should not be used by procedures that invoke    */
     /* Tcl_ParseCommand.                                             */

     2 string       pointer,
       /* The original command string passed to Tcl_ParseCommand.     */
     2 end          pointer,
       /* Points to the character just after the last one in the      */
       /* command string.                                             */
     2 interp       handle Tcl_Interp,
       /* Interpreter to use for error reporting, or NULL. */
     2 term         pointer,
       /* Points to character in string that terminated most recent   */
       /* token. Filled in by ParseTokens. If an error occurs, points */
       /* to beginning of region where the error occurred (e.g. the   */
       /*  open brace if the close brace is missing).                 */
     2 incomplete   fixed bin(31),
       /* This field is set to 1 by Tcl_ParseCommand if the command   */
       /* appears to be incomplete.                                   */
       /* This information is used by Tcl_CommandComplete.            */
     2 staticTokens (NUM_STATIC_TOKENS) type Tcl_Token;
       /* Initial space for tokens for command.                       */
       /* This space should be large enough to accommodate most       */
       /* commands, dynamic space is allocated for very large         */
       /* commands that don't fit here.                               */


 /* The following definitions are the error codes returned by the     */
 /* conversion routines:                                              */
 /*                                                                   */
 /* TCL_OK:         All characters were converted.                    */
 /*                                                                   */
 /* TCL_CONVERT_NOSPACE:     The output buffer would not have been    */
 /*           large enough for all of the converted data, as many     */
 /*           characters as could fit were converted though.          */
 /*                                                                   */
 /* TCL_CONVERT_MULTIBYTE:   The last few bytes in the source string  */
 /*           were the beginning of a multibyte sequence, but         */
 /*           more bytes were needed to complete this                 */
 /*           sequence.  A subsequent call to the conversion          */
 /*           routine should pass the beginning of this               */
 /*           unconverted sequence plus additional bytes              */
 /*           from the source stream to properly convert              */
 /*           the formerly split-up multibyte sequence.               */
 /*                                                                   */
 /* TCL_CONVERT_SYNTAX:      The source stream contained an invalid   */
 /*           character sequence.  This may occur if the              */
 /*           input stream has been damaged or if the input           */
 /*           encoding method was misidentified.  This error          */
 /*           is reported only if TCL_ENCODING_STOPONERROR            */
 /*           was specified.                                          */
 /*                                                                   */
 /* TCL_CONVERT_UNKNOWN:     The source string contained a character  */
 /*           that could not be represented in the target             */
 /*           encoding.  This error is reported only if               */
 /*           TCL_ENCODING_STOPONERROR was specified.                 */

 dcl TCL_CONVERT_MULTIBYTE  fixed bin (31) value(-1);
 dcl TCL_CONVERT_SYNTAX     fixed bin (31) value(-2);
 dcl TCL_CONVERT_UNKNOWN    fixed bin (31) value(-3);
 dcl TCL_CONVERT_NOSPACE    fixed bin (31) value(-4);

 /* The maximum number of bytes that are necessary to represent a */
 /* single Unicode character in UTF-8.                            */

 dcl TCL_UTF_MAX      fixed bin (31) value(3);

 /* This represents a Unicode character. */

 %dcl Tcl_UniChar char ext;
 %Tcl_UniChar = 'WCHAR';

 /* Deprecated Tcl procedures: */
 %if TCL_NO_DEPRECATED ^= '' %then %do;

 Tcl_EvalObj: proc (interp, objPtr)
              returns (fixed bin(31) byvalue optional)
              options(inline) internal;
   dcl interp handle Tcl_Interp,
       objPtr handle Tcl_Obj;
   return (Tcl_EvalObjEx (interp, objPtr, 0));
 end Tcl_EvalObj;

 Tcl_GlobalEvalObj: proc (interp,objPtr)
                    returns (fixed bin(31) byvalue optional)
                    options(inline) internal;
   dcl interp handle Tcl_Interp,
       objPtr handle Tcl_Obj;
   return (Tcl_EvalObjEx (interp, objPtr, TCL_EVAL_GLOBAL));
 end Tcl_GlobalEvalObj;
 %end;

 %dcl Tcl_Ckalloc    char ext; %Tcl_Ckalloc   = 'Tcl_Alloc';
 %dcl Tcl_Ckfree     char ext; %Tcl_Ckfree    = 'Tcl_Free';
 %dcl Tcl_Ckrealloc  char ext; %Tcl_Ckrealloc = 'Tcl_Realloc';
 %dcl Tcl_Return     char ext; %Tcl_Return    = 'Tcl_SetResult';
 %dcl Tcl_TildeSubst char ext; %Tcl_TildeSubst= 'Tcl_TranslateFileName';
 %dcl panic          char ext; %panic         = 'Tcl_Panic';
 %dcl panicVA        char ext; %panicVA       = 'Tcl_PanicVA';

 /* The following constant is used to test for older versions of Tcl  */
 /* in the stubs tables.                                              */
 /*                                                                   */
 /* Jan Nijtman's plus patch uses 0xFCA1BACF, so we need to pick a    */
 /* different value since the stubs tables don't match.               */

 dcl TCL_STUB_MAGIC fixed bin(31) value('FCA3BACF'xn);

 %dcl USE_TCL_STUBS char ext;
 %if USE_TCL_STUBS ^= '' %then %do;
 /* The following function is required to be defined in all stubs     */
 /* aware extensions. The function is actually implemented in the     */
 /* stub library, not the main Tcl library, although there is a       */
 /* trivial implementation in the main library in case an extension   */
 /* is statically linked into an application.                         */

 dcl Tcl_InitStubs
     entry (handle Tcl_Interp,
            char (*) varz byaddr,
            fixed bin (31))
     returns (pointer byvalue optional)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tcl_InitStubs');
 %end; %else %do;
 /* When not using stubs, make it a macro. */

 Tcl_InitStubs: proc (interp, version, exact)
                returns (pointer byvalue optional)
                options (inline) internal;
   dcl interp  handle Tcl_Interp byvalue,
       version char (*) varz byaddr,
       exact   fixed bin (31) byvalue;
   return (Tcl_PkgRequire (interp, 'Tcl', version, exact));
 end Tcl_InitStubs;
 %end;


 /* Include the public function declarations that are accessible */
 /* via the stubs table.                                         */

 %include tclDecls;

 /* Public functions that are not accessible via the stubs table. */

 dcl Tcl_Main
     entry (fixed bin(31),
            (*) pointer byaddr, /* each to char (*) varz */
            type Tcl_AppInitProc)
     options (byvalue nodescriptor linkage(cdecl))
     external('Tcl_Main');


 %if TCL_MEM_DEBUG ^= '' %then %do;
 %activate Tcl_Alloc;
 %activate Tcl_Free;
 %activate Tcl_Realloc;
 %end;

 %end; /* #TCL */
